/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "appconfig.h"
#include <commondefs.h>

#include <qdir.h>

#include <ksimpleconfig.h>
#include <kconfigbase.h>
#include <kdebug.h>

using namespace SubtitleComposer;

AppConfig::AppConfig():
	QObject()
{
}

AppConfig::AppConfig( const AppConfig& config ):
	QObject()
{
	for ( QMapConstIterator<QString,AppConfigGroup*> it = config.m_groups.begin(); it != config.m_groups.end(); ++it )
		setConfigGroup( *it.data() );
}

AppConfig& AppConfig::operator=( const AppConfig& config )
{
	if ( this == &config )
		return *this;

	// remove the now invalid groups (those that are not in config.m_groups)
	QStringList obsoletedKeys; // don't delete the items while iterating the map
	for ( QMapConstIterator<QString,AppConfigGroup*> it = m_groups.begin(); it != m_groups.end(); ++it )
	{
		if ( ! config.m_groups.contains( it.key() ) )
		{
			delete it.data();
			obsoletedKeys << it.key();
		}
	}

	// safe to delete the items now
	for ( QStringList::Iterator it = obsoletedKeys.begin(); it != obsoletedKeys.end(); ++it )
		m_groups.remove( *it );


	for ( QMapConstIterator<QString,AppConfigGroup*> it = config.m_groups.begin(); it != config.m_groups.end(); ++it )
		setConfigGroup( *it.data() );

	return *this;
}

AppConfig::~AppConfig()
{
	for ( QMapIterator<QString,AppConfigGroup*> it = m_groups.begin(); it != m_groups.end(); ++it )
		delete it.data();
}

bool AppConfig::isCompatibleWith( const AppConfig& config )
{
	for ( QMapConstIterator<QString,AppConfigGroup*> it = m_groups.begin(); it != m_groups.end(); ++it )
	{
		if ( ! config.m_groups.contains( it.key() ) )
			return false;
		if ( ! it.data()->isCompatibleWith( *config.m_groups[it.key()] ) )
			return false;
	}

	return true;
}

void AppConfig::loadDefaults()
{
	for ( QMapIterator<QString,AppConfigGroup*> it = m_groups.begin(); it != m_groups.end(); ++it )
		it.data()->loadDefaults();
}

bool AppConfig::load( const QString& path )
{
	if ( ! System::isReadable( path ) )
		return false;

	QDir absPath( path );
	absPath.convertToAbs();
	KSimpleConfig config( absPath.path(), true );

	for ( QMapIterator<QString,AppConfigGroup*> it = m_groups.begin(); it != m_groups.end(); ++it )
		it.data()->readFrom( &config );

	return true;
}

bool AppConfig::save( const QString& path ) const
{

	if ( QFile::exists( path ) && ! System::isWritable( path ) )
		return false;

	QDir absPath( path );
	absPath.convertToAbs();
	System::remove( absPath.path() );
	KSimpleConfig config( absPath.path() );

	for ( QMapConstIterator<QString,AppConfigGroup*> it = m_groups.begin(); it != m_groups.end(); ++it )
		it.data()->writeTo( &config );

	return true;
}


void AppConfig::readFrom( KSimpleConfig* config )
{
	QStringList groups( config->groupList() );
	for ( QStringList::Iterator it = groups.begin(); it != groups.end(); ++it )
	{
		if ( ! m_groups.contains( *it ) )
			continue;

		m_groups[*it]->readFrom( config );
	}
}

void AppConfig::writeTo( KSimpleConfig* config ) const
{
	for ( QMapConstIterator<QString,AppConfigGroup*> it = m_groups.begin(); it != m_groups.end(); ++it )
		it.data()->writeTo( config );
}

AppConfigGroup* AppConfig::configGroup( const QString& groupName )
{
	return m_groups.contains( groupName ) ? m_groups[groupName] : 0;
}

const AppConfigGroup* const AppConfig::configGroup( const QString& groupName ) const
{
	return m_groups.contains( groupName ) ? m_groups[groupName] : 0;
}

void AppConfig::setConfigGroup( const AppConfigGroup& configGroup )
{
	if ( m_groups.contains( configGroup.name() ) )
	{
		if ( m_groups[configGroup.name()]->isCompatibleWith( configGroup ) )
		{
			*m_groups[configGroup.name()] = configGroup;
			return;
		}
		else
		{
			kdWarning() << "attempted to replace group " + configGroup.name() + " with incompatible settings" << endl;
			return;
		}
	}

	m_groups[configGroup.name()] = new AppConfigGroup( configGroup );
	m_groups[configGroup.name()]->m_config = this;

	connect( m_groups[configGroup.name()], SIGNAL(optionChanged(const QString&,const QString&,const QString&)),
			 this, SIGNAL(optionChanged(const QString&,const QString&,const QString&)) );
}

void AppConfig::delConfigGroup( const QString& groupName )
{
	if ( m_groups.contains( groupName ) )
	{
		disconnect( m_groups[groupName], SIGNAL(optionChanged(const QString&,const QString&,const QString&)),
					this, SIGNAL(optionChanged(const QString&,const QString&,const QString&)) );

		delete m_groups[groupName];
		m_groups.remove( groupName );
	}
}
