#ifndef SUBTITLEACTIONS_H
#define SUBTITLEACTIONS_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qptrlist.h>
#include <qstring.h>

namespace SubtitleComposer
{
	class Action
	{
		typedef enum { Executed = 0x1, Done = 0x2 } State;

		friend class ActionsManager;
		friend class CompositeAction;

		public:

			Action( const QString& description=QString::null ):
				m_description( description ),
				m_state( 0 ) {}
			virtual ~Action() {}

			inline const QString& description() const
			{
				return m_description;
			};

			/// true if action has been executed at least once
			inline bool executed()
			{
				return m_state & Executed;
			}

			/// true if action can be undone
			inline bool done()
			{
				return m_state & Done;
			}

			void redo()
			{
				_redo( true );
			}

			void undo()
			{
				_undo( true );
			}

		protected:

			template <class T>
			T* tryCastTo( Action* action )
			{
				try
				{
					return dynamic_cast<T*>( action );
				}
				catch( const std::bad_cast& )
				{
					return 0;
				}
			}

			/// when this method is called both the callee and the action parameter have been executed
			/// so, if the callee can be merged with the previous action, it must recover the state
			/// previous to execution of both actions and return true (if the actions can't be merged,
			/// just return false).
			inline virtual bool mergeWithPrevious( Action* /*action*/ )
			{
				return false;
			}

			void _redo( bool emitSignals )
			{
				if ( ! (m_state & Done) )
				{
					_preRedo();
					_redo();
					if ( emitSignals )
						_emitRedoSignals();
					m_state |= Executed|Done;
				}
			}

			void _undo( bool emitSignals )
			{
				if ( m_state & Done )
				{
					_preUndo();
					_undo();
					if ( emitSignals )
						_emitUndoSignals();
					m_state &= ~Done;
				}
			}

			virtual void _redo() = 0;
			virtual void _undo() = 0;

			virtual void _preRedo() {};
			virtual void _preUndo() {};

			virtual void _emitRedoSignals() {};
			virtual void _emitUndoSignals() {};

		protected:

			QString m_description;

		private:

			int m_state;
	};

	class CompositeAction : public Action
	{
		public:

			CompositeAction( const QString& description, bool immediateExecution=true, bool delaySignals=true ):
				Action( description ),
				m_actions(),
				m_immediateExecution( immediateExecution ),
				m_delaySignals( delaySignals )
			{
				m_actions.setAutoDelete( true );
			}

			virtual ~CompositeAction() {}

			unsigned count()
			{
				return m_actions.count();
			}

			void appendAction( Action* action )
			{
				if ( m_immediateExecution )
					action->_redo( ! m_delaySignals );

				m_actions.append( action );
			}

		protected:

			virtual void _redo()
			{
				if ( ! executed() && m_immediateExecution )
				{
					Action* previousAction = m_actions.first();
					for ( Action* action = m_actions.next(); action; previousAction = action, action = m_actions.next() )
						if ( action->mergeWithPrevious( previousAction ) )
							m_actions.removeRef( previousAction ); // it keeps/makes action the current item

					return; // the actions were executed for the first time when they were appended
				}

				for ( Action* action = m_actions.first(); action; action = m_actions.next() )
					action->_redo( ! m_delaySignals );

				if ( ! executed() )
				{
					Action* previousAction = m_actions.first();
					for ( Action* action = m_actions.next(); action; previousAction = action, action = m_actions.next() )
						if ( action->mergeWithPrevious( previousAction ) )
							m_actions.removeRef( previousAction ); // it keeps/makes action the current item
				}
			}

			virtual void _undo()
			{
				for ( Action* action = m_actions.last(); action != 0; action = m_actions.prev() )
					action->_undo( ! m_delaySignals );
			}

			virtual void _emitRedoSignals()
			{
				if ( m_delaySignals )
					for ( Action* action = m_actions.first(); action; action = m_actions.next() )
						action->_emitRedoSignals();
			}

			virtual void _emitUndoSignals()
			{
				if ( m_delaySignals )
					for ( Action* action = m_actions.last(); action != 0; action = m_actions.prev() )
						action->_emitUndoSignals();
			}

		protected:

			QPtrList<Action> m_actions;
			bool m_immediateExecution;
			bool m_delaySignals;
	};

}

#endif
