/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actionsmanager.h"

using namespace SubtitleComposer;

ActionsManager::ActionsManager():
	m_undoStack(),
	m_redoStack(),
	m_compressionThreshold( 350 )
{
	m_redoStack.setAutoDelete( true );
	m_undoStack.setAutoDelete( true );

	m_time.start();
}

void ActionsManager::execAndStore( Action* action )
{
	if ( action != 0 )
	{
		action->redo();

		bool compressed = false;
		if ( m_time.restart() < m_compressionThreshold && m_redoStack.isEmpty() )
		{
			Action* previousAction = m_undoStack.first();
			if ( previousAction && action->mergeWithPrevious( previousAction ) )
			{
				m_undoStack.removeFirst();
				compressed = true;
			}
		}

		m_redoStack.clear();
		m_undoStack.prepend( action );

		if ( ! compressed )
			emit actionStored();

		emit actionRedone();
	}
}

bool ActionsManager::hasRedo()
{
	return ! m_redoStack.isEmpty();
}

bool ActionsManager::hasUndo()
{
	return ! m_undoStack.isEmpty();
}

QString ActionsManager::redoDescription()
{
	return m_redoStack.isEmpty() ? QString::null : m_redoStack.first()->description();
}

QString ActionsManager::undoDescription()
{
	return m_undoStack.isEmpty() ? QString::null : m_undoStack.first()->description();
}

void ActionsManager::redo()
{
	Action* action = m_redoStack.take( 0 );
	if ( action == 0 )
		return;

	action->redo();
	m_undoStack.prepend( action );

	emit actionRedone();
}

void ActionsManager::undo()
{
	Action* action = m_undoStack.take( 0 );
	if ( action == 0 )
		return;

	action->undo();
	m_redoStack.prepend( action );

	emit actionUndone();
}

void ActionsManager::clearHistory()
{
	m_undoStack.clear();
	m_redoStack.clear();

	emit historyCleared();
}






ActionsManagerProxy::ActionsManagerProxy():
	m_undoStack(),
	m_redoStack()
{
	m_redoStack.setAutoDelete( false );
	m_undoStack.setAutoDelete( false );

	connect( this, SIGNAL( actionStored() ), this, SIGNAL( stateChanged() ) );
	connect( this, SIGNAL( actionRedone() ), this, SIGNAL( stateChanged() ) );
	connect( this, SIGNAL( actionUndone() ), this, SIGNAL( stateChanged() ) );
	connect( this, SIGNAL( historyCleared() ), this, SIGNAL( stateChanged() ) );
}

bool ActionsManagerProxy::hasRedo()
{
	return ! m_redoStack.isEmpty();
}

bool ActionsManagerProxy::hasUndo()
{
	return ! m_undoStack.isEmpty();
}

QString ActionsManagerProxy::redoDescription()
{
	return m_redoStack.isEmpty() ? QString::null : m_redoStack.first()->redoDescription();
}

QString ActionsManagerProxy::undoDescription()
{
	return m_undoStack.isEmpty() ? QString::null : m_undoStack.first()->undoDescription();
}

void ActionsManagerProxy::registerActionsManager( ActionsManager& actionsManager )
{
	actionsManager.clearHistory();

	connect( &actionsManager, SIGNAL( actionStored() ), this, SLOT( onActionStored()) );
	connect( this, SIGNAL( historyCleared() ), &actionsManager, SLOT( clearHistory() ) );
}

void ActionsManagerProxy::unregisterActionsManager( ActionsManager& actionsManager )
{
	disconnect( &actionsManager, SIGNAL( actionStored() ), this, SLOT( onActionStored()) );
	disconnect( this, SIGNAL( historyCleared() ), &actionsManager, SLOT( clearHistory() ) );

	for ( int idx = m_redoStack.findRef( &actionsManager ); idx >= 0; idx = m_redoStack.findNextRef( &actionsManager ) )
		m_redoStack.remove();

	for ( int idx = m_undoStack.findRef( &actionsManager ); idx >= 0; idx = m_undoStack.findNextRef( &actionsManager ) )
		m_undoStack.remove();
}

void ActionsManagerProxy::redo()
{
	ActionsManager* actionsManager = m_redoStack.take( 0 );
	if ( actionsManager == 0 )
		return;

	actionsManager->redo();
	m_undoStack.prepend( actionsManager );

	emit actionRedone();
}

void ActionsManagerProxy::undo()
{
	ActionsManager* actionsManager = m_undoStack.take( 0 );
	if ( actionsManager == 0 )
		return;

	actionsManager->undo();
	m_redoStack.prepend( actionsManager );

	emit actionUndone();
}

void ActionsManagerProxy::clearHistory()
{
	m_undoStack.clear();
	m_redoStack.clear();

	emit historyCleared();
}

void ActionsManagerProxy::onActionStored()
{
	const ActionsManager* actionsManager = static_cast<const ActionsManager*>( sender() );

	m_redoStack.clear();
	m_undoStack.prepend( actionsManager );

	emit actionStored();
}
