#ifndef RANGES_H
#define RANGES_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <qvaluelist.h>
#include <kdebug.h>

namespace SubtitleComposer
{
	class Range
	{
		public:

			static const unsigned MaxIndex=(unsigned)-1;

			Range( unsigned index ):
				m_start( index ),
				m_end( index )
			{
				m_start = index;
				m_end = index;
			}

			Range( unsigned startIndex, unsigned endIndex ):
				m_start( startIndex ),
				m_end( endIndex )
			{
				if ( m_start > m_end )
				{
					kdWarning() << "Attempted to create invalid range";
					m_start = m_end;
				}
			}

			inline static Range full()
			{
				return Range( 0, MaxIndex );
			}

			inline static Range lower( unsigned index )
			{
				return Range( 0, index );
			}

			inline static Range higher( unsigned index )
			{
				return Range( index, MaxIndex );
			}

			Range& operator=( const Range& range )
			{
				if ( this == &range )
					return *this;

				m_start = range.m_start;
				m_end = range.m_end;

				return *this;
			}

			inline unsigned start() const { return m_start; }
			inline unsigned end() const { return m_end; }

			inline bool contains( unsigned index )
			{
				return index >= m_start && index <= m_end;
			}

			inline bool contains( const Range& range )
			{
				return m_end <= range.m_end && m_start >= range.m_start;;
			}

			inline bool operator==( const Range& range ) const
			{
				return m_start == range.m_start && m_end == range.m_end;
			}

			inline bool operator!=( const Range& range ) const
			{
				return m_start != range.m_start || m_end != range.m_end;
			}

			inline bool operator<( const Range& range ) const
			{
				return m_start < range.m_end;
			}

			inline bool operator<=( const Range& range ) const
			{
				return m_start <= range.m_end;
			}

			inline bool operator>( const Range& range ) const
			{
				return m_end > range.m_start;
			}

			inline bool operator>=( const Range& range ) const
			{
				return m_end >= range.m_start;
			}

		private:

			friend class QValueList<Range>;
			friend class QValueListNode<Range>;
			friend class RangeList;

			Range() {}

			unsigned m_start;
			unsigned m_end;
	};

	class RangeList
	{
		public:

			typedef QValueList<Range>::iterator Iterator;
			typedef QValueList<Range>::const_iterator ConstIterator;

			RangeList()
			{
			}

			RangeList( const Range& range )
			{
				m_ranges.append( range );
			}

			RangeList( const RangeList& ranges ):
				m_ranges( ranges.m_ranges )
			{
			}

			bool isEmpty() const
			{
				return m_ranges.empty();
			}

			unsigned count() const
			{
				return m_ranges.count();
			}

			unsigned firstIndex() const
			{
				return m_ranges.empty() ? (unsigned)-1 : m_ranges.first().start();
			}

			unsigned lastIndex() const
			{
				return m_ranges.empty() ? (unsigned)-1 : m_ranges.last().end();
			}

			void trim( unsigned size )
			{
				if ( size )
					trim( Range::lower( size - 1 ) );
				else
					m_ranges.clear();
			}

			void trim( const Range& range )
			{
				QValueList<Range>::iterator it = m_ranges.begin();
				while ( it != m_ranges.end() && range.m_start > (*it).end() )
					it = m_ranges.erase( it );
				if ( it != m_ranges.end() )
				{
					if ( range.m_start > (*it).start() )
						(*it).m_start = range.m_start;

					it = m_ranges.end();
					for ( --it; it != m_ranges.end() && range.m_end < (*it).start(); --it )
						it = m_ranges.erase( it );
					if ( it != m_ranges.end() && range.m_end < (*it).end() )
						(*it).m_end = range.m_end;
				}
			}

			void operator<<( const Range& range )
			{
				if ( ! m_ranges.count() )
				{
					m_ranges.append( range );
					return;
				}

				QValueList<Range>::iterator lowRangeIt = m_ranges.begin();
				QValueList<Range>::iterator highRangeIt = ++lowRangeIt;
				for( ; highRangeIt != m_ranges.end(); ++lowRangeIt, ++highRangeIt )
				{
					Range& lowRange = (*lowRangeIt);
					Range& highRange = (*highRangeIt);

					// range joins lowRange & highRange
					if ( lowRange.m_end >= range.m_start && range.m_end >= highRange.m_start )
					{
						lowRange.m_end = highRange.m_end;
						m_ranges.remove( highRange );
						return;
					}

					// range is between lowRange & highRange
					else if ( lowRange.m_end < range.m_start && range.m_end < highRange.m_start  )
					{
						m_ranges.insert( highRangeIt, range );
						return;
					}

					// range expands one of lowRange boundaries or is contained by lowRange
					else if ( range.m_start >= lowRange.m_start && range.m_start <= lowRange.m_end ||
							  range.m_end >= lowRange.m_start && range.m_end <= lowRange.m_end
							)
					{
						if ( range.m_start < lowRange.m_start )
							lowRange.m_start = range.m_start;
						if ( range.m_end > lowRange.m_end )
							lowRange.m_end = range.m_end;
						return;
					}

					// range is lower than lowRange
					else if ( range.m_end < lowRange.m_start )
					{
						m_ranges.insert( lowRangeIt, range );
						return;
					}
				}

				Range& lowRange = (*lowRangeIt);

				// range is higher than lowRange
				if ( lowRange.m_end < range.m_start )
					m_ranges.append( range );

				// range expands one of lowRange boundaries or is contained by lowRange
				else if ( range.m_start >= lowRange.m_start && range.m_start <= lowRange.m_end ||
							range.m_end >= lowRange.m_start && range.m_end <= lowRange.m_end
						)
				{
					if ( range.m_start < lowRange.m_start )
						lowRange.m_start = range.m_start;
					if ( range.m_end > lowRange.m_end )
						lowRange.m_end = range.m_end;
				}

				// range is lower than lowRange
				else if ( range.m_end < lowRange.m_start )
					m_ranges.insert( lowRangeIt, range );
			}

			inline Iterator begin() { return m_ranges.begin(); }
			inline ConstIterator begin() const { return m_ranges.begin(); }
			inline Iterator end() { return m_ranges.end(); }
			inline ConstIterator end() const { return m_ranges.end(); }

		private:

			QValueList<Range> m_ranges;
	};
}

#endif
