/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "subtitle.h"
#include "subtitleline.h"
#include "subtitleiterator.h"

#include <klocale.h>

// NOTE: avoid iterating the subtitle lines through m_lines.next(), if a signal gets emitted
// while iterating, m_lines.current() item can get changed and things end up chrashing...
// ALWAYS USE SubtitleIterator class (that's what it's there for)!!!!!!

using namespace SubtitleComposer;

double Subtitle::s_defaultFramesPerSecond( 23.976 );

double Subtitle::defaultFramesPerSecond()
{
	return s_defaultFramesPerSecond;
}

void Subtitle::setDefaultFramesPerSecond( double framesPerSecond )
{
	s_defaultFramesPerSecond = framesPerSecond;
}

Subtitle::Subtitle( double framesPerSecond ):
	m_dirty( 0 ),
	m_compositeAction( 0 ),
	m_compositeActionDepth( 0 ),
	m_framesPerSecond( framesPerSecond )
{
	m_lines.setAutoDelete( true );
}

Subtitle::~Subtitle()
{
	delete m_compositeAction;
}

bool Subtitle::isDirty()
{
	return m_dirty > 0;
}

void Subtitle::clearDirty()
{
	m_dirty = 0;
}

ActionsManager& Subtitle::actionsManager()
{
	return m_actionsManager;
}

double Subtitle::framesPerSecond() const
{
	return m_framesPerSecond;
}

void Subtitle::setFramesPerSecond( double framesPerSecond )
{
	if ( m_framesPerSecond != framesPerSecond )
		processAction( new SetFramesPerSecondAction( *this, framesPerSecond ) );
}

void Subtitle::changeFramesPerSecond( double toFramesPerSecond, double fromFramesPerSecond )
{
	if ( toFramesPerSecond <= 0 )
		return;

	if ( fromFramesPerSecond <= 0 )
		fromFramesPerSecond = m_framesPerSecond;

	double scaleFactor = fromFramesPerSecond/toFramesPerSecond;

	if ( scaleFactor == 1.0 )
		return;

	beginCompositeAction( i18n( "Change Frame Rate" ) );

	setFramesPerSecond( toFramesPerSecond );

	for ( SubtitleIterator it( *this, Range::full() ); it.current(); ++it )
	{
		Time showTime = it.current()->showTime();
		showTime.adjust( 0, scaleFactor );

		Time hideTime = it.current()->hideTime();
		hideTime.adjust( 0, scaleFactor );

		it.current()->setTimes( showTime, hideTime );
	}

	endCompositeAction();
}


unsigned Subtitle::linesCount() const
{
	return m_lines.count();
}

SubtitleLine* Subtitle::line( unsigned index )
{
	return m_lines.at( index );
}

const SubtitleLine* Subtitle::line( unsigned index ) const
{
	return m_lines.at( index );
}

SubtitleLine* Subtitle::firstLine()
{
	return m_lines.first();
}

const SubtitleLine* Subtitle::firstLine() const
{
	return m_lines.first();
}

SubtitleLine* Subtitle::lastLine()
{
	return m_lines.last();
}

const SubtitleLine* Subtitle::lastLine() const
{
	return m_lines.last();
}

SubtitleLine* Subtitle::prevLine( const SubtitleLine* line )
{
	int index = m_lines.findRef( line );
	return index > 0 ? m_lines.at( index - 1 ) : 0;
}

SubtitleLine* Subtitle::nextLine( const SubtitleLine* line )
{
	int index = m_lines.findRef( line );
	return (unsigned)(index+1) < m_lines.count() ? m_lines.at( index + 1 ) : 0;
}

unsigned Subtitle::lineNumber( const SubtitleLine* line ) const
{
	int index = m_lines.findRef( line );
	return index < 0 ? 0 : (unsigned)(index + 1);
}

int Subtitle::lineIndex( const SubtitleLine* line ) const
{
	return m_lines.findRef( line );
}

void Subtitle::insertLine( SubtitleLine* line, unsigned index )
{
	SubtitleLineList lines;
	lines.append( line );
	insertLines( lines, index );
}

void Subtitle::insertLines( const SubtitleLineList& lines, unsigned index )
{
	if ( index > m_lines.count() )
		index = m_lines.count();

	SubtitleLinePtrList* linesp = new SubtitleLinePtrList;
	for ( SubtitleLineList::ConstIterator it = lines.begin(); it != lines.end(); ++it )
	{
		if ( *it != 0 )
			linesp->append( *it );
	}

	processAction( new InsertLinesAction( *this, index, linesp ) );
}

void Subtitle::removeLines( const RangeList& ranges )
{
	if ( m_lines.isEmpty() || ranges.isEmpty() )
		return;

	beginCompositeAction( i18n( "Remove Lines" ) );

	RangeList::ConstIterator rangesIt = --ranges.end();
	for ( ; rangesIt != ranges.begin(); --rangesIt )
		processAction( new RemoveLinesAction( *this, (*rangesIt).start(), (*rangesIt).end() ) );
	processAction( new RemoveLinesAction( *this, (*rangesIt).start(), (*rangesIt).end() ) );

	endCompositeAction();
}

void Subtitle::splitLines( const RangeList& ranges )
{
	beginCompositeAction( i18n( "Split Lines" ) );

	for ( SubtitleIterator it( *this, ranges, true ); it.current(); --it )
	{
		SubtitleLine* line = it.current();
		if ( line->lines() > 1 )
		{
			line->simplifySpaces();

			int autoDurationsSum = 0;
			QValueList<int> autoDurations;
			QStringList linesText = QStringList::split( '\n', line->text() );
			for ( QStringList::Iterator it2 = linesText.begin(); it2 != linesText.end(); ++it2 )
			{
				const Time& autoDuration = SubtitleLine::autoDuration( *it2, 60, 50, 50 );
				autoDurations.append( autoDuration.timeMillis() );
				autoDurationsSum += autoDuration.timeMillis();
			}

			double factor = (double)line->durationTime().timeMillis() / autoDurationsSum;

			unsigned subLineIndex = it.index();
			unsigned splitLineIndex = 0;
			for ( QStringList::Iterator it2 = linesText.begin(); it2 != linesText.end(); ++it2, ++subLineIndex, ++splitLineIndex )
			{
				if ( splitLineIndex )
				{
					SubtitleLine* newLine = new SubtitleLine();
					newLine->setStyleFlags( line->styleFlags() );
					newLine->setShowTime( line->hideTime() + 1 );
					insertLine( newLine, subLineIndex );
					line = newLine;
				}

				line->setText( *it2 );
				line->setDurationTime( (int)(factor * autoDurations[splitLineIndex])-1 );
			}
		}
	}

	endCompositeAction();
}

void Subtitle::joinLines( const RangeList& ranges )
{
	beginCompositeAction( i18n( "Join Lines" ) );

	RangeList obsoletedRanges;

	for ( RangeList::ConstIterator rangesIt = ranges.begin(); rangesIt != ranges.end(); ++rangesIt )
	{
		unsigned rangeStart = (*rangesIt).start();
		unsigned rangeEnd = SubtitleAction::getLastIndex( *this, (*rangesIt).end() );

		if ( rangeStart == rangeEnd )
			continue;

		SubtitleLine* firstLine = m_lines.at( rangeStart );
		SubtitleLine* lastLine = m_lines.at( rangeEnd );

		QString text;
		for ( SubtitleIterator it( *this, Range( rangeStart, rangeEnd-1 ) ); it.current(); ++it )
		{
			text.append( it.current()->text() );
			text.append( "\n" );
		}
		text.append( lastLine->text() );

		firstLine->setText( text );
		firstLine->setHideTime( lastLine->hideTime() );

		obsoletedRanges << Range( rangeStart + 1, rangeEnd );
	}

	removeLines( obsoletedRanges );

	endCompositeAction();
}


void Subtitle::shiftLines( const RangeList& ranges, int timeMillis )
{
	if ( timeMillis == 0 )
		return;

	beginCompositeAction( i18n( "Shift Lines" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->shiftTimes( timeMillis );

	endCompositeAction();
}

void Subtitle::adjustLines( const Range& range, const Time& firstTime, const Time& lastTime )
{
	if ( m_lines.isEmpty() || firstTime >= lastTime )
		return;

	unsigned firstIndex = range.start();
	unsigned lastIndex = SubtitleAction::getLastIndex( *this, range.end() );

	if ( firstIndex >= lastIndex )
		return;

	int oldFirstTimeMillis = m_lines.at( firstIndex )->showTime().timeMillis();
	int oldLastTimeMillis = m_lines.at( lastIndex )->showTime().timeMillis();
	int oldDeltaTimeMillis = oldLastTimeMillis - oldFirstTimeMillis;

	int newFirstTimeMillis = firstTime.timeMillis();
	int newLastTimeMillis = lastTime.timeMillis();
	int newDeltaTimeMillis = newLastTimeMillis - newFirstTimeMillis;

	int timeMillisShift = (int)(newFirstTimeMillis-((double)newDeltaTimeMillis/oldDeltaTimeMillis)*oldFirstTimeMillis);
	double scaleFactor = (double)newDeltaTimeMillis/oldDeltaTimeMillis;

	if ( timeMillisShift == 0 && scaleFactor == 1.0 )
		return;

	beginCompositeAction( i18n( "Adjust Lines" ) );

	for ( SubtitleIterator it( *this, range ); it.current(); ++it )
	{
		Time showTime = it.current()->showTime();
		showTime.adjust( timeMillisShift, scaleFactor );

		Time hideTime = it.current()->hideTime();
		hideTime.adjust( timeMillisShift, scaleFactor );

		it.current()->setTimes( showTime, hideTime );
	}

	endCompositeAction();
}

void Subtitle::sortLines( const RangeList& ranges )
{
	beginCompositeAction( i18n( "Sort Lines" ), true, false );

	SubtitleIterator it( *this, ranges );
	SubtitleLine* line = it.current();
	++it;
	for ( SubtitleLine* nextLine = it.current(); nextLine; ++it, line = nextLine, nextLine = it.current() )
	{
		if ( nextLine->showTime() <= line->showTime() ) // nextLine is not at the right position, we'll move it
		{
			int fromIndex = it.index(); // nextLine current index

			// we iterate backwards finding the correct position knowing that all previous lines ARE sorted
			int toIndex = fromIndex - 2;
			for ( it -= 2; it.current(); --it, --toIndex )
			{
				if ( it.current()->showTime() <= nextLine->showTime() )
				{
					break;
				}
			}
			toIndex++;

			processAction( new MoveLineAction( *this, fromIndex, toIndex ) );

			// moving the lines invalidates the iterator so we recreate it and advance it to where it was
			it = SubtitleIterator( *this, ranges );
			it += fromIndex;
			nextLine = it.current();
		}
	}

	endCompositeAction();
}


void Subtitle::applyDurationLimits( const RangeList& ranges, const Time& minDuration, const Time& maxDuration, bool canOverlap )
{
	if ( m_lines.isEmpty() || minDuration > maxDuration )
		return;

	beginCompositeAction( i18n( "Apply Duration Limits" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(); rangesIt != ranges.end(); ++rangesIt )
	{
		Time lineDuration;
		SubtitleIterator it( *this, *rangesIt );
		SubtitleLine* line = it.current();
		++it;
		SubtitleLine* nextLine = it.current();

		for ( ; line; ++it, line = nextLine, nextLine = it.current() )
		{
			lineDuration = line->durationTime();

			if ( lineDuration > maxDuration )
				line->setDurationTime( maxDuration );
			else if ( lineDuration < minDuration )
			{
				if ( ! nextLine ) // the last line doesn't have risk of overlapping
					line->setDurationTime( minDuration );
				else
				{
					if ( canOverlap || line->showTime() + minDuration < nextLine->showTime() )
						line->setDurationTime( minDuration );
					else // setting the duration to minDuration will cause an unwanted overlap
					{
						if ( line->hideTime() < nextLine->showTime() ) // make duration as big as possible without overlap
						{
							Time newHideTime = nextLine->showTime();
							newHideTime.shift( -1 );
							line->setHideTime( newHideTime );
						}
						// else line is already at the maximun duration without overlap (or overlapping) so we don't change it
					}
				}
			}
		}
	}

	endCompositeAction();
}



void Subtitle::setMaximunDurations( const RangeList& ranges )
{
	if ( m_lines.isEmpty() )
		return;

	beginCompositeAction( i18n( "Maximize Durations" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(); rangesIt != ranges.end(); ++rangesIt )
	{
		SubtitleIterator it( *this, *rangesIt );
		SubtitleLine* line = it.current();
		++it;
		SubtitleLine* nextLine = it.current();

		for ( ; line && nextLine; ++it, line = nextLine, nextLine = it.current() )
		{
			if ( line->hideTime() < nextLine->showTime() )
				line->setHideTime( nextLine->showTime() - 1 );
		}
	}

	endCompositeAction();
}


void Subtitle::setAutoDurations( const RangeList& ranges, unsigned charMillis, unsigned wordMillis, unsigned lineMillis, bool canOverlap )
{
	if ( m_lines.isEmpty() )
		return;

	beginCompositeAction( i18n( "Set Automatic Durations" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(); rangesIt != ranges.end(); ++rangesIt )
	{
		Time autoDuration;

		SubtitleIterator it( *this, *rangesIt );
		SubtitleLine* line = it.current();
		++it;
		SubtitleLine* nextLine = it.current();

		for ( ; line; ++it, line = nextLine, nextLine = it.current() )
		{
			autoDuration = line->autoDuration( charMillis, wordMillis, lineMillis );

			if ( ! nextLine ) // the last line doesn't have risk of overlapping
				line->setDurationTime( autoDuration );
			else
			{
				if ( canOverlap || line->showTime() + autoDuration < nextLine->showTime() )
					line->setDurationTime( autoDuration );
				else // setting the duration to autoDuration will cause an unwanted overlap
				{
					Time newHideTime = nextLine->showTime();
					newHideTime.shift( -1 );
					line->setHideTime( newHideTime );
				}
			}
		}
	}

	endCompositeAction();
}



void Subtitle::fixPunctuation( const RangeList& ranges, bool spaces, bool quotes, bool engI, bool ellipsis )
{
	if ( m_lines.isEmpty() || (! spaces && ! quotes && ! engI && ! ellipsis) )
		return;

	beginCompositeAction( i18n( "Fix Lines Punctuation" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(); rangesIt != ranges.end(); ++rangesIt )
	{
		SubtitleIterator it( *this, *rangesIt );

		bool continues = false;
		if ( it.index() > 0 )
			SubtitleLine::fixPunctuation( m_lines.at( it.index() - 1 )->text(), spaces, quotes, engI, ellipsis, &continues );

		for ( ; it.current(); ++it )
			it.current()->setText(SubtitleLine::fixPunctuation(it.current()->text(), spaces, quotes, engI, ellipsis, &continues));
	}

	endCompositeAction();
}

void Subtitle::changeCase( const RangeList& ranges, CaseOp caseOp, bool lowerFirst )
{
	if ( m_lines.isEmpty() )
		return;

	beginCompositeAction( i18n( "Change Case" ) );

	for ( RangeList::ConstIterator rangesIt = ranges.begin(); rangesIt != ranges.end(); ++rangesIt )
	{
		SubtitleIterator it( *this, *rangesIt );

		bool continues = false;
		if ( it.index() > 0 && caseOp == Sentence )
			SubtitleLine::sentenceCase( m_lines.at( it.index() - 1 )->text(), lowerFirst, &continues );

		for ( ; it.current(); ++it )
		{
			switch ( caseOp )
			{
				case Upper:
					it.current()->setText( it.current()->text().upper() );
					break;
				case Lower:
					it.current()->setText( it.current()->text().lower() );
					break;
				case Title:
					it.current()->setText( SubtitleLine::titleCase( it.current()->text(), lowerFirst ) );
					break;
				case Sentence:
					it.current()->setText( SubtitleLine::sentenceCase( it.current()->text(), lowerFirst, &continues ) );
				default:
					break;
			}
		}
	}

	endCompositeAction();
}


void Subtitle::adjustTexts( const RangeList& ranges, unsigned minLengthForLineBreak )
{
	SubtitleCompositeActionExecutor executor( *this, i18n( "Smart Lines Adjust" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->adjustText( minLengthForLineBreak );
}

void Subtitle::unbreakTexts( const RangeList& ranges )
{
	SubtitleCompositeActionExecutor executor( *this, i18n( "Unbreak Lines" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->unbreakText();
}

void Subtitle::simplifySpaces( const RangeList& ranges )
{
	SubtitleCompositeActionExecutor executor( *this, i18n( "Simplify Spaces" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->simplifySpaces();
}


void Subtitle::syncToSubtitle( const Subtitle& referenceSubtitle )
{
	beginCompositeAction( i18n( "Synchronize Subtitles" ) );

	for ( SubtitleIterator it( *this, Range::full() ), refIt( referenceSubtitle, Range::full() );
		  it.current() && refIt.current();
		  ++it, ++refIt )
		it.current()->setTimes( refIt.current()->showTime(), refIt.current()->hideTime() );

	endCompositeAction();
}

void Subtitle::joinSubtitles( const Subtitle& subtitle, unsigned timeMillis )
{
	if ( ! subtitle.m_lines.count() )
		return;

	SubtitleLinePtrList* lines = new SubtitleLinePtrList;
	for ( SubtitleIterator it( *this, Range::full() ); it.current(); ++it )
	{
		SubtitleLine* newLine = new SubtitleLine( *(it.current()) );
		newLine->shiftTimes( timeMillis );
		lines->append( newLine );
	}

	beginCompositeAction( i18n( "Join Subtitles" ) );

	processAction( new InsertLinesAction( *this, m_lines.count(), lines ) );

	endCompositeAction();
}


void Subtitle::splitSubtitle( Subtitle& subtitle, unsigned timeMillis, bool shiftSplittedLines )
{
	if ( ! m_lines.count() )
		return;

	int splitIndex = -1;
	SubtitleLinePtrList* lines = new SubtitleLinePtrList;
	for ( SubtitleIterator it( *this, Range::full() ); it.current(); ++it )
	{
		if ( it.current()->showTime() < timeMillis )
			continue;

		if ( splitIndex < 0 )
			splitIndex = it.current()->number()-1;

		SubtitleLine* newLine = new SubtitleLine( *(it.current()) );
		if ( shiftSplittedLines )
			newLine->shiftTimes( -timeMillis );
		lines->append( newLine );
	}

	if ( splitIndex < 0 )
		delete lines;
	else
	{
		subtitle.beginCompositeAction( i18n( "Split Subtitles" ) );
		if ( subtitle.m_lines.count() )
			subtitle.processAction( new RemoveLinesAction( subtitle, 0, Range::MaxIndex ) );
		subtitle.processAction( new InsertLinesAction( subtitle, 0, lines ) );
		subtitle.endCompositeAction();

		beginCompositeAction( i18n( "Split Subtitles" ) );
		processAction( new RemoveLinesAction( *this, splitIndex, Range::MaxIndex ) );
		endCompositeAction();
	}
}

void Subtitle::setStyleFlags( const RangeList& ranges, int styleFlags )
{
	beginCompositeAction( i18n( "Set Lines Style" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setStyleFlags( styleFlags );

	endCompositeAction();
}

void Subtitle::setStyleFlags( const RangeList& ranges, int styleFlags, bool value )
{
	beginCompositeAction( i18n( "Set Lines Style" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setStyleFlags( styleFlags, value );

	endCompositeAction();
}

void Subtitle::toggleStyleFlag( const RangeList& ranges, SubtitleLine::StyleFlag styleFlag )
{
	SubtitleIterator it( *this, ranges );
	if ( ! it.current() )
		return;

	beginCompositeAction( i18n( "Toggle Lines Style" ) );

	setStyleFlags( ranges, styleFlag, ! (it.current()->styleFlags() & styleFlag) );

	endCompositeAction();
}


void Subtitle::setMarked( const RangeList& ranges, bool value )
{
	beginCompositeAction( value ? i18n( "Set Lines Mark" ) : i18n( "Clear Lines Mark" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setMarked( value );

	endCompositeAction();
}

void Subtitle::toggleMarked( const RangeList& ranges )
{
	SubtitleIterator it( *this, ranges );
	if ( ! it.current() )
		return;

	beginCompositeAction( i18n( "Toggle Lines Marked" ) );

	setMarked( ranges, ! it.current()->marked() );

	endCompositeAction();
}


void Subtitle::clearErrors( const RangeList& ranges, int errorFlags )
{
	beginCompositeAction( i18n( "Clear Lines Errors" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
		it.current()->setErrorFlags( errorFlags, false );

	endCompositeAction();
}

void Subtitle::checkErrors( const RangeList& ranges, int errorFlags, int minDurationMillis, int maxDurationMillis, int minDurationPerCharMillis, int maxDurationPerCharMillis, int maxChars, int maxLines )
{
	beginCompositeAction( i18n( "Check Lines Errors" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
	{
		if ( errorFlags & SubtitleLine::EmptyText )
			it.current()->checkEmptyTextError();

		if ( errorFlags & SubtitleLine::OverlappingTimes )
			it.current()->checkOverlappingTimesError();

		if ( errorFlags & SubtitleLine::TooLongDuration )
			it.current()->checkTooLongDurationError( maxDurationMillis );

		if ( errorFlags & SubtitleLine::TooShortDuration )
			it.current()->checkTooShortDurationError( minDurationMillis );

		if ( errorFlags & SubtitleLine::TooLongDurationPerChar )
			it.current()->checkTooLongDurationPerCharError( maxDurationPerCharMillis );

		if ( errorFlags & SubtitleLine::TooShortDurationPerChar )
			it.current()->checkTooShortDurationPerCharError( minDurationPerCharMillis );

		if ( errorFlags & SubtitleLine::TooManyChars )
			it.current()->checkTooManyCharsError( maxChars );

		if ( errorFlags & SubtitleLine::TooManyLines )
			it.current()->checkTooManyLinesError( maxLines );

		if ( errorFlags & SubtitleLine::UnneededSpaces )
			it.current()->checkUnneededSpacesError();

		if ( errorFlags & SubtitleLine::CapitalAfterEllipsis )
			it.current()->checkCapitalAfterEllipsisError();

		if ( errorFlags & SubtitleLine::UnneededDash )
			it.current()->checkUnneededDashError();
	}

	endCompositeAction();
}

void Subtitle::recheckErrors( const RangeList& ranges, int minDurationMillis, int maxDurationMillis, int minDurationPerCharMillis, int maxDurationPerCharMillis, int maxChars, int maxLines )
{
	beginCompositeAction( i18n( "Check Lines Errors" ) );

	for ( SubtitleIterator it( *this, ranges ); it.current(); ++it )
	{
		int errorFlags = it.current()->errorFlags();

		if ( errorFlags & SubtitleLine::EmptyText )
			it.current()->checkEmptyTextError();

		if ( errorFlags & SubtitleLine::OverlappingTimes )
			it.current()->checkOverlappingTimesError();

		if ( errorFlags & SubtitleLine::TooLongDuration )
			it.current()->checkTooLongDurationError( maxDurationMillis );

		if ( errorFlags & SubtitleLine::TooShortDuration )
			it.current()->checkTooShortDurationError( minDurationMillis );

		if ( errorFlags & SubtitleLine::TooLongDurationPerChar )
			it.current()->checkTooLongDurationPerCharError( maxDurationPerCharMillis );

		if ( errorFlags & SubtitleLine::TooShortDurationPerChar )
			it.current()->checkTooShortDurationPerCharError( minDurationPerCharMillis );

		if ( errorFlags & SubtitleLine::TooManyChars )
			it.current()->checkTooManyCharsError( maxChars );

		if ( errorFlags & SubtitleLine::TooManyLines )
			it.current()->checkTooManyLinesError( maxLines );

		if ( errorFlags & SubtitleLine::UnneededSpaces )
			it.current()->checkUnneededSpacesError();

		if ( errorFlags & SubtitleLine::CapitalAfterEllipsis )
			it.current()->checkCapitalAfterEllipsisError();

		if ( errorFlags & SubtitleLine::UnneededDash )
			it.current()->checkUnneededDashError();
	}

	endCompositeAction();
}


void Subtitle::processAction( Action* action )
{
	if ( m_compositeAction )
		m_compositeAction->appendAction( action );
	else
		m_actionsManager.execAndStore( action );
}

void Subtitle::beginCompositeAction( const QString& title, bool immediateExecution, bool delaySignals )
{
	m_compositeActionDepth++;

	if ( ! m_compositeAction )
		m_compositeAction = new CompositeAction( title, immediateExecution, delaySignals );
}

void Subtitle::endCompositeAction()
{
	m_compositeActionDepth--;

	if ( ! m_compositeActionDepth )
	{
		// NOTE: it is EXTREMELY IMPORTANT to mark the composite action as finished (setting it to null)
		// before performing m_actionsManager.execAndStore() because that call may spawn other composite
		// actions that won't be correctly initialized if m_compositeAction was not set to 0.
		CompositeAction* action = m_compositeAction;
		m_compositeAction = 0;
		if ( action->count() )
			m_actionsManager.execAndStore( action );
		else
			delete action;
	}
}


SubtitleCompositeActionExecutor::SubtitleCompositeActionExecutor( Subtitle& subtitle, const QString& title ):
	m_subtitle( subtitle )
{
	m_subtitle.beginCompositeAction( title );
}

SubtitleCompositeActionExecutor::~SubtitleCompositeActionExecutor()
{
	m_subtitle.endCompositeAction();
}




/// SUBTITLE ACTION
/// ===============

SubtitleAction::SubtitleAction( Subtitle& subtitle, bool dirty, const QString& description ):
	Action( description ),
	m_subtitle( subtitle ),
	m_dirty( dirty ) {}
SubtitleAction::~SubtitleAction() {}

void SubtitleAction::_preRedo()
{
	if ( m_dirty )
		m_subtitle.m_dirty++;
}

void SubtitleAction::_preUndo()
{
	if ( m_dirty )
		m_subtitle.m_dirty--;
}

void SubtitleAction::setLineSubtitle( SubtitleLine* line, Subtitle* subtitle )
{
	line->m_subtitle = subtitle;
}


/// SET FRAMES PER SECOND ACTION
/// ============================

SetFramesPerSecondAction::SetFramesPerSecondAction( Subtitle& subtitle, double framesPerSecond ):
	SubtitleAction( subtitle, true, i18n( "Set Frame Rate" ) ),
	m_framesPerSecond( framesPerSecond ) {}
SetFramesPerSecondAction::~SetFramesPerSecondAction() {}

void SetFramesPerSecondAction::_redo()
{
	double aux = m_subtitle.m_framesPerSecond;
	m_subtitle.m_framesPerSecond = m_framesPerSecond;
	m_framesPerSecond = aux;
}

void SetFramesPerSecondAction::_undo()
{
	_redo();
}

void SetFramesPerSecondAction::_emitRedoSignals()
{
	emit m_subtitle.framesPerSecondChanged( m_subtitle.m_framesPerSecond );
}

void SetFramesPerSecondAction::_emitUndoSignals()
{
	_emitRedoSignals();
}


/// INSERT LINES ACTION
/// ===================

InsertLinesAction::InsertLinesAction( Subtitle& subtitle, unsigned insertIndex, SubtitleLinePtrList* lines ):
	SubtitleAction( subtitle, true, i18n( lines->count() == 1 ? "Insert Line" : "Insert Lines" ) ),
	m_insertIndex( getInsertionIndex( subtitle, insertIndex ) ),
	m_lastIndex( m_insertIndex + lines->count() - 1 ),
	m_lines( lines ) { m_lines->setAutoDelete ( true ); }
InsertLinesAction::~InsertLinesAction() { delete m_lines; }

void InsertLinesAction::_redo()
{
	if ( m_insertIndex > m_lastIndex )
		return;

	unsigned insertOffset = 0;
	SubtitleLine* line = m_lines->first(); // make first item current item
	while ( ( line = m_lines->take() ) )
	{
		m_subtitle.m_lines.insert( m_insertIndex + insertOffset++, line );
		setLineSubtitle( line, &m_subtitle );
	}
}

void InsertLinesAction::_undo()
{
	if ( m_insertIndex > m_lastIndex )
		return;

	m_subtitle.m_lines.at ( m_insertIndex ); // make m_insertIndex current item
	for ( unsigned index = m_insertIndex; index <= m_lastIndex; ++index )
	{
		SubtitleLine* line = m_subtitle.m_lines.take();
		setLineSubtitle( line, 0 );
		m_lines->append( line );
	}
}

void InsertLinesAction::_emitRedoSignals()
{
	emit m_subtitle.linesInserted( m_insertIndex, m_lastIndex );
}

void InsertLinesAction::_emitUndoSignals()
{
	emit m_subtitle.linesRemoved( m_insertIndex, m_lastIndex );
}


/// REMOVE LINES ACTION
/// ===================

RemoveLinesAction::RemoveLinesAction( Subtitle& subtitle, unsigned firstIndex, unsigned lastIndex ):
	SubtitleAction( subtitle, i18n( "Remove Line" ) ),
	m_firstIndex( firstIndex ),
	m_lastIndex( getLastIndex( subtitle, lastIndex ) ),
	m_lines()
{
	m_lines.setAutoDelete( true );

	if ( m_firstIndex != m_lastIndex )
		m_description = i18n( "Remove Lines" );
}

RemoveLinesAction::~RemoveLinesAction()
{
}

void RemoveLinesAction::_redo()
{
	if ( m_firstIndex > m_lastIndex )
		return;

	m_subtitle.m_lines.at( m_firstIndex ); // make m_firstIndex current item
	for ( unsigned index = m_firstIndex; index <= m_lastIndex; ++index )
	{
		SubtitleLine* line = m_subtitle.m_lines.take();
		setLineSubtitle( line, 0 );
		m_lines.append( line );
	}
}

void RemoveLinesAction::_undo()
{
	if ( m_firstIndex > m_lastIndex )
		return;

	unsigned insertOffset = 0;
	SubtitleLine* line = m_lines.first(); // make first item current item
	while ( ( line = m_lines.take() ) )
	{
		m_subtitle.m_lines.insert( m_firstIndex + insertOffset++, line );
		setLineSubtitle( line, &m_subtitle );
	}
}

void RemoveLinesAction::_emitRedoSignals()
{
	emit m_subtitle.linesRemoved( m_firstIndex, m_lastIndex );
}

void RemoveLinesAction::_emitUndoSignals()
{
	emit m_subtitle.linesInserted( m_firstIndex, m_lastIndex );
}




/// MOVE LINE ACTION
/// ================

MoveLineAction::MoveLineAction( Subtitle& subtitle, unsigned fromIndex, unsigned toIndex ):
	SubtitleAction( subtitle, i18n( "Move Line" ) ),
	m_fromIndex( fromIndex ),
	m_toIndex( getLastIndex( subtitle, toIndex ) ) {}
MoveLineAction::~MoveLineAction() {}

bool MoveLineAction::mergeWithPrevious( Action* pa )
{
	MoveLineAction* prevAction = tryCastTo<MoveLineAction>( pa );
	if ( ! prevAction || &prevAction->m_subtitle != &m_subtitle )
		return false;

	bool compressed = false;

	if ( prevAction->m_toIndex == m_fromIndex )
	{
		m_fromIndex = prevAction->m_fromIndex;
		compressed = true;
	}

	// when the distance between fromIndex and toIndex is 1, the action is the same as if the values were swaped
	else if ( m_toIndex - m_fromIndex == 1 || m_fromIndex - m_toIndex == 1 )
	{
		if ( prevAction->m_toIndex == m_toIndex )
		{
			m_toIndex = m_fromIndex;
			m_fromIndex = prevAction->m_fromIndex;
			compressed = true;
		}

		// same as before, but now we consider inverting the previous action too
		else if ( prevAction->m_toIndex - prevAction->m_fromIndex == 1 || prevAction->m_fromIndex - prevAction->m_toIndex == 1 )
		{
			if ( prevAction->m_fromIndex == m_toIndex )
			{
				m_toIndex = m_fromIndex;
				m_fromIndex = prevAction->m_toIndex;
				compressed = true;
			}
		}

	}

	// again, same as before, but now we consider inverting only the previous action
	else if ( prevAction->m_toIndex - prevAction->m_fromIndex == 1 || prevAction->m_fromIndex - prevAction->m_toIndex == 1 )
	{
		if ( prevAction->m_fromIndex == m_fromIndex )
		{
			m_fromIndex = prevAction->m_toIndex;
			compressed = true;
		}
	}

	return compressed;
}

void MoveLineAction::_redo()
{
	SubtitleLine* line = m_subtitle.m_lines.take( m_fromIndex );
	m_subtitle.m_lines.insert( m_toIndex, line );
}

void MoveLineAction::_undo()
{
	SubtitleLine* line = m_subtitle.m_lines.take( m_toIndex );
	m_subtitle.m_lines.insert( m_fromIndex, line );
}

void MoveLineAction::_emitRedoSignals()
{
	emit m_subtitle.linesRemoved( m_fromIndex, m_fromIndex );
	emit m_subtitle.linesInserted( m_toIndex, m_toIndex );
}

void MoveLineAction::_emitUndoSignals()
{
	emit m_subtitle.linesRemoved( m_toIndex, m_toIndex );
	emit m_subtitle.linesInserted( m_fromIndex, m_fromIndex );
}



