#ifndef SUBTITLE_H
#define SUBTITLE_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "ranges.h"
#include "sctime.h"
#include "action.h"
#include "actionsmanager.h"
#include "subtitleline.h"

#include <qobject.h>
#include <qptrlist.h>
#include <qvaluevector.h>
#include <qvaluelist.h>
#include <qstringlist.h>

namespace SubtitleComposer
{
	class Subtitle : public QObject
	{
		Q_OBJECT

		friend class SubtitleLine;
		friend class SubtitleIterator;

		friend class SubtitleAction;
		friend class SetFramesPerSecondAction;
		friend class InsertLinesAction;
		friend class RemoveLinesAction;
		friend class MoveLineAction;

		friend class SubtitleLineAction;

		friend class SubtitleCompositeActionExecutor;

		friend class FormatsManager;

		public:

			typedef enum { Upper=0, Lower, Title, Sentence } CaseOp;

			typedef QValueList<SubtitleLine*> SubtitleLineList;

			static double defaultFramesPerSecond();
			static void setDefaultFramesPerSecond( double framesPerSecond );

			Subtitle( double framesPerSecond=defaultFramesPerSecond() );
			virtual ~Subtitle();

			bool isDirty();
			ActionsManager& actionsManager();

			double framesPerSecond() const;
			void setFramesPerSecond( double framesPerSecond );
			void changeFramesPerSecond( double toFramesPerSecond, double fromFramesPerSecond=-1.0 );

			unsigned linesCount() const;

			SubtitleLine* line( unsigned index );
			const SubtitleLine* line( unsigned index ) const;
			SubtitleLine* firstLine();
			const SubtitleLine* firstLine() const;
			SubtitleLine* lastLine();
			const SubtitleLine* lastLine() const;

			SubtitleLine* prevLine( const SubtitleLine* line );
			SubtitleLine* nextLine( const SubtitleLine* line );

			unsigned lineNumber( const SubtitleLine* line ) const;
			int lineIndex( const SubtitleLine* line ) const;

			void insertLine( SubtitleLine* line, unsigned index=Range::MaxIndex );
			void insertLines( const SubtitleLineList& lines, unsigned index=Range::MaxIndex );
			void removeLines( const RangeList& ranges );

			void splitLines( const RangeList& ranges );
			void joinLines( const RangeList& ranges );

			void shiftLines( const RangeList& ranges, int timeMillis );
			void adjustLines( const Range& range, const Time& firstTime, const Time& lastTime );
			void sortLines( const RangeList& ranges );

			void applyDurationLimits( const RangeList& ranges, const Time& minDuration, const Time& maxDuration, bool canOverlap );
			void setMaximunDurations( const RangeList& ranges );
			void setAutoDurations( const RangeList& ranges, unsigned charMillis, unsigned wordMillis, unsigned lineMillis, bool canOverlap );

			void fixPunctuation( const RangeList& ranges, bool spaces, bool quotes, bool engI, bool ellipsis );
			void changeCase( const RangeList& ranges, CaseOp caseOp, bool lowerFirst=false );
			void adjustTexts( const RangeList& ranges, unsigned minLengthForLineBreak );
			void unbreakTexts( const RangeList& ranges );
			void simplifySpaces( const RangeList& ranges );

			void syncToSubtitle( const Subtitle& referenceSubtitle );
			void splitSubtitle( Subtitle& subtitle2, unsigned timeMillis, bool shiftSplittedLines );
			void joinSubtitles( const Subtitle& subtitle, unsigned timeMillis );

			void setStyleFlags( const RangeList& ranges, int styleFlags );
			void setStyleFlags( const RangeList& ranges, int styleFlags, bool value );
			void toggleStyleFlag( const RangeList& ranges, SubtitleLine::StyleFlag styleFlag );

			void setMarked( const RangeList& ranges, bool value );
			void toggleMarked( const RangeList& ranges );

			void clearErrors( const RangeList& ranges, int errorFlags );
			void checkErrors( const RangeList& ranges, int errorFlags, int minDurationMillis, int maxDurationMillis, int minDurationPerCharMillis, int maxDurationPerCharMillis, int maxChars, int maxLines );
			void recheckErrors( const RangeList& ranges, int minDurationMillis, int maxDurationMillis, int minDurationPerCharMillis, int maxDurationPerCharMillis, int maxChars, int maxLines );

		signals:

			void framesPerSecondChanged( double fps );
			void linesInserted( unsigned firstIndex, unsigned lastIndex );
			void linesRemoved( unsigned firstIndex, unsigned lastIndex );

		private:

			void clearDirty();

			void beginCompositeAction( const QString& title, bool immediateExecution=true, bool delaySignals=true );
			void endCompositeAction();
			void processAction( Action* action );

		private:

			ActionsManager m_actionsManager;
			unsigned long m_dirty;

			CompositeAction* m_compositeAction;
			unsigned long m_compositeActionDepth;

			double m_framesPerSecond;
			mutable QPtrList<SubtitleLine> m_lines;

			static double s_defaultFramesPerSecond;
	};

	class SubtitleCompositeActionExecutor
	{
		public:

			SubtitleCompositeActionExecutor( Subtitle& subtitle, const QString& title );
			~SubtitleCompositeActionExecutor();

		private:

			Subtitle& m_subtitle;
	};

	typedef QPtrList<SubtitleLine> SubtitleLinePtrList;
	typedef QValueVector<bool> BoolList;
	typedef QValueVector<Time> TimeList;

	class SubtitleAction : public Action
	{
		public:

			friend class Subtitle;

			SubtitleAction( Subtitle& subtitle, bool dirty, const QString& description=QString::null );
			virtual ~SubtitleAction();

			void setLineSubtitle( SubtitleLine* line, Subtitle* subtitle );

			inline static unsigned getLastIndex( const Subtitle& subtitle, unsigned lastIndex )
			{
				return lastIndex == Range::MaxIndex ? subtitle.linesCount()-1 : lastIndex;
			}

			inline static unsigned getInsertionIndex( const Subtitle& subtitle, unsigned lastIndex )
			{
				return lastIndex == Range::MaxIndex ? subtitle.linesCount() : lastIndex;
			}

		protected:

			virtual void _preRedo();
			virtual void _preUndo();

		protected:

			Subtitle& m_subtitle;
			bool m_dirty;
	};

	class SetFramesPerSecondAction : public SubtitleAction
	{
		public:

			SetFramesPerSecondAction( Subtitle& subtitle, double framesPerSecond );
			virtual ~SetFramesPerSecondAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			double m_framesPerSecond;
	};

	class InsertLinesAction : public SubtitleAction
	{
		public:

			InsertLinesAction( Subtitle& subtitle, unsigned insertIndex, SubtitleLinePtrList* lines );
			virtual ~InsertLinesAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			unsigned m_insertIndex;
			unsigned m_lastIndex;
			SubtitleLinePtrList* m_lines;
	};

	class RemoveLinesAction : public SubtitleAction
	{
		public:

			RemoveLinesAction( Subtitle& subtitle, unsigned firstIndex, unsigned lastIndex );
			virtual ~RemoveLinesAction();

		protected:

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			unsigned m_firstIndex;
			unsigned m_lastIndex;
			SubtitleLinePtrList m_lines;
	};


	class MoveLineAction : public SubtitleAction
	{
		public:

			MoveLineAction( Subtitle& subtitle, unsigned fromIndex, unsigned toIndex );
			virtual ~MoveLineAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _undo();

			virtual void _emitRedoSignals();
			virtual void _emitUndoSignals();

		private:

			unsigned m_fromIndex;
			unsigned m_toIndex;
	};

}

#endif
