/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "subtitle.h"
#include "subtitleline.h"

#include <qregexp.h>
#include <klocale.h>

#include <math.h>

using namespace SubtitleComposer;

SubtitleLine::ErrorFlag SubtitleLine::errorFlag( SubtitleLine::ErrorID id )
{
	if ( id < 0 || id >= ErrorSIZE )
		return (ErrorFlag)0;

	return (ErrorFlag)(0x1 << id);
}

SubtitleLine::ErrorID SubtitleLine::errorID( SubtitleLine::ErrorFlag flag )
{
	if ( flag < 1 )
		return ErrorUNKNOWN;

	int id = (int)log2( flag );
	return id < ErrorSIZE ? (ErrorID)id : ErrorUNKNOWN;
}

SubtitleLine::SubtitleLine( const QString& text ):
	QObject(),
	m_subtitle( 0 ),
	m_text( text ),
	m_showTime(),
	m_hideTime(),
	m_styleFlags( 0 ),
	m_errorFlags( 0 ),
	m_marked( false )
{
}

SubtitleLine::SubtitleLine( const QString& text, const Time& showTime, const Time& hideTime, int styleFlags ):
	QObject(),
	m_subtitle( 0 ),
	m_text( text ),
	m_showTime( showTime ),
	m_hideTime( hideTime ),
	m_styleFlags( styleFlags ),
	m_errorFlags( 0 ),
	m_marked( false )
{
}

SubtitleLine::SubtitleLine( const SubtitleLine& line ):
	QObject(),
	m_subtitle( 0 ),
	m_text( line.m_text ),
	m_showTime( line.m_showTime ),
	m_hideTime( line.m_hideTime ),
	m_styleFlags( line.m_styleFlags ),
	m_errorFlags( line.m_errorFlags ),
	m_marked( line.m_marked )
{
}

SubtitleLine& SubtitleLine::operator=( const SubtitleLine& line )
{
	if ( this == &line )
		return *this;

	m_text = line.m_text;
	m_showTime = line.m_showTime;
	m_hideTime = line.m_hideTime;
	m_styleFlags = line.m_styleFlags;
	m_errorFlags = line.m_errorFlags;
	m_marked = line.m_marked;

	return *this;
}

unsigned SubtitleLine::number() const
{
	return m_subtitle ? m_subtitle->lineNumber( this ) : 0;
}

int SubtitleLine::index() const
{
	return m_subtitle ? m_subtitle->lineIndex( this ) : -1;
}

Subtitle* SubtitleLine::subtitle()
{
	return m_subtitle;
}

const Subtitle* SubtitleLine::subtitle() const
{
	return m_subtitle;
}

SubtitleLine* SubtitleLine::prevLine()
{
	return m_subtitle ? m_subtitle->prevLine( this ) : 0;
}

SubtitleLine* SubtitleLine::nextLine()
{
	return m_subtitle ? m_subtitle->nextLine( this ) : 0;
}

QString SubtitleLine::text() const
{
	return m_text;
}

void SubtitleLine::setText( const QString& text )
{
	if ( m_text != text )
		processAction( new SetLineTextAction( *this, text ) );
}

QString SubtitleLine::fixPunctuation( const QString& t, bool spaces, bool quotes, bool engI, bool ellipsis, bool* cont )
{
	if ( ! t.length() )
		return t;

	QString text( t );

	if ( spaces )
	{
		text = simplifyWhiteSpaces( text );

		// remove spaces after " or ' at the beginning of line
		text.replace( QRegExp( "^([\"'])\\s" ), "\\1" );

		// remove space before " or ' at the end of line
		text.replace( QRegExp( "\\s([\"'])$" ), "\\1" );

		// if not present, add space after '?', '!', ',', ';', ':', ')' and ']'
		text.replace( QRegExp( "([\\?!,;:\\)\\]])([^\\s\"'])" ), "\\1 \\2" );

		// if not present, add space after '.'
		text.replace( QRegExp( "(\\.)([^\\s\\.\"'])" ), "\\1 \\2" );

		// remove space after '¿', '¡', '(' and '['
		text.replace( QRegExp( "([¿¡\\(\\[])\\s" ), "\\1" );

		// remove space before '?', '!', ',', ';', ':', '.', ')' and ']'
		text.replace( QRegExp( "\\s([\\?!,;:\\.\\)\\]])" ), "\\1" );

		// remove space after ... at the beginning of sentence
		text.replace( QRegExp( "^\\.\\.\\.\\s" ), "..." );
	}

	if ( quotes ) // quotes and double quotes
	{
		text.replace( QRegExp( "`|´|" ), "'" );
		text.replace( QRegExp( "''|«|»" ), "\"" );
	}

	if ( engI ) // fix english I pronoun capitalization
		text.replace( QRegExp( "([\\s\"'\\(\\[])i([\\s'\",;:\\.\\?!\\]\\)]|$)" ), "\\1I\\2" );

	if ( ellipsis ) // fix ellipsis
	{
		text.replace( QRegExp( "[,;]?\\.{2,}" ), "..." );
		text.replace( QRegExp( "[,;]\\s*$" ), "..." );

		if ( text.find( QRegExp( "[\\.:?!\\)\\]'\\\"]$" ) ) == -1 )
			text.append( "..." );

		if ( cont )
		{
			if ( *cont && text.find( QRegExp( "^\\s*\\.{3}[^\\.]?" ) ) == -1 )
				text.replace( QRegExp( "^\\s*\\.*\\s*" ), "..." );

			*cont = text.find( QRegExp( "\\.{3,3}\\s*$" ) ) != -1;
		}
	}
	else
	{
		if ( cont )
		{
			*cont = text.find( QRegExp( "[?!\\)\\]'\\\"]\\s*$" ) ) == -1;

			if ( ! *cont )
				*cont = text.find( QRegExp( "[^\\.]?\\.\\s*$" ) ) == -1;
		}
	}

	return text;
}

QString SubtitleLine::titleCase( const QString& t, bool lowerFirst )
{
	static const QString wordSeparators( " -_([:,;./\\\t\n\"" );

	QString text( lowerFirst ? t.lower() : t );

	bool wordStart = true;
	unsigned length = text.length();
	for ( unsigned idx = 0; idx < length; ++idx )
	{
		QCharRef chr = text.at( idx );
		if ( wordStart )
		{
			if ( ! wordSeparators.contains( chr ) )
			{
				wordStart = false;
				chr = chr.upper();
			}
		}
		else if ( wordSeparators.contains( chr ) )
			wordStart = true;
	}

	return text;
}

QString SubtitleLine::sentenceCase( const QString& t, bool lowerFirst, bool* cont )
{
	if ( ! t.length() )
		return t;

	static const QString sentenceEndChars( ".?!" );

	QString text( lowerFirst ? t.lower() : t );

	bool startSentence = cont ? ! *cont : true;
	int prevDots = 0;
	for ( unsigned idx = 0; idx < text.length(); ++idx )
	{
		QCharRef chr = text.at( idx );
		if ( sentenceEndChars.contains( chr ) )
		{
			if ( chr == '.' )
			{
				prevDots++;
				startSentence = prevDots < 3;
			}
			else
			{
				prevDots = 0;
				startSentence = true;
			}
		}
		else
		{
			if ( startSentence && chr.isLetterOrNumber() )
			{
				chr = chr.upper();
				startSentence = false;
			}

			if ( ! chr.isSpace() )
				prevDots = 0;
		}
	}

	if ( cont )
		*cont = prevDots != 1 && ! startSentence;

	return text;
}


void SubtitleLine::adjustText( unsigned minLengthForBreak )
{
	QString text( m_text );
	text.replace( "\n", " " );

	if ( text.length() <= minLengthForBreak )
	{
		setText( text );
		return;
	}

	static const QRegExp spaceRegExp( "[^ \t\n][ \t\n]" );

	// this is a fricking test case
	// this is a fri|cking test case	IDEAL: 13
	// this is a| fricking test case	LOWER: 10 <--
	// this is a fricking| test case	UPPER: 18

	int position = text.length()/2;
	int lowerPosition = text.findRev( spaceRegExp, position );
	int upperPosition = text.find( spaceRegExp, position );

	if ( (lowerPosition == -1 || lowerPosition == 0) && (upperPosition == -1 || upperPosition == (signed)(text.length()-1)) )
	{
		setText( text );
		return;
	}
	else if ( position - lowerPosition <= upperPosition - position )
		position = lowerPosition;
	else
		position = upperPosition;

	position++;
	text.insert( position, "\n" );
	position++;
	while ( text.at( position ).isSpace() && position < (signed)text.length() )
		text.remove( position, 1 );

	setText( text );
}

void SubtitleLine::unbreakText()
{
	setText( text().replace( "\n", " " ) );
}

QString SubtitleLine::simplifyWhiteSpaces( QString text )
{
	static const QRegExp regExp1( " *\n *" );
	static const QRegExp regExp2( " +" );
	static const QRegExp regExp3( "\n+" );

	text.replace( "\t", " " );
	text.replace( regExp1, "\n" );
	text.replace( regExp2, " " );
	text.replace( regExp3, "\n" );

	return text.stripWhiteSpace();
}

void SubtitleLine::simplifySpaces()
{
	setText( simplifyWhiteSpaces( m_text ) );
}

Time SubtitleLine::showTime() const
{
	return m_showTime;
}

void SubtitleLine::setShowTime( const Time& showTime )
{
	if ( m_showTime != showTime )
		processAction( new SetLineShowTimeAction( *this, showTime ) );
}


Time SubtitleLine::hideTime() const
{
	return m_hideTime;
}

void SubtitleLine::setHideTime( const Time& hideTime )
{
	if ( m_hideTime != hideTime )
		processAction( new SetLineHideTimeAction( *this, hideTime ) );
}

Time SubtitleLine::durationTime() const
{
	return Time( m_hideTime.timeMillis() - m_showTime.timeMillis() );
}

void SubtitleLine::setDurationTime( const Time& durationTime )
{
	setHideTime( m_showTime + durationTime );
}

unsigned SubtitleLine::characters()
{
	return m_text.simplifyWhiteSpace().length();
}

unsigned SubtitleLine::words()
{
	QString text( m_text.simplifyWhiteSpace() );
	return text.length() ? text.contains( ' ' ) + 1 : 0;
}

unsigned SubtitleLine::lines()
{
	QString text( simplifyWhiteSpaces( m_text ) );
	return text.length() ? text.contains( '\n' ) + 1 : 0;
}

Time SubtitleLine::autoDuration( const QString& t, unsigned charMillis, unsigned wordMillis, unsigned lineMillis )
{
	QString text( simplifyWhiteSpaces( t ) );
	if ( ! text.length() )
		return 0;

	unsigned chars = text.length();
	unsigned lines = text.contains( '\n' ) + 1;
	unsigned words = text.contains( ' ' ) + lines;

	return chars*charMillis + words*wordMillis + lines*lineMillis;
}

Time SubtitleLine::autoDuration( unsigned charMillis, unsigned wordMillis, unsigned lineMillis )
{
	return autoDuration( m_text, charMillis, wordMillis, lineMillis );
}

void SubtitleLine::setTimes( const Time& showTime, const Time& hideTime )
{
	if ( m_showTime != showTime || m_hideTime != hideTime )
		processAction( new SetLineTimesAction( *this, showTime, hideTime, i18n( "Set Line Times" ) ) );
}

void SubtitleLine::shiftTimes( int timeMillis )
{
	if ( timeMillis )
		processAction(
			new SetLineTimesAction(
				*this,
				Time( m_showTime.timeMillis() + timeMillis ),
				Time( m_hideTime.timeMillis() + timeMillis ),
				i18n( "Shift Line Times" )
			)
		);
}


/// STYLES

int SubtitleLine::styleFlags() const
{
	return m_styleFlags;
}

void SubtitleLine::setStyleFlags( int styleFlags )
{
	if ( m_styleFlags != styleFlags )
		processAction( new SetLineStyleFlagsAction( *this, styleFlags ) );
}

void SubtitleLine::setStyleFlags( int styleFlags, bool value )
{
	int newStyleFlags = m_styleFlags;

	for ( int styleFlag = 0x1, styleID = 0; styleID < StyleSIZE; ++styleID, styleFlag = styleFlag << 1 )
	{
		if ( styleFlags & styleFlag )
		{
			if ( value )
				newStyleFlags = newStyleFlags | styleFlag;
			else
				newStyleFlags = newStyleFlags & ~styleFlag;
		}
	}

	setStyleFlags( newStyleFlags );
}



/// MARKS

bool SubtitleLine::marked() const
{
	return m_marked;
}

void SubtitleLine::setMarked( bool marked )
{
	if ( m_marked != marked )
		processAction( new ToggleLineMarkedAction( *this ) );
}








/// ERRORS

int SubtitleLine::errorFlags() const
{
	return m_errorFlags;
}

void SubtitleLine::setErrorFlags( int errorFlags )
{
	if ( m_errorFlags != errorFlags )
		processAction( new SetLineErrorsAction( *this, errorFlags ) );
}

void SubtitleLine::setErrorFlags( int errorFlags, bool value )
{
	int newErrorFlags = m_errorFlags;

	for ( int errorFlag = 0x1, errorID = 0; errorID < ErrorSIZE; ++errorID, errorFlag = errorFlag << 1 )
	{
		if ( errorFlags & errorFlag )
		{
			if ( value )
				newErrorFlags = newErrorFlags | errorFlag;
			else
				newErrorFlags = newErrorFlags & ~errorFlag;
		}
	}

	setErrorFlags( newErrorFlags );
}

bool SubtitleLine::checkEmptyTextError( bool update )
{
	static const QRegExp emptyTextRegExp( "^\\s*$" );

	bool error = m_text.isEmpty() || m_text.find( emptyTextRegExp ) != -1;

	if ( update )
		setErrorFlags( SubtitleLine::EmptyText, error );

	return error;
}

bool SubtitleLine::checkOverlappingTimesError( bool update)
{
	SubtitleLine* prevLine = this->prevLine();
	SubtitleLine* nextLine = this->nextLine();

	bool error = prevLine && prevLine->m_hideTime >= m_showTime;
	if ( ! error && nextLine && nextLine->m_showTime <= m_hideTime )
		error = true;

	if ( update )
		setErrorFlags( SubtitleLine::OverlappingTimes, error );

	return error;
}

bool SubtitleLine::checkTooLongDurationError( unsigned maxDurationMillis, bool update )
{
	bool error = (unsigned)durationTime().timeMillis() > maxDurationMillis;

	if ( update )
		setErrorFlags( SubtitleLine::TooLongDuration, error );

	return error;
}

bool SubtitleLine::checkTooShortDurationError( unsigned minDurationMillis, bool update )
{
	bool error = (unsigned)durationTime().timeMillis() < minDurationMillis;

	if ( update )
		setErrorFlags( SubtitleLine::TooShortDuration, error );

	return error;
}

bool SubtitleLine::checkTooLongDurationPerCharError( unsigned maxDurationPerCharMillis, bool update )
{
	bool error = durationTime().timeMillis()/characters() > maxDurationPerCharMillis;

	if ( update )
		setErrorFlags( SubtitleLine::TooLongDurationPerChar, error );

	return error;
}

bool SubtitleLine::checkTooShortDurationPerCharError( unsigned minDurationPerCharMillis, bool update )
{
	bool error = durationTime().timeMillis()/characters() < minDurationPerCharMillis;

	if ( update )
		setErrorFlags( SubtitleLine::TooShortDurationPerChar, error );

	return error;
}

bool SubtitleLine::checkTooManyCharsError( unsigned maxCharacters, bool update )
{
	bool error = characters() > maxCharacters;

	if ( update )
		setErrorFlags( SubtitleLine::TooManyChars, error );

	return error;
}

bool SubtitleLine::checkTooManyLinesError( unsigned maxLines, bool update )
{
	bool error = lines() > maxLines;

	if ( update )
		setErrorFlags( SubtitleLine::TooManyLines, error );

	return error;
}

bool SubtitleLine::checkUnneededSpacesError( bool update )
{
	static const QRegExp unneededSpaceRegExp( "(^\\s|\\s$|¿\\s|¡\\s|\\s\\s|\\s!|\\s\\?|\\s:|\\s;|\\s,|\\s\\.)" );

	bool error = m_text.find( unneededSpaceRegExp ) != -1;

	if ( update )
		setErrorFlags( SubtitleLine::UnneededSpaces, error );

	return error;
}

bool SubtitleLine::checkCapitalAfterEllipsisError( bool update )
{
	static const QRegExp capitalAfterEllipsisRegExp( "^\\s*\\.\\.\\.\\s*" );

	bool error = m_text.find( capitalAfterEllipsisRegExp ) != -1;
	if ( error )
	{
		QChar chr = m_text.at( capitalAfterEllipsisRegExp.matchedLength() );
		error = chr.isLetter() && chr == chr.upper();
	}

	if ( update )
		setErrorFlags( SubtitleLine::CapitalAfterEllipsis, error );

	return error;
}

bool SubtitleLine::checkUnneededDashError( bool update )
{
	static const QRegExp unneededDashRegExp( "(^|\n)\\s*-[^-]" );

	bool error = m_text.contains( unneededDashRegExp ) == 1;

	if ( update )
		setErrorFlags( SubtitleLine::UnneededDash, error );

	return error;
}

void SubtitleLine::processAction( Action* action )
{
	if ( m_subtitle )
		m_subtitle->processAction( action );
	else
	{
		action->redo();
		delete action;
	}
}




/// SUBTITLE LINE ACTION
/// ====================

SubtitleLineAction::SubtitleLineAction( SubtitleLine& line, bool dirty, const QString& description ):
	Action( description ),
	m_line( line ),
	m_dirty( dirty ) {}
SubtitleLineAction::~SubtitleLineAction() {}

void SubtitleLineAction::_undo()
{
	_redo();
}

void SubtitleLineAction::_preRedo()
{
	if ( m_dirty && m_line.m_subtitle )
		m_line.m_subtitle->m_dirty++;
}

void SubtitleLineAction::_preUndo()
{
	if ( m_dirty && m_line.m_subtitle )
		m_line.m_subtitle->m_dirty--;
}

void SubtitleLineAction::_emitUndoSignals()
{
	_emitRedoSignals();
}


/// SET LINE TEXT ACTION
/// ====================

SetLineTextAction::SetLineTextAction( SubtitleLine& line, const QString& text ):
	SubtitleLineAction( line, true, i18n( "Set Line Text" ) ),
	m_text( text ) {}
SetLineTextAction::~SetLineTextAction() {}

bool SetLineTextAction::mergeWithPrevious( Action* pa )
{
	SetLineTextAction* prevAction = _mergeWithPrevious<SetLineTextAction>( pa );
	if ( ! prevAction )
		return false;

	m_text = prevAction->m_text;
	return true;
}

void SetLineTextAction::_redo()
{
	QString aux = m_line.m_text;
	m_line.m_text = m_text;
	m_text = aux;
}

void SetLineTextAction::_emitRedoSignals()
{
	m_line.emit textChanged( m_line.m_text );
}


/// SET LINE SHOW TIME ACTION
/// =========================

SetLineShowTimeAction::SetLineShowTimeAction( SubtitleLine& line, const Time& showTime ):
	SubtitleLineAction( line, true, i18n( "Set Line Show Time" ) ),
	m_showTime( showTime ) {}
SetLineShowTimeAction::~SetLineShowTimeAction() {}

bool SetLineShowTimeAction::mergeWithPrevious( Action* pa )
{
	SetLineShowTimeAction* prevAction = _mergeWithPrevious<SetLineShowTimeAction>( pa );
	if ( ! prevAction )
		return false;

	m_showTime = prevAction->m_showTime;
	return true;
}

void SetLineShowTimeAction::_redo()
{
	Time aux = m_line.m_showTime;
	m_line.m_showTime = m_showTime;
	m_showTime = aux;
}

void SetLineShowTimeAction::_emitRedoSignals()
{
	m_line.emit showTimeChanged( m_line.m_showTime );
}


/// SET LINE HIDE TIME ACTION
/// =========================

SetLineHideTimeAction::SetLineHideTimeAction( SubtitleLine& line, const Time& hideTime ):
	SubtitleLineAction( line, true, i18n( "Set Line Hide Time" ) ),
	m_hideTime( hideTime ) {}
SetLineHideTimeAction::~SetLineHideTimeAction() {}

bool SetLineHideTimeAction::mergeWithPrevious( Action* pa )
{
	SetLineHideTimeAction* prevAction = _mergeWithPrevious<SetLineHideTimeAction>( pa );
	if ( ! prevAction )
		return false;

	m_hideTime = prevAction->m_hideTime;
	return true;
}

void SetLineHideTimeAction::_redo()
{
	Time aux = m_line.m_hideTime;
	m_line.m_hideTime = m_hideTime;
	m_hideTime = aux;
}

void SetLineHideTimeAction::_emitRedoSignals()
{
	m_line.emit hideTimeChanged( m_line.m_hideTime );
}


/// SET LINE TIMES ACTION
/// =====================

SetLineTimesAction::SetLineTimesAction( SubtitleLine& line, const Time& showTime, const Time& hideTime, QString description ):
	SubtitleLineAction( line, true, description ),
	m_showTime( showTime ),
	m_hideTime( hideTime ) {}
SetLineTimesAction::~SetLineTimesAction() {}

bool SetLineTimesAction::mergeWithPrevious( Action* pa )
{
	SetLineTimesAction* prevAction = _mergeWithPrevious<SetLineTimesAction>( pa );
	if ( prevAction )
	{
		m_showTime = prevAction->m_showTime;
		m_hideTime = prevAction->m_hideTime;
	}
	else
	{
		SetLineHideTimeAction* prevAction2 = _mergeWithPrevious<SetLineHideTimeAction>( pa );
		if ( prevAction2 )
			m_hideTime = prevAction2->m_hideTime;
		else
		{
			SetLineShowTimeAction* prevAction3 = _mergeWithPrevious<SetLineShowTimeAction>( pa );
		 	if ( prevAction3 )
				m_showTime = prevAction3->m_showTime;
			else
				return false;
		}
	}

	return true;
}

void SetLineTimesAction::_redo()
{
	if ( m_line.m_showTime != m_showTime )
	{
		Time aux = m_line.m_showTime;
		m_line.m_showTime = m_showTime;
		m_showTime = aux;
	}

	if ( m_line.m_hideTime != m_hideTime )
	{
		Time aux = m_line.m_hideTime;
		m_line.m_hideTime = m_hideTime;
		m_hideTime = aux;
	}
}

void SetLineTimesAction::_emitRedoSignals()
{
	if ( m_line.m_showTime != m_showTime )
		m_line.emit showTimeChanged( m_line.m_showTime );

	if ( m_line.m_hideTime != m_hideTime )
		m_line.emit hideTimeChanged( m_line.m_hideTime );
}


/// SET LINE STYLE FLAGS ACTION
/// ===========================

SetLineStyleFlagsAction::SetLineStyleFlagsAction( SubtitleLine& line, int styleFlags ):
	SubtitleLineAction( line, true, i18n( "Set Line Styles" ) ),
	m_styleFlags( styleFlags ) {}
SetLineStyleFlagsAction::~SetLineStyleFlagsAction() {}

bool SetLineStyleFlagsAction::mergeWithPrevious( Action* pa )
{
	SetLineStyleFlagsAction* prevAction = _mergeWithPrevious<SetLineStyleFlagsAction>( pa );
	if ( ! prevAction )
		return false;

	m_styleFlags = prevAction->m_styleFlags;
	return true;
}

void SetLineStyleFlagsAction::_redo()
{
	int aux = m_line.m_styleFlags;
	m_line.m_styleFlags = m_styleFlags;
	m_styleFlags = aux;
}

void SetLineStyleFlagsAction::_emitRedoSignals()
{
	m_line.emit styleFlagsChanged( m_line.m_styleFlags );
}




/// TOGGLE LINE MARKED ACTION
/// =========================

ToggleLineMarkedAction::ToggleLineMarkedAction( SubtitleLine& line ):
	SubtitleLineAction( line, false, i18n( "Toggle Line Mark" ) ) {}
ToggleLineMarkedAction::~ToggleLineMarkedAction() {}

void ToggleLineMarkedAction::_redo()
{
	m_line.m_marked = ! m_line.m_marked;
}

void ToggleLineMarkedAction::_emitRedoSignals()
{
	m_line.emit markChanged( m_line.m_marked );
}


/// SET LINE ERRORS ACTION
/// ======================

SetLineErrorsAction::SetLineErrorsAction( SubtitleLine& line, int errorFlags ):
	SubtitleLineAction( line, false, i18n( "Set Line Errors" ) ),
	m_errorFlags( errorFlags ) {}
SetLineErrorsAction::~SetLineErrorsAction() {}

bool SetLineErrorsAction::mergeWithPrevious( Action* pa )
{
	SetLineErrorsAction* prevAction = _mergeWithPrevious<SetLineErrorsAction>( pa );
	if ( ! prevAction )
		return false;

	m_errorFlags = prevAction->m_errorFlags;
	return true;
}

void SetLineErrorsAction::_redo()
{
	int aux = m_line.m_errorFlags;
	m_line.m_errorFlags = m_errorFlags;
	m_errorFlags = aux;
}

void SetLineErrorsAction::_emitRedoSignals()
{
	m_line.emit errorFlagsChanged( m_line.m_errorFlags );
}
