#ifndef SUBTITLELINE_H
#define SUBTITLELINE_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include "action.h"
#include "sctime.h"

#include <qobject.h>
#include <qstring.h>

namespace SubtitleComposer
{
	class Subtitle;

	class SubtitleLine : public QObject
	{
		Q_OBJECT

		friend class SubtitleAction;
		friend class SubtitleLineAction;
		friend class SetLineTextAction;
		friend class SetLineShowTimeAction;
		friend class SetLineHideTimeAction;
		friend class SetLineTimesAction;
		friend class SetLineStyleFlagsAction;
		friend class SetLineErrorsAction;
		friend class ToggleLineMarkedAction;

		public:

			typedef enum {
				ItalicID = 0,
				BoldID,
				UnderlineID,
				StyleSIZE,
				StyleUNKNOWN
			} StyleID;

			typedef enum {
				Italic = 0x1 << ItalicID,
				Bold = 0x1 << BoldID,
				Underline = 0x1 << UnderlineID,
				AllStyles = 0xFFFFFFFF }
			StyleFlag;

			typedef enum {
				EmptyTextID = 0,			// Empty text
				TooManyCharsID,				// Too many characters
				TooManyLinesID,				// Too many line breaks
				UnneededSpacesID,			// Unnecessary spaces
				UnneededDashID,				// Unnecessary dash (-)
				CapitalAfterEllipsisID,		// Capital letter after ellipsis
				TooShortDurationID,			// Too short duration
				TooLongDurationID,			// Too long duration
				TooShortDurationPerCharID,	// Too short duration per character
				TooLongDurationPerCharID,	// Too long duration per character
				OverlappingTimesID,			// Overlapping times
				ErrorSIZE,
				ErrorUNKNOWN
			} ErrorID;

			typedef enum {
				EmptyText = 0x1 << EmptyTextID,
				TooManyChars = 0x1 << TooManyCharsID,
				TooManyLines = 0x1 << TooManyLinesID,
				UnneededSpaces = 0x1 << UnneededSpacesID,
				UnneededDash = 0x1 << UnneededDashID,
				CapitalAfterEllipsis = 0x1 << CapitalAfterEllipsisID,
				TooShortDuration = 0x1 << TooShortDurationID,
				TooLongDuration = 0x1 << TooLongDurationID,
				TooShortDurationPerChar = 0x1 << TooShortDurationPerCharID,
				TooLongDurationPerChar = 0x1 << TooLongDurationPerCharID,
				OverlappingTimes = 0x1 << OverlappingTimesID,
				AllErrors = 0xFFFFFFFF,
			} ErrorFlag;

			static ErrorFlag errorFlag( ErrorID id );
			static ErrorID errorID( ErrorFlag flag );

			SubtitleLine( const QString& text=QString::null );
			SubtitleLine( const QString& text, const Time& showTime, const Time& hideTime, int styles=0 );
			SubtitleLine( const SubtitleLine& line );
			SubtitleLine& operator=( const SubtitleLine& line );

			unsigned number() const;
			int index() const;

			Subtitle* subtitle();
			const Subtitle* subtitle() const;

			SubtitleLine* prevLine();
			SubtitleLine* nextLine();

			QString text() const;
			void setText( const QString& text );

			static QString fixPunctuation( const QString& text, bool spaces, bool quotes, bool engI, bool ellipsis, bool* cont );
			static QString titleCase( const QString& text, bool lowerFirst );
			static QString sentenceCase( const QString& text, bool lowerFirst, bool* continues=0 );

			void adjustText( unsigned minLengthForBreak );
			void unbreakText();

			static QString simplifyWhiteSpaces( QString text );
			void simplifySpaces();

			Time showTime() const;
			void setShowTime( const Time& showTime );

			Time hideTime() const;
			void setHideTime( const Time& hideTime );

			Time durationTime() const;
			void setDurationTime( const Time& durationTime );

			unsigned characters();
			unsigned words();
			unsigned lines();

			static Time autoDuration( const QString& text, unsigned charMillis, unsigned wordMillis,unsigned lineMillis );
			Time autoDuration( unsigned charMillis, unsigned wordMillis, unsigned lineMillis );

			void setTimes( const Time& showTime, const Time& hideTime );

			void shiftTimes( int timeMillis );

			int styleFlags() const;
			void setStyleFlags( int styleFlags );
			void setStyleFlags( int styleFlags, bool value );

			bool marked() const;
			void setMarked( bool marked );

			int errorFlags() const;
			void setErrorFlags( int errorFlags );
			void setErrorFlags( int errorFlags, bool value );
			bool checkEmptyTextError( bool update=true );
			bool checkOverlappingTimesError( bool update=true );
			bool checkTooLongDurationError( unsigned maxDurationMillis, bool update=true );
			bool checkTooShortDurationError( unsigned minDurationMillis, bool update=true );
			bool checkTooLongDurationPerCharError( unsigned maxDurationPerCharMillis, bool update=true );
			bool checkTooShortDurationPerCharError( unsigned minDurationPerCharMillis, bool update=true );
			bool checkTooManyCharsError( unsigned maxCharacters, bool update=true );
			bool checkTooManyLinesError( unsigned maxLines, bool update=true );
			bool checkUnneededSpacesError( bool update=true );
			bool checkCapitalAfterEllipsisError( bool update=true );
			bool checkUnneededDashError( bool update=true );

		signals:

			void textChanged( const QString& text );
			void showTimeChanged( const Time& showTime );
			void hideTimeChanged( const Time& hideTime );
			void styleFlagsChanged( int styleFlags );
			void errorFlagsChanged( int errorFlags );
			void markChanged( bool marked );

		private:

			void processAction( Action* action );

		private:

			Subtitle* m_subtitle;
			QString m_text;
			Time m_showTime;
			Time m_hideTime;
			int m_styleFlags;
			int m_errorFlags;
			bool m_marked;
	};


	class SubtitleLineAction : public Action
	{
		public:

			friend class SubtitleLine;

			SubtitleLineAction( SubtitleLine& line, bool dirty, const QString& description=QString::null );
			virtual ~SubtitleLineAction();

		protected:

			virtual void _undo();

			virtual void _preRedo();
			virtual void _preUndo();

			virtual void _emitUndoSignals();

			template <class T>
			T* _mergeWithPrevious( Action* prevAction )
			{
				T* castedPrevAction = tryCastTo<T>( prevAction );
				return castedPrevAction ? (&castedPrevAction->m_line == &m_line ? castedPrevAction : 0) : 0;
			}

		protected:

			SubtitleLine& m_line;
			bool m_dirty;
	};

	class SetLineTextAction : public SubtitleLineAction
	{
		public:

			SetLineTextAction( SubtitleLine& line, const QString& text );
			virtual ~SetLineTextAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _emitRedoSignals();

		private:

			QString m_text;
	};

	class SetLineShowTimeAction : public SubtitleLineAction
	{
		friend class SetLineTimesAction;

		public:

			SetLineShowTimeAction( SubtitleLine& line, const Time& showTime );
			virtual ~SetLineShowTimeAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _emitRedoSignals();

		private:

			Time m_showTime;
	};

	class SetLineHideTimeAction : public SubtitleLineAction
	{
		friend class SetLineTimesAction;

		public:

			SetLineHideTimeAction( SubtitleLine& line, const Time& hideTime );
			virtual ~SetLineHideTimeAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _emitRedoSignals();

		private:

			Time m_hideTime;
	};

	class SetLineTimesAction : public SubtitleLineAction
	{
		public:

			SetLineTimesAction( SubtitleLine& line, const Time& showTime, const Time& hideTime, QString description=QString::null );
			virtual ~SetLineTimesAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _emitRedoSignals();

		private:

			typedef enum { ShowTime = 0x1, HideTime = 0x2 } SignalFlags;

			Time m_showTime;
			Time m_hideTime;
	};

	class SetLineStyleFlagsAction : public SubtitleLineAction
	{
		public:

			SetLineStyleFlagsAction( SubtitleLine& line, int styleFlags );
			virtual ~SetLineStyleFlagsAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _emitRedoSignals();

		private:

			int m_styleFlags;
	};

	class SetLineErrorsAction : public SubtitleLineAction
	{
		public:

			SetLineErrorsAction( SubtitleLine& line, int errorFlags );
			virtual ~SetLineErrorsAction();

		protected:

			virtual bool mergeWithPrevious( Action* prevAction );

			virtual void _redo();
			virtual void _emitRedoSignals();

		private:

			int m_errorFlags;
	};

	class ToggleLineMarkedAction : public SubtitleLineAction
	{
		public:

			ToggleLineMarkedAction( SubtitleLine& line );
			virtual ~ToggleLineMarkedAction();

		protected:

			virtual void _redo();
			virtual void _emitRedoSignals();
	};

}

#endif
