#ifndef SUBSTATIONALPHAINPUTFORMAT_H
#define SUBSTATIONALPHAINPUTFORMAT_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <inputformat.h>
#include <qregexp.h>

namespace SubtitleComposer
{
	class SubStationAlphaInputFormat : public InputFormat
	{
		friend class FormatsManager;
		friend class AdvancedSubStationAlphaInputFormat;

		public:
			~SubStationAlphaInputFormat() {}

			virtual bool parseSubtitles( Subtitle& subtitle, const QString& subtitleText ) const
			{
				QString subtitleText2( subtitleText );
				// we can't parse it otherwise because of the commands can have parameters
				// separated by comas which break or generated regular expressions
// 				subtitleText2.replace( m_commandsRegExp, "" );

				int eventsStart = subtitleText2.find( m_eventsRegExp );

				if ( eventsStart == -1 ||
					 ! subtitleText2.contains( "[Script Info]" ) ||
					 ! subtitleText2.contains( m_styless ) )
					return false;

				// EXAMPLE:
				// Format: Marked, Start, End, Style, Name, MarginL, MarginR, MarginV, Effect, Text
				if ( m_formatRegExp.search( subtitleText2, eventsStart ) == -1 )
					return false;

				QStringList formatTokens = QStringList::split( ",", m_formatRegExp.cap( 1 ) );
				if ( formatTokens.size() < 3 )
					return false;

				for ( QStringList::Iterator it = formatTokens.begin(); it != formatTokens.end(); ++it )
					(*it) = (*it).stripWhiteSpace().lower();

				int startTokenPos = formatTokens.findIndex( "start" ) + 1;
				int endTokenPos = formatTokens.findIndex( "end" ) + 1;
				int textTokenPos = formatTokens.findIndex( "text" ) + 1;

				if ( startTokenPos < 1 || endTokenPos < 1 || textTokenPos < 1 )
					return false;

				subtitle.setFramesPerSecond( Subtitle::defaultFramesPerSecond() );

				QString eventPattern( "Dialogue: ([^,\n]*)" );
				for ( unsigned idx = 2; idx < formatTokens.size(); ++idx )
					eventPattern += ",([^,\n]*)";
				QRegExp eventRegExp( eventPattern + ",([^\n]*)\n" );

				int lineStart = m_formatRegExp.pos() + m_formatRegExp.matchedLength();
				while ( eventRegExp.search( subtitleText2, lineStart ) != -1 )
				{
					// EXAMPLE
					// Dialogue: Marked=0,0:00:01.18,0:00:06.85,DefaultVCD, NTP,0000,0000,0000,,{\pos(400,570)}Like pity on nobody

					if ( m_timeRegExp.search( eventRegExp.cap( startTokenPos ) ) == -1 )
						continue;

					Time showTime(
						m_timeRegExp.cap( 1 ).toInt(),
						m_timeRegExp.cap( 2 ).toInt(),
						m_timeRegExp.cap( 3 ).toInt(),
						m_timeRegExp.cap( 4 ).toInt() * 10
					);

					if ( m_timeRegExp.search( eventRegExp.cap( endTokenPos ) ) == -1 )
						continue;

					Time hideTime(
						m_timeRegExp.cap( 1 ).toInt(),
						m_timeRegExp.cap( 2 ).toInt(),
						m_timeRegExp.cap( 3 ).toInt(),
						m_timeRegExp.cap( 4 ).toInt() * 10
					);

					QString text( eventRegExp.cap( textTokenPos ) );

					int style = 0;
					if ( text.contains( m_boldRegExp ) )
						style |= SubtitleLine::Bold;
					if ( text.contains( m_italicRegExp ) )
						style |= SubtitleLine::Italic;

					text.replace( "\\N", "\n" );
					text.replace( "\\n", "\n" );
					text.replace( m_boldRegExp, "" );
					text.replace( m_italicRegExp, "" );
					text.replace( m_commandsRegExp, "" );

					subtitle.insertLine( new SubtitleLine( text.stripWhiteSpace(), showTime, hideTime, style ) );

					lineStart += eventRegExp.matchedLength();
				}

				return subtitle.linesCount() > 0;
			}

		protected:

			SubStationAlphaInputFormat():
				InputFormat( "SubStation Alpha", "ssa" ),
				m_styless( "[V4 Styles]" ),
				m_eventsRegExp( "\n\\[Events\\]\n" ),
				m_formatRegExp( "\nFormat:([^\n]*)\n" ),
				m_timeRegExp( "([0-9]):([0-5][0-9]):([0-5][0-9]).([0-9][0-9])" ),
				m_boldRegExp( "\\{\\\\b[10]\\}" ),
				m_italicRegExp( "\\{\\\\i[10]\\}" ),
				m_commandsRegExp( "\\{\\\\[a-zA-Z0-9]+\\( *[a-zA-Z0-9]+ *(, *[a-zA-Z0-9]+){0,} *\\)\\}" )
			{
			}

			SubStationAlphaInputFormat( const QString& name, const QStringList& extensions, const QString& styles ):
				InputFormat( name, extensions ),
				m_styless( styles ),
				m_eventsRegExp( "\n\\[Events\\]\n" ),
				m_formatRegExp( "\nFormat:([^\n]*)\n" ),
				m_timeRegExp( "([0-9]):([0-5][0-9]):([0-5][0-9]).([0-9][0-9])" ),
				m_boldRegExp( "\\{\\\\b[10]\\}" ),
				m_italicRegExp( "\\{\\\\i[10]\\}" ),
				m_commandsRegExp( "\\{\\\\[a-zA-Z0-9]+\\( *[a-zA-Z0-9]+ *(, *[a-zA-Z0-9]+){0,} *\\)\\}" )
			{
			}

			const QString m_styless;
			mutable QRegExp m_eventsRegExp;
			mutable QRegExp m_formatRegExp;
			mutable QRegExp m_timeRegExp;
			mutable QRegExp m_boldRegExp;
			mutable QRegExp m_italicRegExp;
			mutable QRegExp m_commandsRegExp;
	};

	class AdvancedSubStationAlphaInputFormat : public SubStationAlphaInputFormat
	{
		friend class FormatsManager;

		protected:

			AdvancedSubStationAlphaInputFormat():
				SubStationAlphaInputFormat( "Advanced SubStation Alpha", "ass", "[V4+ Styles]" )
			{
			}
	};

}

#endif

