/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actions.h"

#include <kapplication.h>
#include <kconfig.h>
#include <klocale.h>
#include <kstandarddirs.h>

#include <qfile.h>

#include <kpopupmenu.h>
#include <ktoolbar.h>
#include <ktoolbarbutton.h>
#include <qtoolbutton.h>
#include <qwhatsthis.h>

using namespace SubtitleComposer;


SubtitleActionWrappersManager::SubtitleActionWrappersManager( KActionCollection* actionCollection ):
	m_actionCollection( actionCollection ),
	m_subtitleActions()
{
	m_subtitleActions.setAutoDelete( false );
}

SubtitleActionWrappersManager& SubtitleActionWrappersManager::instance()
{
	static SubtitleActionWrappersManager instance;
	return instance;
}

KActionCollection* SubtitleActionWrappersManager::actionCollection()
{
	return m_actionCollection;
}

void SubtitleActionWrappersManager::setActionCollection( KActionCollection* actionCollection )
{
	m_actionCollection = actionCollection;
}

void SubtitleActionWrappersManager::setSubtitle( Subtitle* subtitle )
{
	for ( QPtrListIterator<SubtitleActionWrapper> it( m_subtitleActions ); it.current(); ++it )
		it.current()->setSubtitle( subtitle );
}

void SubtitleActionWrappersManager::setViewState( int itemsCount, bool hasSelection )
{
	for ( QPtrListIterator<SubtitleActionWrapper> it( m_subtitleActions ); it.current(); ++it )
		it.current()->setViewState( itemsCount, hasSelection );
}

void SubtitleActionWrappersManager::addAction( SubtitleActionWrapper* subtitleAction )
{
	m_subtitleActions.append( subtitleAction );
}








SubtitleActionWrapper::SubtitleActionWrapper( KAction* action, EnableMode enableMode ):
	m_action( action ),
	m_enableMode( enableMode ),
	m_actionEnabled( action->isEnabled() ),
	m_subViewEnabled( false ),
	m_updateActionEnabled( true )
{
	connect( action, SIGNAL( enabled(bool) ), this, SLOT( onActionEnabled(bool) ) );

	updateEnabled();
}

SubtitleActionWrapper::EnableMode SubtitleActionWrapper::enableMode()
{
	return m_enableMode;
}

void SubtitleActionWrapper::setEnableMode( SubtitleActionWrapper::EnableMode enableMode )
{
	m_enableMode = enableMode;
}

void SubtitleActionWrapper::setSubtitle( Subtitle* subtitle )
{
	if ( subtitle && m_enableMode == Opened )
	{
		m_subViewEnabled = true;
		updateEnabled();
	}
	else if ( ! subtitle )
	{
		m_subViewEnabled = false;
		updateEnabled();
	}
}

void SubtitleActionWrapper::setViewState( int itemsCount, bool hasSelection )
{
	switch ( m_enableMode )
	{
		case Opened:
			m_subViewEnabled = true;
			break;
		case HasLine:
			m_subViewEnabled = itemsCount > 0;
			break;
		case HasMultipleLines:
			m_subViewEnabled = itemsCount > 1;
			break;
		case HasSelectedLine:
			m_subViewEnabled = itemsCount > 0 && hasSelection;
			break;
		default:
			return;
	}

	updateEnabled();
}

bool SubtitleActionWrapper::isEnabled()
{
	return m_actionEnabled && m_subViewEnabled;
}

void SubtitleActionWrapper::setEnabled( bool enabled )
{
	m_actionEnabled = enabled;
	updateEnabled();
}

void SubtitleActionWrapper::updateEnabled()
{
	m_updateActionEnabled = false;

	m_action->setEnabled( m_actionEnabled && m_subViewEnabled );

	m_updateActionEnabled = true;
}

void SubtitleActionWrapper::onActionEnabled( bool enabled )
{
	if ( m_updateActionEnabled )
	{
		m_actionEnabled = enabled;
		updateEnabled();
	}
}




KActionExt::KActionExt( KActionCollection* parent, const char* name ):
	KAction( parent, name )
{
}

int KActionExt::plug( QWidget* widget, int index )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
		return KAction::plug( widget, index );

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	toolButton->setTextLabel( text() );
	toolButton->setIconSet( iconSet() );

	connect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
	connect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );

	return 0;
}

void KActionExt::unplug( QWidget* widget )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
	{
		KAction::unplug( widget );
		return;
	}

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	disconnect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
	disconnect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );
}




KToggleActionExt::KToggleActionExt( KActionCollection* parent, const char* name ):
	KToggleAction( parent, name ),
	m_offText( QString::null ),
	m_onText( QString::null )
{
}

void KToggleActionExt::setText( const QString& text )
{
	setText( text, text );
}

void KToggleActionExt::setText( const QString& offText, const QString& onText )
{
	m_offText = offText;
	m_onText = onText;

	KToggleAction::setText( isChecked() ? m_onText : m_offText );

	emit signalSetTextLabel( KToggleAction::text() );
}

int KToggleActionExt::plug( QWidget* widget, int index )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
		return KToggleAction::plug( widget, index );

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	toolButton->setToggleButton( true );
	toolButton->setTextLabel( KToggleAction::text() );
	toolButton->setOn( isChecked() );
	toolButton->setIconSet( iconSet() );

	connect( this, SIGNAL(toggled(bool)), toolButton, SLOT(setOn(bool)) );
	connect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
	connect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );

	connect( this, SIGNAL(signalSetTextLabel(const QString&)), toolButton, SLOT(setTextLabel(const QString&)) );

	return 0;
}

void KToggleActionExt::unplug( QWidget* widget )
{
	if ( ! ::qt_cast<QToolButton*>( widget ) )
	{
		KToggleAction::unplug( widget );
		return;
	}

	QToolButton* toolButton = static_cast<QToolButton*>( widget );

	disconnect( this, SIGNAL(toggled(bool)), toolButton, SLOT(setOn(bool)) );
	disconnect( this, SIGNAL(enabled(bool)), toolButton, SLOT(setEnabled(bool)) );
	disconnect( toolButton, SIGNAL(clicked()), this, SLOT(slotActivated()) );

	disconnect( this, SIGNAL(signalSetTextLabel(const QString&)), toolButton, SLOT(setTextLabel(const QString&)) );
}

void KToggleActionExt::setChecked( bool checked )
{
	if ( isChecked() != checked )
	{
		KToggleAction::setChecked( checked );
		// the signal is not emited when it should by KToggleAction::setChecked( checked )
		// so we take the issue in our hands...
		emit toggled( checked );

		KToggleAction::setText( isChecked() ? m_onText : m_offText );
		emit signalSetTextLabel( KToggleAction::text() );
	}
}


KRecentFilesActionExt::KRecentFilesActionExt( QObject* parent, const char* name ):
	KSelectAction( parent, name ),
	m_maxItems( 10 ),
	m_ignoreURLQueryItems( false ),
	m_popup( new KPopupMenu )
{
	connect( this, SIGNAL(activated(const QString&)), this, SLOT(onItemActivated(const QString&)) );
	connect( m_popup, SIGNAL(activated(int)), this, SLOT(onItemActivated(int)) );
	connect( m_popup, SIGNAL(aboutToShow()), this, SLOT(onPopupMenuAboutToShow()) );

	setMenuAccelsEnabled( false );
}

KRecentFilesActionExt::~KRecentFilesActionExt()
{
	delete m_popup;
}

bool KRecentFilesActionExt::hasItems() const
{
	return m_urls.count();
}

void KRecentFilesActionExt::setItems( QStringList& items )
{
	if ( ! items.isEmpty() )
	{
		items.append( QString::null );
		items.append( i18n( "Clear History" ) );
	}

	KSelectAction::setItems( items );
}

QStringList KRecentFilesActionExt::buildItems( const KURL::List& urls )
{
	QStringList items;

	QString file;
	unsigned idx = 0;
	for ( KURL::List::ConstIterator it = urls.begin(); it != urls.end() && idx < m_maxItems; ++it, ++idx )
	{
		file = m_ignoreURLQueryItems && (*it).isLocalFile() ? (*it).path() : (*it).pathOrURL();
		items.append( (*it).fileName() + " [" + file + "]" );
	}

	return items;
}

void KRecentFilesActionExt::buildAndSetItems( const KURL::List& urls )
{
	QStringList items( buildItems( urls ) );
	setItems( items );
}

const KURL::List& KRecentFilesActionExt::urls() const
{
	return m_urls;
}

void KRecentFilesActionExt::setURLs( const KURL::List& urls )
{
	if ( &m_urls != &urls )
	{
		m_urls.clear();
		unsigned idx = 0;
		for ( KURL::List::ConstIterator it = urls.begin(); it != urls.end() && idx < m_maxItems; ++it, ++idx )
			m_urls.append( (*it) );
	}

	buildAndSetItems( m_urls );
}

uint KRecentFilesActionExt::maxItems() const
{
	return m_maxItems;
}

bool KRecentFilesActionExt::ignoreURLQueryItems() const
{
	return m_ignoreURLQueryItems;
}

void KRecentFilesActionExt::setIgnoreURLQueryItems( bool value )
{
	if ( m_ignoreURLQueryItems != value )
	{
		m_ignoreURLQueryItems = value;
		buildAndSetItems( m_urls );
	}
}

void KRecentFilesActionExt::setMaxItems( uint maxItems )
{
	if ( m_maxItems != maxItems && maxItems > 0 )
	{
		m_maxItems = maxItems;

		if ( m_urls.count() <= maxItems )
			return;

		m_urls.erase( m_urls.begin() += maxItems, m_urls.end() );

		buildAndSetItems( m_urls );
	}
}

void KRecentFilesActionExt::add( const KURL& url )
{
	if ( url.isLocalFile() && ! KGlobal::dirs()->relativeLocation( "tmp", url.path() ).startsWith( "/" ) )
		return;

	const QString file = m_ignoreURLQueryItems && url.isLocalFile() ? url.path() : url.pathOrURL();

	KURL refURL( url );
	if ( m_ignoreURLQueryItems )
		refURL.setQuery( "" );

	bool remove;
	unsigned idx = 0;
	for ( KURL::List::Iterator it = m_urls.begin(); it != m_urls.end(); ++it, ++idx )
	{
		if ( m_ignoreURLQueryItems )
		{
			KURL itURL( (*it) );
			itURL.setQuery( "" );
			remove = itURL == refURL;
		}
		else
			remove = (*it) == refURL;

		if ( remove )
		{
			m_urls.remove( it );
			break;
		}
	}

	if ( m_urls.count() >= m_maxItems )
		m_urls.pop_back();

	m_urls.prepend( url );

	buildAndSetItems( m_urls );
}

void KRecentFilesActionExt::remove( const KURL& url )
{
	KURL refURL( url );
	if ( m_ignoreURLQueryItems )
		refURL.setQuery( "" );

	bool remove;
	unsigned idx = 0;
	for ( KURL::List::Iterator it = m_urls.begin(); it != m_urls.end(); ++it, ++idx )
	{
		if ( m_ignoreURLQueryItems )
		{
			KURL itURL( (*it) );
			itURL.setQuery( "" );
			remove = itURL == refURL;
		}
		else
			remove = (*it) == refURL;

		if ( remove )
		{
			QStringList items = KSelectAction::items();
			m_urls.remove( it );

			buildAndSetItems( m_urls );
			return;
		}
	}
}

void KRecentFilesActionExt::clear()
{
	m_urls.clear();

	QStringList list;
	setItems( list );
}

void KRecentFilesActionExt::loadEntries( KConfig* config, const QString& groupName )
{
	KConfigGroupSaver saver( config, groupName.isEmpty() ?  "RecentFiles" : groupName );

	m_urls.clear();

	unsigned itemsCount = config->readNumEntry( "Files", m_maxItems );
	if ( itemsCount > m_maxItems )
		itemsCount = m_maxItems;

	QString key( "File%1" );
	for ( unsigned idx = 1 ; idx <= itemsCount ; idx++ )
	{
		QString value = config->readPathEntry( key.arg( idx ) );
		if ( ! value.isEmpty() )
		{
			KURL url = KURL::fromPathOrURL( value );

			// Don't restore if file doesn't exist anymore
			if ( url.isLocalFile() && ! QFile::exists( url.path() ) )
			{
				itemsCount++;
				continue;
			}

			m_urls.append( url );
		}
	}

	buildAndSetItems( m_urls );
}

void KRecentFilesActionExt::saveEntries( KConfig* config, const QString& group )
{
	QString oldGroupName = config->group();
	QString groupName( group.isEmpty() ? "RecentFiles" : group );

	config->deleteGroup( groupName, true );
	config->setGroup( groupName );

	int idx = 1;
	QString key( "File%1" );
	for ( KURL::List::Iterator it = m_urls.begin(); it != m_urls.end(); ++it, ++idx )
		config->writePathEntry( key.arg( idx ), (*it).pathOrURL() );
	config->writeEntry( "Files", m_urls.count() );

	config->setGroup( oldGroupName );
}

void KRecentFilesActionExt::onItemActivated( const QString& text )
{
	// NOTE: emit a copy of the URL since the slot where it is connected might call add(),
	// remove() or clear() methods (potentially destroying the relevant m_urls item).

	int itemsCount = m_urls.count();
	int idx = KSelectAction::items().findIndex( text );
	if ( idx == itemsCount + 1 )
		clear();
	else if ( idx >= 0 && idx < itemsCount )
		emit urlSelected( KURL( m_urls[idx] ) );
}

void KRecentFilesActionExt::onItemActivated( int id )
{
	// NOTE: emit a copy of the URL since the slot where it is connected might call add(),
	// remove() or clear() methods (potentially destroying the relevant m_urls item).

	int itemsCount = m_urls.count();
	if ( id == itemsCount )
		clear();
	else if ( id >= 0 && id < itemsCount )
		emit urlSelected( KURL( m_urls[id] ) );
}

void KRecentFilesActionExt::onPopupMenuAboutToShow()
{
	m_popup->clear();

	int id = 0;
	QStringList items = KSelectAction::items();
	for ( QStringList::Iterator it = items.begin(); it != items.end(); ++it )
	{
		if ( (*it).isEmpty() )
			m_popup->insertSeparator();
		else
			m_popup->insertItem( *it, id++ );
	}
}

int KRecentFilesActionExt::plug( QWidget* widget, int index )
{
	if ( kapp && ! kapp->authorizeKAction( name() ) )
		return -1;

	if ( ::qt_cast<KToolBar*>( widget ) )
	{
		KToolBar* toolBar = (KToolBar*)widget;

		int id = KAction::getToolButtonID();

		toolBar->insertButton(
			icon(),
			id, SIGNAL( clicked() ),
			this, SLOT( slotClicked() ),
			isEnabled(),
			plainText(),
			index,
			m_parentCollection ?
				m_parentCollection->instance() :
				KGlobal::instance()
		);

		addContainer( toolBar, id );

		connect( toolBar, SIGNAL( destroyed() ), this, SLOT( slotDestroyed() ) );

		toolBar->setDelayedPopup( id, m_popup, true );

		if ( ! whatsThis().isEmpty() )
			QWhatsThis::add( toolBar->getButton( id ), whatsThisWithIcon() );

		return containerCount() - 1;
	}

	return KSelectAction::plug( widget, index );
}

void KRecentFilesActionExt::slotClicked()
{
	KAction::slotActivated();
}

void KRecentFilesActionExt::slotActivated()
{
	emit activated( currentItem() );
	emit activated( currentText() );
}

void KRecentFilesActionExt::virtual_hook( int id, void* data )
{
	KSelectAction::virtual_hook( id, data );
}

