/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actionsdialogs.h"
#include "filedialogs.h"
#include "errorsviewitem.h"
#include <playersmanager.h>
#include <commondefs.h>

#include <kapplication.h>
#include <kaboutdata.h>
#include <kiconloader.h>
#include <kcharsets.h>

#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <timeedit.h>
#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <kcombobox.h>
#include <knuminput.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kurlcompletion.h>

using namespace SubtitleComposer;

ActionDialog::ActionDialog( const QString& title, QWidget* parent, const char* name ):
	KDialogBase( parent, name, true, title, KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok )
{
	m_mainWidget = new QWidget( this );
	m_mainLayout = new QGridLayout( m_mainWidget );
	m_mainLayout->setAlignment( Qt::AlignTop );
	m_mainLayout->setMargin( 0 );
	m_mainLayout->setSpacing( 5 );

	setMainWidget( m_mainWidget );

	// attempt at showing the full action name on the title...
	// it would work better if we could get the fontMetrics used by kwin
	m_mainWidget->setMinimumWidth( fontMetrics().width( title + kapp->aboutData()->appName() ) + 120 );
}

QButtonGroup* ActionDialog::createButtonGroup( const QString& title, bool addToLayout )
{
	QButtonGroup* buttonGroup = new QButtonGroup( m_mainWidget );
	buttonGroup->setTitle( title );
	buttonGroup->setColumnLayout( 0, Qt::Vertical );
	buttonGroup->layout()->setSpacing( 5 );

	if ( addToLayout )
		m_mainLayout->addWidget( buttonGroup, m_mainLayout->numRows(), 0 );

	return buttonGroup;
}

QGroupBox* ActionDialog::createGroupBox( const QString& title, bool addToLayout )
{
	QGroupBox* groupBox = new QGroupBox( m_mainWidget );
	groupBox->setTitle( title );
	groupBox->setColumnLayout( 0, Qt::Vertical );
	groupBox->layout()->setSpacing( 5 );

	if ( addToLayout )
		m_mainLayout->addWidget( groupBox, m_mainLayout->numRows(), 0 );

	return groupBox;
}

QGridLayout* ActionDialog::createLayout( QGroupBox* groupBox )
{
	QGridLayout* gridLayout = new QGridLayout( groupBox->layout() );
	gridLayout->setAlignment( Qt::AlignTop );
	return gridLayout;
}


SelectableTargetActionDialogBase::SelectableTargetActionDialogBase( const QString& title, QWidget* parent, const char* name ):
	ActionDialog( title, parent, name ),
	m_targetButtonGroup( 0 ),
	m_targetButtonGroupLayout( 0 ),
	m_selectedLinesRadioButton( 0 ),
	m_allLinesRadioButton( 0 ),
	m_selectedLinesOnlyMode( false ),
	m_selectedLinesOldValue( false )
{
}

QButtonGroup* SelectableTargetActionDialogBase::createTargetButtonGroup( const QString& title, bool addToLayout )
{
	m_targetButtonGroup = createButtonGroup( title, addToLayout );

	m_allLinesRadioButton = new QRadioButton( m_targetButtonGroup );
	m_allLinesRadioButton->setText( i18n( "All lines" ) );
	if ( ! m_selectedLinesOnlyMode )
		m_allLinesRadioButton->setChecked( true );

	m_selectedLinesRadioButton = new QRadioButton( m_targetButtonGroup );
	m_selectedLinesRadioButton->setText( i18n( "Selected lines" ) );
	if ( m_selectedLinesOnlyMode )
		m_selectedLinesRadioButton->setChecked( true );

	m_targetButtonGroupLayout = createLayout( m_targetButtonGroup );
	m_targetButtonGroupLayout->addMultiCellWidget( m_allLinesRadioButton, 0, 0, 0, 1 );
	m_targetButtonGroupLayout->addMultiCellWidget( m_selectedLinesRadioButton, 1, 1, 0, 1 );

	if ( m_selectedLinesOnlyMode )
		m_targetButtonGroup->hide();

	return m_targetButtonGroup;
}

bool SelectableTargetActionDialogBase::selectedLinesOnlyMode()
{
	return m_selectedLinesOnlyMode;
}

void SelectableTargetActionDialogBase::setSelectedLinesOnlyMode( bool value )
{
	if ( m_selectedLinesOnlyMode != value )
	{
		m_selectedLinesOnlyMode = value;

		if ( m_targetButtonGroup )
		{
			if ( m_selectedLinesOnlyMode )
			{
				// save the previous value so we can restore it latter when SelectedLinesOnlyMode is disabled
				m_selectedLinesOldValue = m_selectedLinesRadioButton->isChecked();

				m_selectedLinesRadioButton->setChecked( true );
				m_targetButtonGroup->hide();
			}
			else
			{
				if ( ! m_selectedLinesOldValue )
					m_allLinesRadioButton->setChecked( true );

				m_targetButtonGroup->show();
			}

			adjustSize();
		}
	}
}

bool SelectableTargetActionDialogBase::allLines()
{
	return m_allLinesRadioButton->isChecked();
}

bool SelectableTargetActionDialogBase::selectedLines()
{
	return m_selectedLinesRadioButton->isChecked();
}


SelectableTargetActionDialog::SelectableTargetActionDialog( const QString& title, const QString& desc, QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( title, parent, name )
{
	createTargetButtonGroup( desc );
}

int SelectableTargetActionDialog::exec()
{
	return selectedLinesOnlyMode() ? QDialog::Accepted : SelectableTargetActionDialogBase::exec();
}

SelectableSubtitleDialog::SelectableSubtitleDialog( const QString& defaultEncoding, const QString& title, QWidget* parent, const char* name ):
	ActionDialog( title, parent, name ),
	m_defaultEncoding( defaultEncoding )
{
}

QButtonGroup* SelectableSubtitleDialog::createSubtitleButtonGroup( const QString& title, bool addToLayout )
{
	m_subtitleButtonGroup = createButtonGroup( title, addToLayout );

	QLabel* subtitlePathLabel = new QLabel( m_subtitleButtonGroup );
	subtitlePathLabel->setText( i18n( "Path" ) );

	m_subtitlePathLineEdit = new KLineEdit( m_subtitleButtonGroup );
	m_subtitlePathLineEdit->setCompletionObject( new KURLCompletion() );

	KPushButton* subtitleButton = new KPushButton( m_subtitleButtonGroup );
	subtitleButton->setToggleButton( false );
	subtitleButton->setIconSet( kapp->iconLoader()->loadIconSet( "fileopen", KIcon::Small ) );
	QToolTip::add( subtitleButton, i18n( "Select subtitle" ) );

	connect( subtitleButton, SIGNAL( clicked() ), SLOT( selectSubtitle() ) );

	QLabel* subtitleEncodingLabel = new QLabel( m_subtitleButtonGroup );
	subtitleEncodingLabel->setText( i18n( "Encoding" ) );

	m_subtitleEncodingComboBox = new KComboBox( m_subtitleButtonGroup );
	QStringList encodings( KGlobal::charsets()->availableEncodingNames() );
	for ( QStringList::Iterator it = encodings.begin(); it != encodings.end(); ++it )
	{
		bool found = false;
		QTextCodec* codec = KGlobal::charsets()->codecForName( *it, found );
		if ( found )
			m_subtitleEncodingComboBox->insertItem( QString( codec->name() ).upper() );
	}
	m_subtitleEncodingComboBox->setCurrentItem( m_defaultEncoding );


	QHBoxLayout* subtitlePathLayout = new QHBoxLayout();
	subtitlePathLayout->addWidget( m_subtitlePathLineEdit );
	subtitlePathLayout->addWidget( subtitleButton );

	QHBoxLayout* subtitleEncodingLayout = new QHBoxLayout();
	subtitleEncodingLayout->addStretch();
	subtitleEncodingLayout->addWidget( m_subtitleEncodingComboBox );

	m_subtitleButtonGroupLayout = createLayout( m_subtitleButtonGroup );
	m_subtitleButtonGroupLayout->addWidget( subtitlePathLabel, 0, 0 );
	m_subtitleButtonGroupLayout->addMultiCellLayout( subtitlePathLayout, 0, 0, 1, 3 );
	m_subtitleButtonGroupLayout->addWidget( subtitleEncodingLabel, 1, 0 );
	m_subtitleButtonGroupLayout->addMultiCellLayout( subtitleEncodingLayout, 1, 1, 2, 3 );

	return m_subtitleButtonGroup;
}

void SelectableSubtitleDialog::selectSubtitle()
{
	OpenSubtitleDialog openDlg( subtitlePath(), subtitleEncoding() );

	if ( openDlg.exec() == QDialog::Accepted )
	{
		m_subtitlePathLineEdit->setText( openDlg.selectedFile() );
		m_subtitleEncodingComboBox->setCurrentItem( openDlg.selectedEncoding().upper() );
	}
}

QString SelectableSubtitleDialog::subtitlePath()
{
	return m_subtitlePathLineEdit->text();
}

QString SelectableSubtitleDialog::subtitleEncoding()
{
	return m_subtitleEncodingComboBox->currentText();
}


AdjustLinesDialog::AdjustLinesDialog( QWidget* parent, const char* name ):
	ActionDialog( i18n( "Adjust Subtitles" ), parent, name )
{
	QButtonGroup* buttonGroup = createButtonGroup( i18n( "Settings" ) );

	QLabel* firstLineLabel = new QLabel( i18n( "First spoken line" ), buttonGroup );
	firstLineLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_firstLineTimeEdit = new TimeEdit( buttonGroup );

	QLabel* lastLineLabel = new QLabel( i18n( "Last spoken line" ), buttonGroup );
	lastLineLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_lastLineTimeEdit = new TimeEdit( buttonGroup );

	QGridLayout* buttonGroupLayout = createLayout( buttonGroup );
	buttonGroupLayout->addWidget( firstLineLabel, 0, 0 );
	buttonGroupLayout->addWidget( m_firstLineTimeEdit, 0, 1 );
	buttonGroupLayout->addWidget( lastLineLabel, 1, 0 );
	buttonGroupLayout->addWidget( m_lastLineTimeEdit, 1, 1 );
}

Time AdjustLinesDialog::firstLineTime()
{
	return Time( m_firstLineTimeEdit->value() );
}

void AdjustLinesDialog::setFirstLineTime( const Time& time )
{
	m_firstLineTimeEdit->setValue( time.timeMillis() );
}

Time AdjustLinesDialog::lastLineTime()
{
	return Time( m_lastLineTimeEdit->value() );
}

void AdjustLinesDialog::setLastLineTime( const Time& time )
{
	m_lastLineTimeEdit->setValue( time.timeMillis() );
}

DurationLimitsDialog::DurationLimitsDialog( const Time& minDuration, const Time& maxDuration, QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Duration Limits" ), parent, name )
{
	m_maxGroupBox = createGroupBox( i18n( "Enforce maximun duration" ) );
	m_maxGroupBox->setCheckable( true );

	QLabel* maxDurationLabel = new QLabel( i18n( "Maximun duration" ), m_maxGroupBox );
	maxDurationLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_maxDurationTimeEdit = new TimeEdit( m_maxGroupBox, "m_maxDurationTimeEdit" );


	m_minGroupBox = createGroupBox( i18n( "Enforce minimun duration" ) );
	m_minGroupBox->setCheckable( true );

	QLabel* minDurationLabel = new QLabel( i18n( "Minimun duration" ), m_minGroupBox );
	minDurationLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_preventOverlapCheckBox = new QCheckBox( m_minGroupBox );
	m_preventOverlapCheckBox->setText( i18n( "Prevent overlapping" ) );
	m_preventOverlapCheckBox->setChecked( true );

	m_minDurationTimeEdit = new TimeEdit( m_minGroupBox );

	createTargetButtonGroup();


	QGridLayout* m_maxGroupBoxLayout = createLayout( m_maxGroupBox );
	m_maxGroupBoxLayout->addWidget( maxDurationLabel, 0, 0 );
	m_maxGroupBoxLayout->addWidget( m_maxDurationTimeEdit, 0, 1 );

	QGridLayout* m_minGroupBoxLayout = createLayout( m_minGroupBox );
	m_minGroupBoxLayout->addWidget( minDurationLabel, 0, 0 );
	m_minGroupBoxLayout->addMultiCellWidget( m_preventOverlapCheckBox, 1, 1, 0, 1 );
	m_minGroupBoxLayout->addWidget( m_minDurationTimeEdit, 0, 1 );

	connect( m_maxGroupBox, SIGNAL( toggled(bool) ), maxDurationLabel, SLOT( setEnabled(bool) ) );
	connect( m_maxGroupBox, SIGNAL( toggled(bool) ), m_maxDurationTimeEdit, SLOT( setEnabled(bool) ) );

	connect( m_minGroupBox, SIGNAL( toggled(bool) ), m_preventOverlapCheckBox, SLOT( setEnabled(bool) ) );
	connect( m_minGroupBox, SIGNAL( toggled(bool) ), minDurationLabel, SLOT( setEnabled(bool) ) );
	connect( m_minGroupBox, SIGNAL( toggled(bool) ), m_minDurationTimeEdit, SLOT( setEnabled(bool) ) );

	connect( m_maxDurationTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onMaxDurationValueChanged(int) ) );
	connect( m_minDurationTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( onMinDurationValueChanged(int) ) );

	m_maxDurationTimeEdit->setValue( maxDuration.timeMillis() );
	m_minDurationTimeEdit->setValue( minDuration.timeMillis() );
}

void DurationLimitsDialog::onMaxDurationValueChanged( int value )
{
	if ( m_minDurationTimeEdit->value() > value )
		m_minDurationTimeEdit->setValue( value );
}

void DurationLimitsDialog::onMinDurationValueChanged( int value )
{
	if ( m_maxDurationTimeEdit->value() < value )
		m_maxDurationTimeEdit->setValue( value );
}

Time DurationLimitsDialog::minDuration()
{
	return Time( m_minDurationTimeEdit->value() );
}

Time DurationLimitsDialog::maxDuration()
{
	return Time( m_maxDurationTimeEdit->value() );
}

bool DurationLimitsDialog::enforceMaxDuration()
{
	return m_maxGroupBox->isChecked();
}

bool DurationLimitsDialog::enforceMinDuration()
{
	return m_minGroupBox->isChecked();
}

bool DurationLimitsDialog::preventOverlap()
{
	return m_preventOverlapCheckBox->isChecked();
}


AutoDurationsDialog::AutoDurationsDialog( unsigned charMillis, unsigned wordMillis, unsigned lineMillis, QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Automatic Durations" ), parent, name )
{
	QGroupBox* settingsGroupBox = createGroupBox( i18n( "Settings" ) );

	QLabel* charMillisLabel = new QLabel( i18n( "Milliseconds per character" ), settingsGroupBox );
	charMillisLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_charMillisSpinBox = new KIntNumInput( settingsGroupBox );
	m_charMillisSpinBox->setMinValue( 0 );
	m_charMillisSpinBox->setMaxValue( 1000 );
	m_charMillisSpinBox->setValue( charMillis );

	QLabel* wordMillisLabel = new QLabel( i18n( "Milliseconds per word" ), settingsGroupBox );
	wordMillisLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_wordMillisSpinBox = new KIntNumInput( settingsGroupBox );
	m_wordMillisSpinBox->setMinValue( 0 );
	m_wordMillisSpinBox->setMaxValue( 1000 );
	m_wordMillisSpinBox->setValue( wordMillis );

	QLabel* lineMillisLabel = new QLabel( i18n( "Milliseconds per line" ), settingsGroupBox );
	lineMillisLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_lineMillisSpinBox = new KIntNumInput( settingsGroupBox );
	m_lineMillisSpinBox->setMinValue( 0 );
	m_lineMillisSpinBox->setMaxValue( 1000 );
	m_lineMillisSpinBox->setValue( lineMillis );

	m_preventOverlapCheckBox = new QCheckBox( settingsGroupBox );
	m_preventOverlapCheckBox->setText( i18n( "Prevent overlapping" ) );
	m_preventOverlapCheckBox->setChecked( true );

	QGridLayout* settingsGroupBoxLayout = createLayout( settingsGroupBox );
	settingsGroupBoxLayout->addWidget( charMillisLabel, 0, 0 );
	settingsGroupBoxLayout->addWidget( m_charMillisSpinBox, 0, 1 );
	settingsGroupBoxLayout->addWidget( wordMillisLabel, 1, 0 );
	settingsGroupBoxLayout->addWidget( m_wordMillisSpinBox, 1, 1 );
	settingsGroupBoxLayout->addWidget( lineMillisLabel, 2, 0 );
	settingsGroupBoxLayout->addWidget( m_lineMillisSpinBox, 2, 1 );
	settingsGroupBoxLayout->addMultiCellWidget( m_preventOverlapCheckBox, 3, 3, 0, 1 );

	createTargetButtonGroup();
}

unsigned AutoDurationsDialog::charMillis()
{
	return m_charMillisSpinBox->value();
}

unsigned AutoDurationsDialog::wordMillis()
{
	return m_wordMillisSpinBox->value();
}

unsigned AutoDurationsDialog::lineMillis()
{
	return m_lineMillisSpinBox->value();
}

bool AutoDurationsDialog::preventOverlap()
{
	return m_preventOverlapCheckBox->isChecked();
}



ShiftLinesDialog::ShiftLinesDialog( QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Shift Subtitles" ), parent, name )
{
	QGroupBox* groupBox = createGroupBox( i18n( "Shifting" ) );

	m_directionComboBox = new KComboBox( false, groupBox );
	m_directionComboBox->clear();
	m_directionComboBox->insertItem( i18n( "+ | Forwards" ) );
	m_directionComboBox->insertItem( i18n( "- | Backwards" ) );
	m_directionComboBox->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );

	m_shiftTimeEdit = new TimeEdit( groupBox );

	QGridLayout* groupBoxLayout = createLayout( groupBox );
	groupBoxLayout->addWidget( m_directionComboBox, 0, 0 );
	groupBoxLayout->addWidget( m_shiftTimeEdit, 0, 1 );

	createTargetButtonGroup();
}

void ShiftLinesDialog::resetShiftTime()
{
	m_shiftTimeEdit->setValue( 0 );
}

int ShiftLinesDialog::shiftTimeMillis()
{
	return m_directionComboBox->currentItem() == 0 ? m_shiftTimeEdit->value() : - m_shiftTimeEdit->value();
}





SmartLinesAdjustDialog::SmartLinesAdjustDialog( unsigned minLengthForLineBreak, QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Smart Lines Adjust" ), parent, name )
{
	QGroupBox* settingsGroupBox = createGroupBox( i18n( "Settings" ) );

	QLabel* m_minLengthForLineBreakLabel = new QLabel( i18n( "Minimum length for line break" ), settingsGroupBox );
	m_minLengthForLineBreakLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );

	m_minLengthForLineBreakSpinBox = new KIntNumInput( settingsGroupBox );
	m_minLengthForLineBreakSpinBox->setMinValue( 0 );
	m_minLengthForLineBreakSpinBox->setMaxValue( 1000 );
	m_minLengthForLineBreakSpinBox->setValue( minLengthForLineBreak );

	QGridLayout* settingsGroupBoxLayout = createLayout( settingsGroupBox );
	settingsGroupBoxLayout->addWidget( m_minLengthForLineBreakLabel, 0, 0 );
	settingsGroupBoxLayout->addWidget( m_minLengthForLineBreakSpinBox, 0, 1 );

	createTargetButtonGroup();
}

unsigned SmartLinesAdjustDialog::minLengthForLineBreak()
{
	return m_minLengthForLineBreakSpinBox->value();
}




ChangeLinesTextCaseDialog::ChangeLinesTextCaseDialog( QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Change Case" ), parent, name )
{
	QButtonGroup* caseButtonGroup = createButtonGroup( i18n( "Change to" ) );

	m_lowerRadioButton = new QRadioButton( caseButtonGroup );
	m_lowerRadioButton->setText( i18n( "Lower case" ) );
	m_lowerRadioButton->setChecked( true );

	m_upperRadioButton = new QRadioButton( caseButtonGroup );
	m_upperRadioButton->setText( i18n( "Upper case" ) );

	m_titleRadioButton = new QRadioButton( caseButtonGroup );
	m_titleRadioButton->setText( i18n( "Title case" ) );

	m_sentenceRadioButton = new QRadioButton( caseButtonGroup );
	m_sentenceRadioButton->setText( i18n( "Sentence case" ) );

	m_lowerFirstCheckBox = new QCheckBox( caseButtonGroup );
	m_lowerFirstCheckBox->setText( i18n( "Change to lower case first" ) );
	m_lowerFirstCheckBox->setEnabled( false );

	createTargetButtonGroup();

	QHBoxLayout* lowerFirstLayout = new QHBoxLayout();
	lowerFirstLayout->setMargin( 0 );
	lowerFirstLayout->setSpacing( 0 );
	lowerFirstLayout->addSpacing( 20 );
	lowerFirstLayout->addWidget( m_lowerFirstCheckBox );

	QGridLayout* caseGroupBoxLayout = createLayout( caseButtonGroup );
	caseGroupBoxLayout->addWidget( m_lowerRadioButton, 0, 0 );
	caseGroupBoxLayout->addWidget( m_upperRadioButton, 1, 0 );
	caseGroupBoxLayout->addWidget( m_titleRadioButton, 2, 0 );
	caseGroupBoxLayout->addWidget( m_sentenceRadioButton, 3, 0 );
	caseGroupBoxLayout->addLayout( lowerFirstLayout, 4, 0 );

	connect( caseButtonGroup, SIGNAL( clicked(int) ), this, SLOT( onCaseButtonGroupClicked(int) ) );
}

Subtitle::CaseOp ChangeLinesTextCaseDialog::caseOperation()
{
	if ( m_lowerRadioButton->isChecked() )
		return Subtitle::Lower;
	else if ( m_upperRadioButton->isChecked() )
		return Subtitle::Upper;
	else if ( m_titleRadioButton->isChecked() )
		return Subtitle::Title;
	else // if ( m_sentenceRadioButton->isChecked() )
		return Subtitle::Sentence;
}

bool ChangeLinesTextCaseDialog::lowerFirst()
{
	return m_lowerFirstCheckBox->isChecked();
}

void ChangeLinesTextCaseDialog::onCaseButtonGroupClicked( int id )
{
	if ( id < 2 )
		m_lowerFirstCheckBox->setChecked( false );
	m_lowerFirstCheckBox->setEnabled( id >= 2 );
}



FixLinesPunctuationDialog::FixLinesPunctuationDialog( QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Fix Punctuation" ), parent, name )
{
	QGroupBox* caseButtonGroup = createGroupBox( i18n( "Settings" ) );

	m_spacesCheckBox = new QCheckBox( caseButtonGroup );
	m_spacesCheckBox->setText( i18n( "Fix and cleanup spaces" ) );
	m_spacesCheckBox->setChecked( true );

	m_quotesCheckBox = new QCheckBox( caseButtonGroup );
	m_quotesCheckBox->setText( i18n( "Fix quotes and double quotes" ) );
	m_quotesCheckBox->setChecked( true );

	m_englishICheckBox = new QCheckBox( caseButtonGroup );
	m_englishICheckBox->setText( i18n( "Fix English 'I' pronoun" ) );
	m_englishICheckBox->setChecked( true );

	m_ellipsisCheckBox = new QCheckBox( caseButtonGroup );
	m_ellipsisCheckBox->setText( i18n( "Add ellipisis indicating non finished lines" ) );
	m_ellipsisCheckBox->setChecked( true );

	createTargetButtonGroup();

	QGridLayout* caseGroupBoxLayout = createLayout( caseButtonGroup );
	caseGroupBoxLayout->addWidget( m_spacesCheckBox, 0, 0 );
	caseGroupBoxLayout->addWidget( m_quotesCheckBox, 1, 0 );
	caseGroupBoxLayout->addWidget( m_englishICheckBox, 2, 0 );
	caseGroupBoxLayout->addWidget( m_ellipsisCheckBox, 3, 0 );
}

bool FixLinesPunctuationDialog::spaces()
{
	return m_spacesCheckBox->isChecked();
}

bool FixLinesPunctuationDialog::quotes()
{
	return m_quotesCheckBox->isChecked();
}

bool FixLinesPunctuationDialog::englishI()
{
	return m_englishICheckBox->isChecked();
}

bool FixLinesPunctuationDialog::ellipisis()
{
	return m_ellipsisCheckBox->isChecked();
}




CheckErrorsDialog::CheckErrorsDialog( QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Search Errors" ), parent, name )
{
	QGroupBox* errorsGroupBox = createGroupBox( i18n( "Errors to search for" ) );

	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
	{
		m_errorsCheckBox[id] = new QCheckBox( errorsGroupBox );
		m_errorsCheckBox[id]->setText( ErrorsViewItem::errorText( (SubtitleLine::ErrorID)id ) );
		m_errorsCheckBox[id]->setChecked( true );
	}

	QGroupBox* otherGroupBox = createGroupBox( i18n( "Miscellaneous" ) );

	m_clearOtherErrorsCheckBox = new QCheckBox( otherGroupBox );
	m_clearOtherErrorsCheckBox->setText( i18n( "Clear other errors" ) );
	m_clearOtherErrorsCheckBox->setChecked( true );

	m_clearMarksCheckBox = new QCheckBox( otherGroupBox );
	m_clearMarksCheckBox->setText( i18n( "Clear user set marks" ) );

	createTargetButtonGroup();

	QGridLayout* errorsGroupBoxLayout = createLayout( errorsGroupBox );
	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
		errorsGroupBoxLayout->addWidget( m_errorsCheckBox[id], id, 0 );

	QGridLayout* otherGroupBoxLayout = createLayout( otherGroupBox );
	otherGroupBoxLayout->addWidget( m_clearOtherErrorsCheckBox, 0, 0 );
	otherGroupBoxLayout->addWidget( m_clearMarksCheckBox, 1, 0 );
}

int CheckErrorsDialog::errorsToClear()
{
	int errors = 0;
	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
		if ( m_errorsCheckBox[id]->isChecked() )
			errors |= SubtitleLine::errorFlag( (SubtitleLine::ErrorID)id );

	return errors;
}

bool CheckErrorsDialog::clearOtherErrors()
{
	return m_clearOtherErrorsCheckBox->isChecked();
}

bool CheckErrorsDialog::clearMarks()
{
	return m_clearMarksCheckBox->isChecked();
}



ClearErrorsDialog::ClearErrorsDialog( QWidget* parent, const char* name ):
	SelectableTargetActionDialogBase( i18n( "Clear Errors" ), parent, name )
{
	QGroupBox* errorsGroupBox = createGroupBox( i18n( "Errors to clear" ) );

	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
	{
		m_errorsCheckBox[id] = new QCheckBox( errorsGroupBox );
		m_errorsCheckBox[id]->setText( ErrorsViewItem::errorText( (SubtitleLine::ErrorID)id ) );
		m_errorsCheckBox[id]->setChecked( true );
	}

	m_marksCheckBox = new QCheckBox( errorsGroupBox );
	m_marksCheckBox->setText( i18n( "User marks" ) );
	m_marksCheckBox->setChecked( true );

	createTargetButtonGroup();

	QGridLayout* errorsGroupBoxLayout = createLayout( errorsGroupBox );
	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
		errorsGroupBoxLayout->addWidget( m_errorsCheckBox[id], id, 0 );

	errorsGroupBoxLayout->addWidget( m_marksCheckBox, SubtitleLine::ErrorSIZE, 0 );
}

int ClearErrorsDialog::errorsToClear()
{
	int errors = 0;
	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
		if ( m_errorsCheckBox[id]->isChecked() )
			errors |= SubtitleLine::errorFlag( (SubtitleLine::ErrorID)id );

	return errors;
}

bool ClearErrorsDialog::clearMarks()
{
	return m_marksCheckBox->isChecked();
}




ChangeFrameRateDialog::ChangeFrameRateDialog( double fromFramesPerSecond, QWidget* parent, const char* name ):
	ActionDialog( i18n( "Change Frame Rate" ), parent, name )
{
	QButtonGroup* buttonGroup = createButtonGroup( i18n( "Settings" ));

	QLabel* fromFramesPerSecondLabel = new QLabel( buttonGroup );
	fromFramesPerSecondLabel->setText( i18n( "Current frame rate" ) );

	m_fromFramesPerSecondComboBox = new KComboBox( false, buttonGroup );
	m_fromFramesPerSecondComboBox->setEditable( true );
	m_fromFramesPerSecondComboBox->insertStringList( QStringList::split( ' ', "15 20 23.976 24 25 29.970 30" ) );
	m_fromFramesPerSecondComboBox->setCurrentItem( 2 );
	setFromFramesPerSecond( fromFramesPerSecond );

	QLabel* toFramesPerSecondLabel = new QLabel( buttonGroup );
	toFramesPerSecondLabel->setText( i18n( "New frame rate" ) );

	m_toFramesPerSecondComboBox = new KComboBox( false, buttonGroup );
	m_toFramesPerSecondComboBox->setEditable( true );
	m_toFramesPerSecondComboBox->insertStringList( QStringList::split( ' ', "15 20 23.976 24 25 29.970 30" ) );
	m_toFramesPerSecondComboBox->setCurrentItem( 2 );

	QGridLayout* buttonGroupLayout = createLayout( buttonGroup );
	buttonGroupLayout->addWidget( fromFramesPerSecondLabel, 0, 0 );
	buttonGroupLayout->addWidget( m_fromFramesPerSecondComboBox, 0, 1 );
	buttonGroupLayout->addWidget( toFramesPerSecondLabel, 1, 0 );
	buttonGroupLayout->addWidget( m_toFramesPerSecondComboBox, 1, 1 );
}

double ChangeFrameRateDialog::fromFramesPerSecond()
{
	return m_fromFramesPerSecondComboBox->lineEdit()->text().toDouble();
}

void ChangeFrameRateDialog::setFromFramesPerSecond( double framesPerSecond )
{
	m_fromFramesPerSecondComboBox->lineEdit()->setText( QString::number( framesPerSecond, 'f', 3 ) );
}

double ChangeFrameRateDialog::toFramesPerSecond()
{
	return m_toFramesPerSecondComboBox->lineEdit()->text().toDouble();
}

void ChangeFrameRateDialog::setNewFramesPerSecond( double framesPerSecond )
{
	m_toFramesPerSecondComboBox->lineEdit()->setText( QString::number( framesPerSecond, 'f', 3 ) );
}

void ChangeFrameRateDialog::accept()
{
	bool ok;
	m_fromFramesPerSecondComboBox->lineEdit()->text().toDouble( &ok );
	if ( ! ok )
	{
		KMessageBox::error( this, i18n( "Invalid current frame rate value specified:\n%1" ).arg( m_fromFramesPerSecondComboBox->lineEdit()->text() ) );
		return;
	}

	m_toFramesPerSecondComboBox->lineEdit()->text().toDouble( &ok );
	if ( ! ok )
	{
		KMessageBox::error( this, i18n( "Invalid new frame rate value specified:\n%1" ).arg( m_toFramesPerSecondComboBox->lineEdit()->text() ) );
		return;
	}

	ActionDialog::accept();
}








SynchronizeWithSubtitleDialog::SynchronizeWithSubtitleDialog( const QString& defaultEncoding, QWidget* parent, const char* name ):
	SelectableSubtitleDialog( defaultEncoding, i18n( "Synchronize With Subtitle" ), parent, name )
{
	createSubtitleButtonGroup( i18n( "Reference subtitle" ) );

	QButtonGroup* syncModeGroup = createButtonGroup( i18n( "Synchronization mode" ));

	m_adjustRadioButton = new QRadioButton( syncModeGroup );
	m_adjustRadioButton->setText( i18n( "Adjust to reference subtitle first and last lines show times" ) );
	m_adjustRadioButton->setChecked( true );

	m_synchronizeRadioButton = new QRadioButton( syncModeGroup );
	m_synchronizeRadioButton->setText( i18n( "Copy timing information from reference subtitle line by line" ) );

	QGridLayout* syncModeLayout = createLayout( syncModeGroup );
	syncModeLayout->addWidget( m_adjustRadioButton, 0, 0 );
	syncModeLayout->addWidget( m_synchronizeRadioButton, 1, 0 );
}

bool SynchronizeWithSubtitleDialog::adjustToReferenceSubtitle()
{
	return m_adjustRadioButton->isChecked();
}

bool SynchronizeWithSubtitleDialog::synchronizeToReferenceTimes()
{
	return m_synchronizeRadioButton->isChecked();
}


JoinSubtitlesDialog::JoinSubtitlesDialog( const QString& defaultEncoding, QWidget* parent, const char* name ):
	SelectableSubtitleDialog( defaultEncoding, i18n( "Join Subtitles" ), parent, name )
{
	createSubtitleButtonGroup();

	m_shiftSubtitleCheckBox = new QCheckBox( m_subtitleButtonGroup );
	m_shiftSubtitleCheckBox->setText( i18n( "Shift subtitle forwards before append" ) );
	m_shiftSubtitleCheckBox->setChecked( true );

	m_shiftTimeFromVideoButton = new KPushButton( m_subtitleButtonGroup );
	m_shiftTimeFromVideoButton->setToggleButton( false );
	m_shiftTimeFromVideoButton->setIconSet( GUI::customIconSet( "video", KIcon::Toolbar, 16 ) );
	QToolTip::add( m_shiftTimeFromVideoButton, i18n( "Set from video length" ) );
	connect( m_shiftTimeFromVideoButton, SIGNAL( clicked() ), SLOT( setShiftTimeFromVideo() ) );

	m_shiftTimeEdit = new TimeEdit( m_subtitleButtonGroup );
	connect( m_shiftSubtitleCheckBox, SIGNAL( toggled(bool) ), m_shiftTimeFromVideoButton, SLOT( setEnabled(bool) ) );
	connect( m_shiftSubtitleCheckBox, SIGNAL( toggled(bool) ), m_shiftTimeEdit, SLOT( setEnabled(bool) ) );

	QHBoxLayout* shiftTimeLayout = new QHBoxLayout();
	shiftTimeLayout->addStretch();
	shiftTimeLayout->addWidget( m_shiftTimeFromVideoButton );
	shiftTimeLayout->addWidget( m_shiftTimeEdit );

	m_subtitleButtonGroupLayout->addMultiCellWidget( m_shiftSubtitleCheckBox, 2, 2, 0, 2 );
	m_subtitleButtonGroupLayout->addLayout( shiftTimeLayout, 2, 3 );
}

void JoinSubtitlesDialog::setShiftTimeFromVideo()
{
	Player* activePlayer = PlayersManager::activePlayer();
	if ( activePlayer )
		m_shiftTimeEdit->setValue( (int)(activePlayer->length()*1000 + 0.5) );
}

Time JoinSubtitlesDialog::shiftTime()
{
	return m_shiftSubtitleCheckBox->isChecked() ? m_shiftTimeEdit->value() : 0;
}

void JoinSubtitlesDialog::show()
{
	Player* activePlayer = PlayersManager::activePlayer();
	m_shiftTimeFromVideoButton->setEnabled( activePlayer && ! activePlayer->filePath().isEmpty() );
	if ( m_shiftTimeFromVideoButton->isEnabled() )
		setShiftTimeFromVideo();

	SelectableSubtitleDialog::show();
}



SplitSubtitleDialog::SplitSubtitleDialog( QWidget* parent, const char* name ):
	ActionDialog( i18n( "Split Subtitle" ), parent, name )
{
	QGroupBox* settingsGroupBox = createGroupBox( i18n( "Settings" ) );

	QLabel* splitTimeLabel = new QLabel( settingsGroupBox );
	splitTimeLabel->setText( i18n( "Split time" ) );

	m_splitTimeFromVideoButton = new KPushButton( settingsGroupBox );
	m_splitTimeFromVideoButton->setToggleButton( false );
	m_splitTimeFromVideoButton->setIconSet( GUI::customIconSet( "video", KIcon::Toolbar, 16 ) );
	QToolTip::add( m_splitTimeFromVideoButton, i18n( "Set from video length" ) );
	connect( m_splitTimeFromVideoButton, SIGNAL( clicked() ), SLOT( setSplitTimeFromVideo() ) );

	m_splitTimeEdit = new TimeEdit( settingsGroupBox );

	m_shiftNewSubtitleCheckBox = new QCheckBox( settingsGroupBox );
	m_shiftNewSubtitleCheckBox->setText( i18n( "Shift new subtitle part backwards after split" ) );
	m_shiftNewSubtitleCheckBox->setChecked( true );

	QHBoxLayout* splitTimeLayout = new QHBoxLayout();
	splitTimeLayout->addStretch();
	splitTimeLayout->addWidget( m_splitTimeFromVideoButton );
	splitTimeLayout->addWidget( m_splitTimeEdit );

	QGridLayout* settingsLayout = createLayout( settingsGroupBox );
	settingsLayout->addWidget( splitTimeLabel, 0, 0 );
	settingsLayout->addLayout( splitTimeLayout, 0, 1 );
	settingsLayout->addMultiCellWidget( m_shiftNewSubtitleCheckBox, 1, 1, 0, 1 );
}

void SplitSubtitleDialog::setSplitTimeFromVideo()
{
	Player* activePlayer = PlayersManager::activePlayer();
	if ( activePlayer )
		m_splitTimeEdit->setValue( (int)(activePlayer->length()*1000 + 0.5) );
}

Time SplitSubtitleDialog::splitTime()
{
	return m_splitTimeEdit->value();
}

bool SplitSubtitleDialog::shiftNewSubtitle()
{
	return m_shiftNewSubtitleCheckBox->isChecked();
}

void SplitSubtitleDialog::show()
{
	Player* activePlayer = PlayersManager::activePlayer();
	m_splitTimeFromVideoButton->setEnabled( activePlayer && ! activePlayer->filePath().isEmpty() );
	if ( m_splitTimeFromVideoButton->isEnabled() )
		setSplitTimeFromVideo();

	ActionDialog::show();
}



GotoLineDialog::GotoLineDialog( int line, int linesCount, QWidget* parent, const char* name ):
	KDialogBase( parent, name, true, i18n( "Go to Line" ), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok )
{
	QWidget* mainWidget = new QWidget( this );
	setMainWidget( mainWidget );

	QVBoxLayout* mainLayout = new QVBoxLayout( mainWidget, 0, spacingHint() );
	mainLayout->addSpacing( spacingHint() ); // A little bit extra space
	mainLayout->addStretch( 10 );

	QLabel* label = new QLabel( mainWidget, i18n( "&Go to line:" ), mainWidget );
	mainLayout->addWidget( label );

	lineNumInput = new KIntNumInput( line, mainWidget );
	lineNumInput->setRange( 1, linesCount );
	lineNumInput->setEditFocus( true );
	lineNumInput->setFocus();
	mainLayout->addWidget( lineNumInput );
}

int GotoLineDialog::lineNumber()
{
	return lineNumInput->value();
}

