/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "configdialog.h"
#include "config/generalconfigwidget.h"
#include "config/subtitletextconfigwidget.h"
#include "config/errorsconfigwidget.h"
#include <playersmanager.h>
#include <commondefs.h>

#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qtooltip.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qscrollview.h>

#include <klocale.h>
#include <kiconloader.h>
#include <kstandarddirs.h>

using namespace SubtitleComposer;

ConfigDialog::ConfigDialog( const AppConfig& config, QWidget* parent, const char* name ):
	KDialogBase::KDialogBase(	KDialogBase::IconList,	// dialogFace
								0,						// flags
								parent,					// parent
								name,					// name
								false,					// modal
								"Configure",			// caption
								Ok|Apply|Cancel|Default,// buttonMask
								Ok, 					// defaultButton
								true ),					// separator
	m_acceptedConfig( config ),
	m_config( config )
{
	showButton( Default, true );

	KIconLoader iconLoader;

	m_configWidgets.insert( General, new GeneralConfigWidget( *m_config.configGroup( "General" ) ) );
	addPage( m_configWidgets.at( General ), i18n( "General" ),
			 iconLoader.loadIcon( "misc", KIcon::Desktop ), i18n( "General" ) );

	m_configWidgets.insert( Texts, new SubtitleTextConfigWidget( *m_config.configGroup( "Subtitle Text" ) ) );
	addPage( m_configWidgets.at( Texts ), i18n( "Subtitle Text" ),
			 iconLoader.loadIcon( "fonts", KIcon::Desktop ), i18n( "Subtitle Text" ) );

	m_configWidgets.insert( Errors, new ErrorsConfigWidget( *m_config.configGroup( "Errors" ) ) );
	addPage( m_configWidgets.at( Errors ), i18n( "Errors" ),
			 iconLoader.loadIcon( "error", KIcon::Desktop ), i18n( "Errors" ) );

	unsigned idx = PlayerFirst;
	QStringList playerNames( PlayersManager::playerNames() );
	for ( QStringList::Iterator it = playerNames.begin(); it != playerNames.end(); ++it, ++idx )
	{
		m_configWidgets.append( PlayersManager::player( *it )->newAppConfigGroupWidget( 0 ) );
		addPage( m_configWidgets.at( idx ), *it, GUI::customIcon( (*it).lower(), KIcon::Desktop ), *it );
	}

	setControlsFromConfig();

	connect( this, SIGNAL( applyClicked(void) ), this, SLOT( acceptConfig(void) ) );
	connect( this, SIGNAL( okClicked(void) ), this, SLOT( acceptConfigAndClose(void) ) );
	connect( this, SIGNAL( cancelClicked(void) ), this, SLOT( rejectConfigAndClose(void) ) );
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( setActivePageControlsFromDefaults(void) ) );
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( enableApply(void) ) );

	// Apply button connections/setup
	connect( this, SIGNAL( defaultClicked(void) ), this, SLOT( enableApply(void) ) );
	for ( QPtrListIterator<AppConfigGroupWidget> it( m_configWidgets ); it.current(); ++it )
		connect( it.current(), SIGNAL( settingsChanged() ), this, SLOT( enableApply(void) ) );

	enableButtonApply( false );

	resize( QSize( 530, 300 ).expandedTo( minimumSizeHint() ) );
}

ConfigDialog::~ConfigDialog()
{
	m_configWidgets.setAutoDelete( false );
}

const AppConfig& ConfigDialog::getConfig() const
{
	return m_acceptedConfig;
}

void ConfigDialog::setConfig( const AppConfig& config )
{
	bool visible = isVisible();

	if ( visible )
		hide();

	m_acceptedConfig = m_config = config;
	setControlsFromConfig();

	enableButtonApply( false );

	if ( visible )
		show();
}

void ConfigDialog::setControlsFromConfig()
{
	for ( unsigned pageIndex = 0; pageIndex < m_configWidgets.count(); ++pageIndex )
		setControlsFromConfig( pageIndex );
}

void ConfigDialog::setControlsFromConfig( unsigned pageIndex )
{
	if ( pageIndex < m_configWidgets.count() )
	{
		AppConfigGroup* configGroup = m_config.configGroup( m_configWidgets.at( pageIndex )->configGroup().name() );
		if ( configGroup )
		{
			m_configWidgets.at( pageIndex )->setConfigGroup( *configGroup );
			m_configWidgets.at( pageIndex )->setControlsFromConfig();
		}
	}
}

void ConfigDialog::setActivePageControlsFromDefaults()
{
	setControlsFromDefaults( activePageIndex() );
}

void ConfigDialog::setControlsFromDefaults( unsigned pageIndex )
{
	if ( pageIndex < m_configWidgets.count() )
		m_configWidgets.at( pageIndex )->setControlsFromDefaults();
}


void ConfigDialog::setConfigFromControls()
{
	for ( unsigned pageIndex = 0; pageIndex < m_configWidgets.count(); ++pageIndex )
		setConfigFromControls( pageIndex );
}

void ConfigDialog::setConfigFromControls( unsigned pageIndex )
{
	if ( pageIndex < m_configWidgets.count() )
	{
		m_configWidgets.at( pageIndex )->setConfigFromControls();
		m_config.setConfigGroup( m_configWidgets.at( pageIndex )->configGroup() ); // replace the group
	}
}

void ConfigDialog::acceptConfig()
{
	setConfigFromControls();
	m_acceptedConfig = m_config;

	enableButtonApply( false );

	emit accepted();
}

void ConfigDialog::acceptConfigAndClose()
{
	acceptConfig();
	hide();
}

void ConfigDialog::rejectConfig()
{
	m_config = m_acceptedConfig;
	setControlsFromConfig();
	enableButtonApply( false );
}

void ConfigDialog::rejectConfigAndClose()
{
	hide();
	rejectConfig();
}

void ConfigDialog::addPage( QWidget* page, const QString& itemName, const QPixmap& pixmap, const QString& header )
{
	QVBox *frame = addVBoxPage( itemName, header, pixmap );
	frame->setSpacing( 0 );
	frame->setMargin( 0 );
	page->reparent( (QWidget*)frame, 0, QPoint() );
}

void ConfigDialog::addPage( QWidget* page, const QString& itemName, const QString& pixmapName, const QString& header )
{
	QVBox *frame = addVBoxPage( itemName, header, pixmapName );
	frame->setSpacing( 0 );
	frame->setMargin( 0 );
	page->reparent( (QWidget*)frame, 0, QPoint() );
}

void ConfigDialog::enableApply()
{
	enableButtonApply( true );
}

