/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "currentlinewidget.h"
#include <timeedit.h>
#include <textedit.h>
#include <qxtsignalwaiter.h>

#include <kglobalsettings.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kshortcut.h>
#include <kaccel.h>
#include <kdebug.h>
#include <kspell.h>

#include <qpopupmenu.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qtimer.h>

using namespace SubtitleComposer;

CurrentLineWidget::CurrentLineWidget( QWidget* parent ):
	QWidget( parent, "CurrentLineWidget" ),
// 	m_timer( new QTimer( this ) ),
	m_line( 0 ),
	m_updateLine( true ),
	m_updateControl( true ),
	m_textEdit( new TextEdit( this ) )
{
	m_textEdit->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum );

	QGroupBox* timesControlsGroupBox = new QGroupBox( this, "TimesGroupBox" );
	timesControlsGroupBox->setColumnLayout( 0, Qt::Vertical );
	timesControlsGroupBox->layout()->setSpacing( 5 );
	timesControlsGroupBox->layout()->setMargin( 6 );

	QLabel* showLabel = new QLabel( i18n( "<b>Show</b>" ), timesControlsGroupBox );
	m_showTimeEdit = new TimeEdit( timesControlsGroupBox );

	QLabel* hideLabel = new QLabel( i18n( "<b>Hide</b>" ), timesControlsGroupBox );
	m_hideTimeEdit = new TimeEdit( timesControlsGroupBox );

	QLabel* durationLabel = new QLabel( i18n( "<b>Duration</b>" ), timesControlsGroupBox );
	m_durationTimeEdit = new TimeEdit( timesControlsGroupBox ) ;

	QGridLayout* timesControlsGroupBoxLayout = new QGridLayout( timesControlsGroupBox->layout() );
	timesControlsGroupBoxLayout->addWidget( showLabel, 0, 0 );
	timesControlsGroupBoxLayout->addWidget( m_showTimeEdit, 0, 1 );
	timesControlsGroupBoxLayout->addWidget( hideLabel, 1, 0 );
	timesControlsGroupBoxLayout->addWidget( m_hideTimeEdit, 1, 1 );
	timesControlsGroupBoxLayout->addWidget( durationLabel, 2, 0 );
	timesControlsGroupBoxLayout->addWidget( m_durationTimeEdit, 2, 1 );

	QGridLayout* mainLayout = new QGridLayout( this, 3, 2 );
	mainLayout->setMargin( 0 );
	mainLayout->setSpacing( 5 );

	mainLayout->addWidget( timesControlsGroupBox, 0, 0 );
	mainLayout->addWidget( m_textEdit, 0, 1 );

	connect( m_durationTimeEdit, SIGNAL(valueChanged(int)), this, SLOT(hideTimeFromDurationTime(int)) );
	connect( m_hideTimeEdit, SIGNAL(valueChanged(int)), this, SLOT(durationTimeFromHideTime(int)) );
	connect( m_showTimeEdit, SIGNAL(valueChanged(int)), this, SLOT(durationTimeFromShowTime(int)) );

	closeSubtitle();
}

CurrentLineWidget::~CurrentLineWidget()
{
}

TextEdit* CurrentLineWidget::textEdit()
{
	return m_textEdit;
}

void CurrentLineWidget::closeSubtitle()
{
	m_line = 0;
	updateControlsFromLine();
}

void CurrentLineWidget::setCurrentLine( SubtitleLine* line )
{
	if ( m_line != line )
	{
		if ( m_line )
			disconnect( m_line, 0, m_textEdit, 0 );

		m_line = line;

		if ( m_line )
		{
			connect( m_line, SIGNAL(textChanged(const QString&)), this, SLOT(updateControlsFromLine()) );
			connect( m_line, SIGNAL(showTimeChanged(const Time&)), this, SLOT(updateControlsFromLine()) );
			connect( m_line, SIGNAL(hideTimeChanged(const Time&)), this, SLOT(updateControlsFromLine()) );
			connect( m_line, SIGNAL(styleFlagsChanged(int)), this, SLOT(updateControlsFromLine()) );

			connect( m_textEdit, SIGNAL( textChanged() ), this, SLOT( updateLineFromControls() ) ) ;
			connect( m_showTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( updateLineFromControls() ) ) ;
			connect( m_hideTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( updateLineFromControls() ) ) ;
			connect( m_durationTimeEdit, SIGNAL( valueChanged(int) ), this, SLOT( updateLineFromControls() ) ) ;
		}

		updateControlsFromLine();
	}
}

void CurrentLineWidget::updateControlsFromLine()
{
	if ( m_updateControl )
	{
		m_updateLine = false;

		m_textEdit->clear();
		if ( m_line != 0 )
		{
			setEnabled( true );

			int styleFlags = m_line->styleFlags();
			m_textEdit->setBold( styleFlags & SubtitleLine::Bold );
			m_textEdit->setItalic( styleFlags & SubtitleLine::Italic );
			m_textEdit->setUnderline( styleFlags & SubtitleLine::Underline );

			m_textEdit->setText( m_line->text() );

			m_showTimeEdit->setValue( m_line->showTime().timeMillis() );
			m_hideTimeEdit->setValue( m_line->hideTime().timeMillis() );
			m_durationTimeEdit->setValue( m_line->durationTime().timeMillis() );
		}
		else
		{
			setEnabled( false );

			m_showTimeEdit->setValue( 0 );
			m_hideTimeEdit->setValue( 0 );
			m_durationTimeEdit->setValue( 0 );
		}

		m_updateLine = true;
	}
}

void CurrentLineWidget::updateLineFromControls()
{
	if ( m_updateLine )
	{
		if ( m_line != 0 )
		{
			QString text( m_textEdit->text() );
			if ( text != m_line->text() )
			{
				m_updateControl = false;
				m_line->setText( text );
				m_updateControl = true;
			}
			else if ( m_line->showTime().timeMillis() != m_showTimeEdit->value() )
			{
				m_updateControl = false;
				m_line->setShowTime( m_showTimeEdit->value() );
				m_updateControl = true;
			}
			else if ( m_line->hideTime().timeMillis() != m_hideTimeEdit->value() )
			{
				m_updateControl = false;
				m_line->setHideTime( m_hideTimeEdit->value() );
				m_updateControl = true;
			}
		}
	}
}

void CurrentLineWidget::hideTimeFromDurationTime( int durationTime )
{
	if ( m_updateControl )
		m_hideTimeEdit->setValue( m_showTimeEdit->value() + durationTime );
}

void CurrentLineWidget::durationTimeFromShowTime( int showTime )
{
	if ( m_updateControl )
	{
		int duration = m_hideTimeEdit->value() - showTime;

		if ( duration < 0 )
			m_updateControl = false;

		m_durationTimeEdit->setValue( duration );

		if ( duration < 0 )
			m_updateControl = true;
	}
}

void CurrentLineWidget::durationTimeFromHideTime( int hideTime )
{
	if ( m_updateControl )
		m_durationTimeEdit->setValue( hideTime - m_showTimeEdit->value() );
}

#include "currentlinewidget.moc"
