/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actions.h"
#include "actionsdialogs.h"
#include "linesview.h"
#include "errorsview.h"
#include "errorsviewitem.h"
#include "config/errorsconfigdefs.h"

#include "subtitle.h"
#include "subtitleline.h"
#include "subtitleiterator.h"

#include <qheader.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlabel.h>

#include <kapplication.h>
#include <kpopupmenu.h>
#include <kdebug.h>
#include <klocale.h>
#include <kstdaccel.h>
#include <kconfig.h>

using namespace SubtitleComposer;

ErrorsView::ErrorsView( const AppConfig& config, LinesView* linesView, QWidget* parent ):
	KListView( parent, "ErrorsView" ),
	m_linesView( linesView ),
	m_errorsConfig( *config.configGroup( "Errors" ) ),
	m_subtitle( 0 ),
	m_linesCount( 0 ),
	m_errorsCount( 0 ),
	m_marksCount( 0 )
{
	addColumn( i18n( "Errors Information" ) );

	setColumnWidthMode( 0, QListView::Maximum );

	header()->setClickEnabled( false );
	setSorting( -1 );

	setAllColumnsShowFocus( true );
	setSelectionMode( QListView::Extended );
	setResizeMode( QListView::LastColumn );
	setRootIsDecorated( true );
	setTreeStepSize( 10 );

	connect( this, SIGNAL( currentChanged(QListViewItem*) ), this, SLOT( onCurrentChanged(QListViewItem*) ) );
}

ErrorsView::~ErrorsView()
{
}

unsigned ErrorsView::linesCount()
{
	return m_linesCount;
}

unsigned ErrorsView::errorsCount()
{
	return m_errorsCount;
}

unsigned ErrorsView::marksCount()
{
	return m_marksCount;
}

void ErrorsView::loadSubtitle( Subtitle* subtitle )
{
	m_subtitle = subtitle;

	connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( syncInsertedLines(unsigned,unsigned) ) );
	connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( syncRemovedLines(unsigned,unsigned) ) );

	setUpdatesEnabled( false );

	int size = m_subtitle->linesCount();

	ErrorsViewItem* after = 0;
	for ( int idx = 0; idx < size; ++idx )
		after = createItem( m_subtitle->line( idx ), after, idx + 1 );

	setUpdatesEnabled( true );
}

void ErrorsView::closeSubtitle()
{
	m_subtitle->disconnect( this );
	m_subtitle = 0;

	clear();
}


void ErrorsView::showCheckErrors()
{
	m_linesView->checkErrors();
}

void ErrorsView::showClearErrors()
{
	m_linesView->clearErrors();
}

void ErrorsView::showSettings()
{
	m_linesView->showErrorsConfig();
}

void ErrorsView::recheckAll()
{
	m_subtitle->recheckErrors(
		Range::full(),
		m_errorsConfig.optionAsInt( OPT_MIN_DURATION ),
		m_errorsConfig.optionAsInt( OPT_MAX_DURATION ),
		m_errorsConfig.optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ),
		m_errorsConfig.optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ),
		m_errorsConfig.optionAsInt( OPT_MAX_CHARACTERS ),
		m_errorsConfig.optionAsInt( OPT_MAX_LINES )
	);
}

void ErrorsView::recheckSelected()
{
	SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Check Lines Errors" ) );

	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ); it.current(); ++it )
	{
		int errorFlags;
		SubtitleLine* line;

		if ( it.current()->parent() )
		{
			NestedErrorsViewItem* item = static_cast<NestedErrorsViewItem*>( it.current() );
			line = item->parent()->line();
			errorFlags = item->errorFlags();
		}
		else
		{
			line = (static_cast<ErrorsViewItem*>( it.current() ))->line();
			errorFlags = line->errorFlags();
		}

		if ( errorFlags & SubtitleLine::EmptyText )
			line->checkEmptyTextError();
		if ( errorFlags & SubtitleLine::OverlappingTimes )
			line->checkOverlappingTimesError();
		if ( errorFlags & SubtitleLine::TooLongDuration )
			line->checkTooLongDurationError( m_errorsConfig.optionAsInt( OPT_MAX_DURATION ) );
		if ( errorFlags & SubtitleLine::TooShortDuration )
			line->checkTooShortDurationError( m_errorsConfig.optionAsInt( OPT_MIN_DURATION ) );
		if ( errorFlags & SubtitleLine::TooLongDurationPerChar )
			line->checkTooLongDurationPerCharError( m_errorsConfig.optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ) );
		if ( errorFlags & SubtitleLine::TooShortDurationPerChar )
			line->checkTooShortDurationPerCharError( m_errorsConfig.optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ) );
		if ( errorFlags & SubtitleLine::TooManyChars )
			line->checkTooManyCharsError( m_errorsConfig.optionAsInt( OPT_MAX_CHARACTERS ) );
		if ( errorFlags & SubtitleLine::TooManyLines )
			line->checkTooManyLinesError( m_errorsConfig.optionAsInt( OPT_MAX_LINES ) );
		if ( errorFlags & SubtitleLine::UnneededSpaces )
			line->checkUnneededSpacesError();
		if ( errorFlags & SubtitleLine::CapitalAfterEllipsis )
			line->checkCapitalAfterEllipsisError();
		if ( errorFlags & SubtitleLine::UnneededDash )
			line->checkUnneededDashError();
	}
}

void ErrorsView::clearAllErrors()
{
	m_subtitle->clearErrors( Range::full(), SubtitleLine::AllErrors );
}

void ErrorsView::clearSelectedErrors()
{
	SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Clear Lines Errors" ) );

	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ); it.current(); ++it )
	{
		int errorFlags;
		SubtitleLine* line;

		if ( it.current()->parent() )
		{
			NestedErrorsViewItem* item = static_cast<NestedErrorsViewItem*>( it.current() );
			line = item->parent()->line();
			errorFlags = item->errorFlags();
		}
		else
		{
			line = (static_cast<ErrorsViewItem*>( it.current() ))->line();
			errorFlags = line->errorFlags();
		}

		line->setErrorFlags( errorFlags, false );
	}
}


void ErrorsView::clearAllMarks()
{
	m_subtitle->setMarked( Range::full(), false );
}

void ErrorsView::clearSelectedMarks()
{
	SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Clear Lines Marks" ) );

	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ); it.current(); ++it )
	{
		if ( it.current()->parent() )
			(static_cast<NestedErrorsViewItem*>( it.current() ))->parent()->line()->setMarked( false );
		else
			(static_cast<ErrorsViewItem*>( it.current() ))->line()->setMarked( false );;
	}
}


void ErrorsView::contentsMouseDoubleClickEvent( QMouseEvent* e )
{
	QListViewItem* item = itemAt( viewport()->mapFromGlobal( e->globalPos() ) );

	onCurrentChanged( item );
}

void ErrorsView::contextMenuEvent( QContextMenuEvent* e )
{
	int lines = 0, errors = 0, marks = 0;

	for ( QListViewItemIterator it( this, QListViewItemIterator::Selected ); it.current(); ++it )
	{
		if ( it.current()->parent() )
		{
			if ( (static_cast<NestedErrorsViewItem*>( it.current() ))->errorFlags() )
				errors++;
			else
				marks++;
		}
		else
		{
			if ( (static_cast<ErrorsViewItem*>( it.current() ))->line()->errorFlags() )
				errors++;
			else
				marks++;
			lines++;
		}

		if ( lines > 1 && errors > 1 && lines > 1 )
			break;
	}

	if ( lines || errors || marks )
	{
		KPopupMenu menu;

		if ( (lines > 1 && marks) || marks > 1 )
			menu.insertItem( i18n( "Clear Marks" ), this, SLOT( clearSelectedMarks() ) );
		else if ( marks )
			menu.insertItem( i18n( "Clear Mark" ), this, SLOT( clearSelectedMarks() ) );

		if ( lines && errors || errors > 1 )
			menu.insertItem( i18n( "Clear Errors" ), this, SLOT( clearSelectedErrors() ) );
		else if ( errors )
			menu.insertItem( i18n( "Clear Error" ), this, SLOT( clearSelectedErrors() ) );

		if ( ! m_errorsConfig.optionAsBool( OPT_AUTO_CLEAR_FIXED ) )
		{
			if ( lines && errors || errors )
				menu.insertItem( i18n( "Recheck and Clear Fixed" ), this, SLOT( recheckSelected() ) );
		}

		menu.exec( e->globalPos() );
		e->ignore();
	}

	KListView::contextMenuEvent( e );
}

void ErrorsView::setCurrentItem( QListViewItem* newCurrentItem, bool clearSelection )
{
	if ( ! newCurrentItem || ! newCurrentItem->isVisible() )
		return;

	QListViewItem* oldCurrentItem = currentItem();

	if ( clearSelection )
		this->clearSelection();
	newCurrentItem->setSelected( true );

	ensureItemVisible( newCurrentItem );

	if ( oldCurrentItem != newCurrentItem && newCurrentItem != oldCurrentItem->parent() )
	{
		KListView::setCurrentItem( newCurrentItem );
		emit currentChanged( newCurrentItem );
	}
}

SubtitleLine* ErrorsView::itemLine( QListViewItem* item )
{
	return item ? (
		item->parent() ?
			(static_cast<ErrorsViewItem*>( item->parent() ))->line() :
			(static_cast<ErrorsViewItem*>( item ))->line()
		) :
		0;
}

SubtitleLine* ErrorsView::currentLine()
{
	return itemLine( currentItem() );
}

void ErrorsView::setCurrentLine( SubtitleLine* line )
{
	QListViewItem* newCurrentItem = item( line->index() );
	if ( ! newCurrentItem )
		return;

	if ( line && line != currentLine() )
		setCurrentItem( newCurrentItem, true );
}

ErrorsViewItem* ErrorsView::item( unsigned targetIndex )
{
	unsigned itemIndex = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++itemIndex )
	{
		if ( itemIndex == targetIndex )
			return static_cast<ErrorsViewItem*>( item );
	}
	return 0;
}

ErrorsViewItem* ErrorsView::createItem( SubtitleLine* line, ErrorsViewItem* after, unsigned number )
{
	ErrorsViewItem* item = new ErrorsViewItem( line, this, after );
	item->setNumber( number ? number : (after ? (after->line()->number() + 1) : 1) );

	return item;
}

void ErrorsView::syncInsertedLines( unsigned firstIndex, unsigned lastIndex )
{
	unsigned index = firstIndex;
	ErrorsViewItem* item = index ? this->item( index - 1 ) : 0;
	for ( ; index <= lastIndex; ++index )
		item = createItem( m_subtitle->line( index ), item, index + 1 );

	// update the remaining lines numbers
	for ( item = item->nextSiblingCasted(); item != 0; item = item->nextSiblingCasted() )
		item->setNumber( ++index );
}

void ErrorsView::syncRemovedLines( unsigned firstIndex, unsigned lastIndex )
{
	ErrorsViewItem *item = this->item( firstIndex ), *itemAux;
	unsigned index = firstIndex;

	for ( ; index <= lastIndex; ++index )
	{
		itemAux = item->nextSiblingCasted();
		delete item;
		item = itemAux;
	}

	QListViewItem* currentItem( this->currentItem() );
	if ( currentItem )
	{
		if ( ! currentItem->isSelected() )
		{
			QListViewItem* itemAbove = currentItem->itemAbove();
			if ( itemAbove && itemAbove->isSelected() )
				setCurrentItem( itemAbove, false );
			else
			{
				QListViewItem* itemBelow = currentItem->itemBelow();
				if ( itemBelow && itemBelow->isSelected() )
					setCurrentItem( itemBelow, false );
			}
		}
	}

	// update the remaining lines numbers
	for ( ; item != 0; item = item->nextSiblingCasted() )
		item->setNumber( ++firstIndex );
}

void ErrorsView::onCurrentChanged( QListViewItem* currentItem )
{
	if ( currentItem != 0 )
	{
		if ( ! currentItem->parent() )
			emit currentLineChanged( (static_cast<ErrorsViewItem*>( currentItem ))->line() );
		else
			emit currentLineChanged( (static_cast<ErrorsViewItem*>( currentItem->parent() ))->line() );
	}
}

void ErrorsView::keyPressEvent( QKeyEvent* event )
{
	if ( event->key() == Qt::Key_Delete )
	{
		SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Check Lines Errors" ) );

		clearSelectedErrors();
		clearSelectedMarks();

		event->accept();
	}
	else
		KListView::keyPressEvent( event );
}

void ErrorsView::updateLinesCount( bool increment )
{
	m_linesCount += (increment ? 1 : -1);
	emit linesCountChanged( m_linesCount );
}

void ErrorsView::updateErrorsCount( bool increment )
{
	m_errorsCount += (increment ? 1 : -1);
	emit errorsCountChanged( m_linesCount );
}

void ErrorsView::updateMarksCount( bool increment )
{
	m_marksCount += (increment ? 1 : -1);
	emit marksCountChanged( m_marksCount );
}

void ErrorsView::updateLineErrors( SubtitleLine* line )
{
	if ( m_errorsConfig.optionAsBool( OPT_AUTO_CLEAR_FIXED ) )
		m_subtitle->recheckErrors(
			Range( line->index() ),
			m_errorsConfig.optionAsInt( OPT_MIN_DURATION ),
			m_errorsConfig.optionAsInt( OPT_MAX_DURATION ),
			m_errorsConfig.optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ),
			m_errorsConfig.optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ),
			m_errorsConfig.optionAsInt( OPT_MAX_CHARACTERS ),
			m_errorsConfig.optionAsInt( OPT_MAX_LINES )
		);
}

LinesErrorsDialog::LinesErrorsDialog( const AppConfig& config, LinesView* linesView, QWidget* parent, const char* name ):
	KDialogBase( parent, name, false, i18n( "Subtitle Errors" ), 0 )
{
	QWidget* mainWidget = new QWidget( this );
	setMainWidget( mainWidget );

	QGridLayout* mainLayout = new QGridLayout( mainWidget );
	mainLayout->setAlignment( Qt::AlignTop );
	mainLayout->setSpacing( 5 );

	m_clearFixedButton = new QPushButton( mainWidget );
	m_clearFixedButton->setText( i18n( "Clear Fixed" ) );
	m_clearFixedButton->setEnabled( ! config.configGroup( "Errors" )->optionAsBool( OPT_AUTO_CLEAR_FIXED ) );

	QPushButton* checkErrorsButton = new QPushButton( mainWidget );
	checkErrorsButton->setText( i18n( "Check Errors..." ) );

	QPushButton* clearErrorsButton = new QPushButton( mainWidget );
	clearErrorsButton->setText( i18n( "Clear Errors..." ) );

	QPushButton* settingsButton = new QPushButton( mainWidget );
	settingsButton->setText( i18n( "Settings..." ) );

	m_errorsView = new ErrorsView( config, linesView, mainWidget );

	m_statsLabel = new QLabel( mainWidget );

	mainLayout->addWidget( m_clearFixedButton, 0, 0 );
	mainLayout->addItem( new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum ), 0, 1 );
	mainLayout->addWidget( checkErrorsButton, 0, 2 );
	mainLayout->addWidget( clearErrorsButton, 0, 3 );
	mainLayout->addWidget( settingsButton, 0, 4 );
	mainLayout->addMultiCellWidget( m_errorsView, 1, 1, 0, 4 );
	mainLayout->addMultiCellWidget( m_statsLabel, 2, 2, 0, 4 );

	resize( 450, height() );

	onStatsChanged();

	connect( m_clearFixedButton, SIGNAL(clicked()), m_errorsView, SLOT(recheckAll()) );
	connect( checkErrorsButton, SIGNAL(clicked()), m_errorsView, SLOT(showCheckErrors()) );
	connect( clearErrorsButton, SIGNAL(clicked()), m_errorsView, SLOT(showClearErrors()) );
	connect( settingsButton, SIGNAL(clicked()), m_errorsView, SLOT(showSettings()) );

	connect( m_errorsView, SIGNAL(linesCountChanged(unsigned)), this, SLOT(onStatsChanged()) );
	connect( m_errorsView, SIGNAL(errorsCountChanged(unsigned)), this, SLOT(onStatsChanged()) );
	connect( m_errorsView, SIGNAL(marksCountChanged(unsigned)), this, SLOT(onStatsChanged()) );

	connect( config.configGroup( "Errors" ), SIGNAL(optionChanged(const QString&, const QString&)),
			 this, SLOT(onOptionChanged(const QString&,const QString&)) );
}

ErrorsView* LinesErrorsDialog::errorsView()
{
	return m_errorsView;
}

void LinesErrorsDialog::onStatsChanged()
{
/*	m_statsLabel->setText(
		i18n( "1 line has errors and/or marks. ", "%n lines have errors and/or marks. ", m_errorsView->linesCount() ) +
		i18n( "1 error total. ", "%n errors total. ", m_errorsView->errorsCount() ) +
		i18n( "1 mark total.", "%n marks total.", m_errorsView->marksCount() )
	);*/

	m_statsLabel->setText(
		i18n( "%1 lines - %2 errors - %3 marks" )
			.arg( m_errorsView->linesCount() )
			.arg( m_errorsView->errorsCount() )
			.arg( m_errorsView->marksCount() )
	);
}

void LinesErrorsDialog::onOptionChanged( const QString& option, const QString& value )
{
	if ( option == OPT_AUTO_CLEAR_FIXED )
		m_clearFixedButton->setEnabled( value != "true" );
}

#include "errorsview.moc"
