/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "errorsview.h"
#include "errorsviewitem.h"
#include "config/errorsconfigdefs.h"
#include "subtitleline.h"

#include <kapplication.h>
#include <kiconloader.h>
#include <klocale.h>

using namespace SubtitleComposer;

NestedErrorsViewItem::NestedErrorsViewItem( SubtitleLine::ErrorID id, ErrorsViewItem* parent ):
	QObject( 0 ),
	KListViewItem( parent ),
	m_id( id )
{
	update();

	setPixmap( 0, id == SubtitleLine::ErrorSIZE ? markPixmap() : errorPixmap() );
}

NestedErrorsViewItem::~NestedErrorsViewItem()
{
}

ErrorsViewItem* NestedErrorsViewItem::parent()
{
	return static_cast<ErrorsViewItem*>( KListViewItem::parent() );
}

int NestedErrorsViewItem::errorFlags()
{
	return SubtitleLine::errorFlag( m_id );
}

void NestedErrorsViewItem::update()
{
	QString text( ErrorsViewItem::errorText( m_id, true ) );

	switch ( m_id )
	{
		case SubtitleLine::TooLongDurationID:
			text = text.arg( parent()->m_line->durationTime().timeMillis() );
			break;
		case SubtitleLine::TooShortDurationID:
			text = text.arg( parent()->m_line->durationTime().timeMillis() );
			break;
		case SubtitleLine::TooLongDurationPerCharID:
			text = text.arg( parent()->m_line->durationTime().timeMillis()/parent()->m_line->characters() );
			break;
		case SubtitleLine::TooShortDurationPerCharID:
			text = text.arg( parent()->m_line->durationTime().timeMillis()/parent()->m_line->characters() );
			break;
		case SubtitleLine::TooManyCharsID:
			text = text.arg( parent()->m_line->characters() );
			break;
		case SubtitleLine::TooManyLinesID:
			text = text.arg( parent()->m_line->lines() );
			break;
		case SubtitleLine::ErrorSIZE:
			text = i18n( "Has user mark." );
			break;
		default:
			break;
	}

	setText( 0, text );
}

const QPixmap& NestedErrorsViewItem::markPixmap()
{
	static QPixmap okPixmap;

	if ( okPixmap.isNull() )
		okPixmap = kapp->iconLoader()->loadIcon( "messagebox_warning", KIcon::Desktop, 16 );

	return okPixmap;
}

const QPixmap& NestedErrorsViewItem::errorPixmap()
{
	static QPixmap errorPixmap;

	if ( errorPixmap.isNull() )
		errorPixmap = kapp->iconLoader()->loadIcon( "messagebox_critical", KIcon::Desktop, 16 );

	return errorPixmap;
}










ErrorsViewItem::ErrorsViewItem( SubtitleLine* line, ErrorsView* parent, ErrorsViewItem* after ):
	QObject( 0 ),
	KListViewItem( parent, after ),
	m_line( line )
{
	for ( int id = 0; id <= SubtitleLine::ErrorSIZE; ++id )
		m_items[id] = 0;

	int errorFlags = m_line->errorFlags();
	bool marked = m_line->marked();

	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
		setupItem( (SubtitleLine::ErrorID)id, errorFlags & SubtitleLine::errorFlag( (SubtitleLine::ErrorID)id ) );
	setupItem( SubtitleLine::ErrorSIZE, marked );

	setVisible( errorFlags || marked );
	if ( isVisible() )
	{
		errorsView()->updateLinesCount( true );
		if ( errorFlags )
			errorsView()->updateErrorsCount( true );
		if ( marked )
			errorsView()->updateMarksCount( true );
	}

	connect( line, SIGNAL( errorFlagsChanged(int) ), this, SLOT( onErrorFlagsChanged(int) ) );
	connect( line, SIGNAL( markChanged(bool) ), this, SLOT( onMarkChanged(bool) ) );

	connect( line, SIGNAL( textChanged(const QString&) ), this, SLOT( onTextChanged(const QString&) ) );
	connect( line, SIGNAL( showTimeChanged(const Time&) ), this, SLOT( onTimesChanged(const Time&) ) );
	connect( line, SIGNAL( hideTimeChanged(const Time&) ), this, SLOT( onTimesChanged(const Time&) ) );
}

ErrorsViewItem::~ErrorsViewItem()
{
}

ErrorsView* ErrorsViewItem::errorsView()
{
	return static_cast<ErrorsView*>( listView() );
}

SubtitleLine* ErrorsViewItem::line()
{
	return m_line;
}

void ErrorsViewItem::setNumber( unsigned number )
{
	setText( 0, i18n( "Line %1" ).arg( number ) );
}

ErrorsViewItem* ErrorsViewItem::nextSiblingCasted()
{
	return static_cast<ErrorsViewItem*>( nextSibling() );
}

void ErrorsViewItem::onTextChanged( const QString& )
{
	if ( m_items[SubtitleLine::EmptyTextID] )
		m_items[SubtitleLine::EmptyTextID]->update();

	if ( m_items[SubtitleLine::TooShortDurationPerCharID] )
		m_items[SubtitleLine::TooShortDurationPerCharID]->update();

	if ( m_items[SubtitleLine::TooLongDurationPerCharID] )
		m_items[SubtitleLine::TooLongDurationPerCharID]->update();

	if ( m_items[SubtitleLine::TooManyCharsID] )
		m_items[SubtitleLine::TooManyCharsID]->update();

	if ( m_items[SubtitleLine::TooManyLinesID] )
		m_items[SubtitleLine::TooManyLinesID]->update();

	if ( m_items[SubtitleLine::UnneededSpacesID] )
		m_items[SubtitleLine::UnneededSpacesID]->update();

	if ( m_items[SubtitleLine::CapitalAfterEllipsisID] )
		m_items[SubtitleLine::CapitalAfterEllipsisID]->update();

	if ( m_items[SubtitleLine::UnneededDashID] )
		m_items[SubtitleLine::UnneededDashID]->update();

	errorsView()->updateLineErrors( m_line );
}

void ErrorsViewItem::onTimesChanged( const Time& )
{
	if ( m_items[SubtitleLine::TooShortDurationID] )
		m_items[SubtitleLine::TooShortDurationID]->update();

	if ( m_items[SubtitleLine::TooLongDurationID] )
		m_items[SubtitleLine::TooLongDurationID]->update();

	if ( m_items[SubtitleLine::TooShortDurationPerCharID] )
		m_items[SubtitleLine::TooShortDurationPerCharID]->update();

	if ( m_items[SubtitleLine::TooLongDurationPerCharID] )
		m_items[SubtitleLine::TooLongDurationPerCharID]->update();

	errorsView()->updateLineErrors( m_line );
}

void ErrorsViewItem::setupItem( SubtitleLine::ErrorID id, bool active )
{
	if ( m_items[id] )
	{
		if ( ! active )
		{
			delete m_items[id];
			m_items[id] = 0;
		}
	}
	else
	{
		if ( active )
		{
			m_items[id] = new NestedErrorsViewItem( id, this );
			setOpen( true );
		}
	}
}

void ErrorsViewItem::onErrorFlagsChanged( int errorFlags )
{
	bool hadErrorFlags = false; // we have to extract information about the previous errorFlags state
	for ( int id = 0; id < SubtitleLine::ErrorSIZE; ++id )
	{
		if ( m_items[id] )
			hadErrorFlags = true;
		setupItem( (SubtitleLine::ErrorID)id, errorFlags & SubtitleLine::errorFlag( (SubtitleLine::ErrorID)id ) );
	}

	bool visible = errorFlags || m_line->marked();
	if ( visible != isVisible() )
	{
		errorsView()->updateLinesCount( visible );
		setVisible( visible );
	}

	bool hasErrorFlags = !!errorFlags;
	if ( hadErrorFlags != hasErrorFlags )
		errorsView()->updateErrorsCount( hasErrorFlags );

}

void ErrorsViewItem::onMarkChanged( bool marked )
{
	bool wasMarked = m_items[SubtitleLine::ErrorSIZE]; // we have to extract information about the previous marked state
	setupItem( SubtitleLine::ErrorSIZE, marked );

	bool visible = m_line->errorFlags() || marked;
	if ( visible != isVisible() )
	{
		errorsView()->updateLinesCount( visible );
		setVisible( visible );
	}

	if ( wasMarked != marked )
		errorsView()->updateMarksCount( marked );
}

const QString& ErrorsViewItem::errorText( SubtitleLine::ErrorFlag errorFlag, bool full )
{
	return errorText( SubtitleLine::errorID( errorFlag ), full );
}

const QString& ErrorsViewItem::errorText( SubtitleLine::ErrorID errorID, bool full )
{
	static QString texts[SubtitleLine::ErrorSIZE];
	static QString texts2[SubtitleLine::ErrorSIZE];

	if ( texts[SubtitleLine::EmptyTextID].isEmpty() )
	{
		texts[SubtitleLine::EmptyTextID]				= i18n( "Empty text" );
		texts[SubtitleLine::OverlappingTimesID]			= i18n( "Overlapping times" );
		texts[SubtitleLine::TooLongDurationID]			= i18n( "Too long duration" );
		texts[SubtitleLine::TooShortDurationID]			= i18n( "Too short duration" );
		texts[SubtitleLine::TooLongDurationPerCharID]	= i18n( "Too long duration per character" );
		texts[SubtitleLine::TooShortDurationPerCharID]	= i18n( "Too short duration per character" );
		texts[SubtitleLine::TooManyCharsID]				= i18n( "Too many characters" );
		texts[SubtitleLine::TooManyLinesID]				= i18n( "Too many lines" );
		texts[SubtitleLine::UnneededSpacesID]			= i18n( "Unnecessary spaces" );
		texts[SubtitleLine::CapitalAfterEllipsisID]		= i18n( "Capital letter after line continuation ellipsis" );
		texts[SubtitleLine::UnneededDashID]				= i18n( "Unnecessary dash" );

		texts2[SubtitleLine::EmptyTextID]				= i18n( "Has no text." );
		texts2[SubtitleLine::OverlappingTimesID]		= i18n( "Overlaps with previous and/or next line." );
		texts2[SubtitleLine::TooLongDurationID]			= i18n( "Has too long duration (%1 msecs)." );
		texts2[SubtitleLine::TooShortDurationID]		= i18n( "Has too short duration (%1 msecs)." );
		texts2[SubtitleLine::TooLongDurationPerCharID]	= i18n( "Has too long duration per character (%1 msecs/char)." );
		texts2[SubtitleLine::TooShortDurationPerCharID]	= i18n( "Has too short duration per character (%1 msecs/char)." );
		texts2[SubtitleLine::TooManyCharsID]			= i18n( "Has too many characters (%1 chars)." );
		texts2[SubtitleLine::TooManyLinesID]			= i18n( "Has too many lines (%1 lines)." );
		texts2[SubtitleLine::UnneededSpacesID]			= i18n( "Has unnecessary spaces." );
		texts2[SubtitleLine::CapitalAfterEllipsisID]	= i18n( "Has capital letter after line continuation ellipsis." );
		texts2[SubtitleLine::UnneededDashID]			= i18n( "Has an unnecessary dash." );
	}

	if ( errorID < 0 || errorID >= SubtitleLine::ErrorSIZE )
		return QString::null;
	else
		return full ? texts2[errorID] : texts[errorID];
}

#include "errorsviewitem.moc"
