/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actions.h"
#include "actionsdialogs.h"
#include "linesview.h"
#include "linesviewitem.h"
#include "errorsview.h"
#include "configdialog.h"
#include "config/generalconfigdefs.h"
#include "config/errorsconfigdefs.h"

#include "subtitle.h"
#include "subtitleline.h"
#include "subtitleiterator.h"
#include <formatsmanager.h>

#include <qtooltip.h>
#include <qpainter.h>
#include <qheader.h>
#include <qlayout.h>
#include <qtextcodec.h>

#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kstdaccel.h>
#include <kcharsets.h>

#include <kurl.h>
#include <kurldrag.h>
#include <kpopupmenu.h>
#include <kmessagebox.h>

#include <kspell.h>
#include <kspelldlg.h>
#include <ksconfig.h>

#include <kfinddialog.h>
#include <kfind.h>
#include <kreplacedialog.h>
#include <kreplace.h>

#include <qbutton.h>

using namespace SubtitleComposer;

LinesView::LinesView( const AppConfig& config, ConfigDialog* configDialog, QWidget* parent ):
	KListView( parent, "ListView" ),
	m_generalConfig( *config.configGroup( "General" ) ),
	m_errorsConfig( *config.configGroup( "Errors" ) ),
	m_subtitle( 0 ),
	m_playingItem( 0 ),
	m_find( 0 ),
	m_replace( 0 ),
	m_findDialog( 0 ),
	m_replaceDialog( 0 ),
	m_spell( 0 ),
	m_configDialog( configDialog ),
	m_errorsDialog( 0 ),
	m_nextItemCurrentOnDelete( true ),
	m_contextMenuMode( false )
{
	addColumn( i18n( "Num" ) );
	addColumn( i18n( "Show Time" ) );
	addColumn( i18n( "Hide Time" ) );
	addColumn( i18n( "Text" ) );

	setColumnWidthMode( Number, QListView::Maximum );
	setColumnWidthMode( ShowTime, QListView::Maximum );
	setColumnAlignment( ShowTime, Qt::AlignHCenter );
	setColumnWidthMode( HideTime, QListView::Maximum );
	setColumnAlignment( HideTime, Qt::AlignHCenter );
	setColumnWidthMode( Text, QListView::Maximum );

	header()->setClickEnabled( false );
	setSorting( -1 );

	setAllColumnsShowFocus( true );
	setSelectionMode( QListView::Extended );
	setResizeMode( QListView::LastColumn );
	setShowToolTips( true );

	connect( this, SIGNAL( selectionChanged() ), this, SLOT( updateActionsState() ) );

	connect( this, SIGNAL( currentChanged(QListViewItem*) ), this, SLOT( onCurrentChanged(QListViewItem*) ) );

	setupActions();

	m_errorsDialog = new LinesErrorsDialog( config, this, this );

	connect( m_errorsDialog->errorsView(), SIGNAL(currentLineChanged(SubtitleLine*)),
			 this, SLOT(setCurrentLine(SubtitleLine*)) );
	connect( this, SIGNAL(currentLineChanged(SubtitleLine*)),
			 m_errorsDialog->errorsView(), SLOT(setCurrentLine(SubtitleLine*)) );
}

LinesView::~LinesView()
{
	delete m_replace;
}

void LinesView::setupActions()
{
	SubtitleActionWrappersManager& manager = SubtitleActionWrappersManager::instance();
	KActionCollection* actionCollection = manager.actionCollection();

	KAction* insertLineBeforeAction = new KAction( actionCollection, "insert_line_before" );
	manager.addAction( new SubtitleActionWrapper( insertLineBeforeAction ) );
	insertLineBeforeAction->setText( i18n( "Insert Before" ) );
	insertLineBeforeAction->setToolTip( i18n( "Insert empty line before current one" ) );
	insertLineBeforeAction->setShortcut( KShortcut( "Ctrl+Insert" ) );
	connect( insertLineBeforeAction, SIGNAL( activated() ), this, SLOT( insertLineBefore() ) );

	KAction* insertLineAfterAction = new KAction( actionCollection, "insert_line_after" );
	manager.addAction( new SubtitleActionWrapper( insertLineAfterAction ) );
	insertLineAfterAction->setText( i18n( "Insert After" ) );
	insertLineAfterAction->setToolTip( i18n( "Insert empty line after current one" ) );
	insertLineAfterAction->setShortcut( KShortcut( "Insert" ) );
	connect( insertLineAfterAction, SIGNAL( activated() ), this, SLOT( insertLineAfter() ) );

	KAction* removeLinesAction = new KAction( actionCollection, "remove_lines" );
	manager.addAction( new SubtitleActionWrapper( removeLinesAction, SubtitleActionWrapper::HasSelectedLine ) );
	removeLinesAction->setText( i18n( "Remove" ) );
	removeLinesAction->setToolTip( i18n( "Remove selected lines" ) );
	removeLinesAction->setShortcut( KShortcut( "Delete" ) );
	connect( removeLinesAction, SIGNAL( activated() ), this, SLOT( removeLines() ) );

	KAction* splitLinesAction = new KAction( actionCollection, "split_lines" );
	manager.addAction( new SubtitleActionWrapper( splitLinesAction, SubtitleActionWrapper::HasSelectedLine ) );
	splitLinesAction->setText( i18n( "Split Lines" ) );
	splitLinesAction->setToolTip( i18n( "Split selected lines" ) );
	connect( splitLinesAction, SIGNAL( activated() ), this, SLOT( splitLines() ) );

	KAction* joinLinesAction = new KAction( actionCollection, "join_lines" );
	manager.addAction( new SubtitleActionWrapper( joinLinesAction, SubtitleActionWrapper::HasSelectedLine ) );
	joinLinesAction->setText( i18n( "Join Lines" ) );
	joinLinesAction->setToolTip( i18n( "Join selected lines" ) );
	connect( joinLinesAction, SIGNAL( activated() ), this, SLOT( joinLines() ) );

	KAction* selectAllLinesAction = new KAction( actionCollection, "select_all_lines" );
	manager.addAction( new SubtitleActionWrapper( selectAllLinesAction, SubtitleActionWrapper::HasLine ) );
	selectAllLinesAction->setText( i18n( "Select All" ) );
	selectAllLinesAction->setToolTip( i18n( "Select all lines" ) );
	selectAllLinesAction->setShortcut( KStdAccel::selectAll() );
	connect( selectAllLinesAction, SIGNAL( activated() ), this, SLOT( selectAllLines() ) );

	KAction* findAction = new KAction( actionCollection, "find" );
	manager.addAction( new SubtitleActionWrapper( findAction, SubtitleActionWrapper::HasLine ) );
	findAction->setText( i18n( "Find..." ) );
	findAction->setToolTip( i18n( "Find occurrences of strings or regular expressions" ) );
	findAction->setIcon( "find" );
	findAction->setShortcut( KStdAccel::find() );
	connect( findAction, SIGNAL( activated() ), this, SLOT( find() ) );

	KAction* findNextAction = new KAction( actionCollection, "find_next" );
	manager.addAction( new SubtitleActionWrapper( findNextAction, SubtitleActionWrapper::HasLine ) );
	findNextAction->setText( i18n( "Find Next" ) );
	findNextAction->setToolTip( i18n( "Find next occurrence of string or regular expression" ) );
	findNextAction->setIcon( "next" );
	findNextAction->setShortcut( KStdAccel::findNext() );
	connect( findNextAction, SIGNAL( activated() ), this, SLOT( findNext() ) );

	KAction* findPreviousAction = new KAction( actionCollection, "find_previous" );
	manager.addAction( new SubtitleActionWrapper( findPreviousAction, SubtitleActionWrapper::HasLine ) );
	findPreviousAction->setText( i18n( "Find Previous" ) );
	findPreviousAction->setToolTip( i18n( "Find previous occurrence of string or regular expression" ) );
	findPreviousAction->setIcon( "previous" );
	findPreviousAction->setShortcut( KStdAccel::findPrev() );
	connect( findPreviousAction, SIGNAL( activated() ), this, SLOT( findPrevious() ) );

	KAction* replaceAction = new KAction( actionCollection, "replace" );
	manager.addAction( new SubtitleActionWrapper( replaceAction, SubtitleActionWrapper::HasLine ) );
	replaceAction->setText( i18n( "Replace..." ) );
	replaceAction->setToolTip( i18n( "Replace occurrences of strings or regular expressions" ) );
	replaceAction->setShortcut( KStdAccel::replace() );
	connect( replaceAction, SIGNAL( activated() ), this, SLOT( replace() ) );

	KAction* gotoLineAction = new KAction( actionCollection, "go_to_line" );
	manager.addAction( new SubtitleActionWrapper( gotoLineAction, SubtitleActionWrapper::HasLine ) );
	gotoLineAction->setText( i18n( "Go to Line..." ) );
	gotoLineAction->setToolTip( i18n( "Go to specified line" ) );
	gotoLineAction->setShortcut( KStdAccel::gotoLine() );
	connect( gotoLineAction, SIGNAL( activated() ), this, SLOT( gotoLine() ) );

	KAction* checkSpellingAction = new KAction( actionCollection, "check_spelling" );
	manager.addAction( new SubtitleActionWrapper( checkSpellingAction, SubtitleActionWrapper::HasLine ) );
	checkSpellingAction->setText( i18n( "Spelling..." ) );
	checkSpellingAction->setToolTip( i18n( "Check lines spelling" ) );
	checkSpellingAction->setIcon( "spellcheck" );
	connect( checkSpellingAction, SIGNAL( activated() ), this, SLOT( checkSpelling() ) );




	KAction* checkErrorsAction = new KAction( actionCollection, "check_errors" );
	manager.addAction( new SubtitleActionWrapper( checkErrorsAction, SubtitleActionWrapper::HasSelectedLine ) );
	checkErrorsAction->setText( i18n( "Check Errors..." ) );
	checkErrorsAction->setToolTip( i18n( "Check for errors in the current subtitle" ) );
	checkErrorsAction->setShortcut( KShortcut( "Ctrl+E" ) );
	connect( checkErrorsAction, SIGNAL( activated() ), this, SLOT( checkErrors() ) );

	KAction* clearErrorsAction = new KAction( actionCollection, "clear_errors" );
	manager.addAction( new SubtitleActionWrapper( clearErrorsAction, SubtitleActionWrapper::HasSelectedLine ) );
	clearErrorsAction->setText( i18n( "Clear Errors..." ) );
	clearErrorsAction->setToolTip( i18n( "Clear errors from selected lines" ) );
	connect( clearErrorsAction, SIGNAL( activated() ), this, SLOT( clearErrors() ) );

	KAction* showErrorsAction = new KAction( actionCollection, "show_errors" );
	manager.addAction( new SubtitleActionWrapper( showErrorsAction, SubtitleActionWrapper::HasLine ) );
	showErrorsAction->setText( i18n( "Show Errors..." ) );
	showErrorsAction->setToolTip( i18n( "Show errors information for the current subtitle" ) );
	showErrorsAction->setShortcut( KShortcut( "Shift+E" ) );
	connect( showErrorsAction, SIGNAL( activated() ), this, SLOT( showErrors() ) );

	KAction* findNextErrorAction = new KAction( actionCollection, "find_next_error" );
	manager.addAction( new SubtitleActionWrapper( findNextErrorAction, SubtitleActionWrapper::HasLine ) );
	findNextErrorAction->setText( i18n( "Find Next Error" ) );
	findNextErrorAction->setToolTip( i18n( "Find next line with errors" ) );
	findNextErrorAction->setShortcut( KShortcut( "F4" ) );
	connect( findNextErrorAction, SIGNAL( activated() ), this, SLOT( findNextError() ) );

	KAction* findPreviousErrorAction = new KAction( actionCollection, "find_previous_error" );
	manager.addAction( new SubtitleActionWrapper( findPreviousErrorAction, SubtitleActionWrapper::HasLine ) );
	findPreviousErrorAction->setText( i18n( "Find Previous Error" ) );
	findPreviousErrorAction->setToolTip( i18n( "Find previous line with errors" ) );
	findPreviousErrorAction->setShortcut( KShortcut( "Shift+F4" ) );
	connect( findPreviousErrorAction, SIGNAL( activated() ), this, SLOT( findPreviousError() ) );

	KAction* toggleLinesMarkAction = new KAction( actionCollection, "toggle_lines_mark" );
	manager.addAction( new SubtitleActionWrapper( toggleLinesMarkAction, SubtitleActionWrapper::HasSelectedLine ) );
	toggleLinesMarkAction->setText( i18n( "Toggle Mark" ) );
	toggleLinesMarkAction->setToolTip( i18n( "Toggle selected lines mark" ) );
	toggleLinesMarkAction->setShortcut( KShortcut( "Ctrl+M" ) );
	connect( toggleLinesMarkAction, SIGNAL( activated() ), this, SLOT( toggleLinesMark() ) );


	KAction* toggleLinesBoldAction = new KAction( actionCollection, "toggle_lines_bold" );
	manager.addAction( new SubtitleActionWrapper( toggleLinesBoldAction, SubtitleActionWrapper::HasSelectedLine ) );
	toggleLinesBoldAction->setText( i18n( "Toggle Bold" ) );
	toggleLinesBoldAction->setIcon( "text_bold" );
	toggleLinesBoldAction->setToolTip( i18n( "Toggle selected lines bold attribute" ) );
	toggleLinesBoldAction->setShortcut( KShortcut( "Ctrl+B" ) );
	connect( toggleLinesBoldAction, SIGNAL( activated() ), this, SLOT( toggleLinesBold() ) );

	KAction* toggleLinesItalicAction = new KAction( actionCollection, "toggle_lines_italic" );
	manager.addAction( new SubtitleActionWrapper( toggleLinesItalicAction, SubtitleActionWrapper::HasSelectedLine ) );
	toggleLinesItalicAction->setText( i18n( "Toggle Italic" ) );
	toggleLinesItalicAction->setIcon( "text_italic" );
	toggleLinesItalicAction->setToolTip( i18n( "Toggle selected lines italic attribute" ) );
	toggleLinesItalicAction->setShortcut( KShortcut( "Ctrl+I" ) );
	connect( toggleLinesItalicAction, SIGNAL( activated() ), this, SLOT( toggleLinesItalic() ) );

	KAction* toggleLinesUnderlineAction = new KAction( actionCollection, "toggle_lines_underline" );
	manager.addAction( new SubtitleActionWrapper( toggleLinesUnderlineAction, SubtitleActionWrapper::HasSelectedLine ) );
	toggleLinesUnderlineAction->setText( i18n( "Toggle Underline" ) );
	toggleLinesUnderlineAction->setIcon( "text_under" );
	toggleLinesUnderlineAction->setToolTip( i18n( "Toggle selected lines underline attribute" ) );
	toggleLinesUnderlineAction->setShortcut( KShortcut( "Ctrl+U" ) );
	connect( toggleLinesUnderlineAction, SIGNAL( activated() ), this, SLOT( toggleLinesUnderline() ) );

	KAction* shiftLinesAction = new KAction( actionCollection, "shift_lines" );
	manager.addAction( new SubtitleActionWrapper( shiftLinesAction, SubtitleActionWrapper::HasLine ) );
	shiftLinesAction->setText( i18n( "Shift Lines..." ) );
	shiftLinesAction->setToolTip( i18n( "Shift lines an specified amount of time" ) );
	shiftLinesAction->setShortcut( KShortcut( "Ctrl+D" ) );
	connect( shiftLinesAction, SIGNAL( activated() ), this, SLOT( shiftLines() ) );

	KAction* adjustLinesAction = new KAction( actionCollection, "adjust_lines" );
	manager.addAction( new SubtitleActionWrapper( adjustLinesAction, SubtitleActionWrapper::HasMultipleLines ) );
	adjustLinesAction->setText( i18n( "Adjust Lines..." ) );
	adjustLinesAction->setToolTip( i18n( "Linearly adjust lines to a specified interval" ) );
	adjustLinesAction->setShortcut( KShortcut( "Ctrl+J" ) );
	connect( adjustLinesAction, SIGNAL( activated() ), this, SLOT( adjustLines() ) );

	KAction* sortLinesAction = new KAction( actionCollection, "sort_lines" );
	manager.addAction( new SubtitleActionWrapper( sortLinesAction, SubtitleActionWrapper::HasMultipleLines ) );
	sortLinesAction->setText( i18n( "Sort Lines..." ) );
	sortLinesAction->setToolTip( i18n( "Sort lines based on their show time" ) );
	connect( sortLinesAction, SIGNAL( activated() ), this, SLOT( sortLines() ) );

	KAction* changeFrameRateAction = new KAction( actionCollection, "change_frame_rate" );
	manager.addAction( new SubtitleActionWrapper( changeFrameRateAction ) );
	changeFrameRateAction->setText( i18n( "Change Frame Rate..." ) );
	changeFrameRateAction->setToolTip( i18n( "Retime subtitle lines by reinterpreting its frame rate" ) );
	connect( changeFrameRateAction, SIGNAL( activated() ), this, SLOT( changeFrameRate() ) );

	KAction* durationLimitsAction = new KAction( actionCollection, "duration_limits" );
	manager.addAction( new SubtitleActionWrapper( durationLimitsAction, SubtitleActionWrapper::HasLine ) );
	durationLimitsAction->setText( i18n( "Duration Limits..." ) );
	durationLimitsAction->setToolTip( i18n( "Enforce lines minimun and/or maximun duration limits" ) );
	durationLimitsAction->setShortcut( KShortcut( "Ctrl+L" ) );
	connect( durationLimitsAction, SIGNAL( activated() ), this, SLOT( durationLimits() ) );

	KAction* autoDurationsAction = new KAction( actionCollection, "automatic_durations" );
	manager.addAction( new SubtitleActionWrapper( autoDurationsAction, SubtitleActionWrapper::HasLine ) );
	autoDurationsAction->setText( i18n( "Automatic Durations..." ) );
	autoDurationsAction->setToolTip( i18n( "Set lines durations based on amount of words and/or line breaks" ) );
	connect( autoDurationsAction, SIGNAL( activated() ), this, SLOT( autoDurations() ) );

	KAction* maximizeDurationsAction = new KAction( actionCollection, "maximize_durations" );
	manager.addAction( new SubtitleActionWrapper( maximizeDurationsAction, SubtitleActionWrapper::HasLine ) );
	maximizeDurationsAction->setText( i18n( "Maximize Durations..." ) );
	maximizeDurationsAction->setToolTip( i18n( "Extend lines durations up to their next lines show time" ) );
	connect( maximizeDurationsAction, SIGNAL( activated() ), this, SLOT( maximizeDurations() ) );

	KAction* syncWithSubtitleAction = new KAction( actionCollection, "sync_with_subtitle" );
	manager.addAction( new SubtitleActionWrapper( syncWithSubtitleAction, SubtitleActionWrapper::HasLine ) );
	syncWithSubtitleAction->setText( i18n( "Synchronize With Subtitle..." ) );
	syncWithSubtitleAction->setToolTip( i18n( "Extend lines durations up to the following line show time" ) );
	connect( syncWithSubtitleAction, SIGNAL( activated() ), this, SLOT( syncWithSubtitle() ) );

	QString shiftMillis( m_generalConfig.option( OPT_SHIFT_SELECTED_LINES_MSECS ) );

	KAction* shiftSelectedLinesFwdAction = new KAction( actionCollection, "shift_selected_lines_forwards" );
	manager.addAction( new SubtitleActionWrapper( shiftSelectedLinesFwdAction, SubtitleActionWrapper::HasSelectedLine ) );
	shiftSelectedLinesFwdAction->setText( i18n( "Shift +%1 Milliseconds" ).arg( shiftMillis ) );
	shiftSelectedLinesFwdAction->setToolTip( i18n( "Shift selected lines +%1 milliseconds" ).arg( shiftMillis ) );
	shiftSelectedLinesFwdAction->setShortcut( KShortcut( "Shift++" ) );
	connect( shiftSelectedLinesFwdAction, SIGNAL( activated() ), this, SLOT( shiftSelectedLinesForwards() ) );

	KAction* shiftSelectedLinesBwdAction = new KAction( actionCollection, "shift_selected_lines_backwards" );
	manager.addAction( new SubtitleActionWrapper( shiftSelectedLinesBwdAction, SubtitleActionWrapper::HasSelectedLine ) );
	shiftSelectedLinesBwdAction->setText( i18n( "Shift -%1 Milliseconds" ).arg( shiftMillis ) );
	shiftSelectedLinesBwdAction->setToolTip( i18n( "Shift selected lines -%1 milliseconds" ).arg( shiftMillis ) );
	shiftSelectedLinesBwdAction->setShortcut( KShortcut( "Shift+-" ) );
	connect( shiftSelectedLinesBwdAction, SIGNAL( activated() ), this, SLOT( shiftSelectedLinesBackwards() ) );

	KAction* editLineTextAction = new KAction( actionCollection, "edit_line_text_inline" );
	manager.addAction( new SubtitleActionWrapper( editLineTextAction, SubtitleActionWrapper::HasSelectedLine ) );
	editLineTextAction->setText( i18n( "Edit Line Inplace" ) );
	editLineTextAction->setToolTip( i18n( "Edit current line in place" ) );
	editLineTextAction->setShortcut( KShortcut( "F2" ) );
	connect( editLineTextAction, SIGNAL( activated() ), this, SLOT( editLineTextInline() ) );

	KAction* smartLinesAdjustAction = new KAction( actionCollection, "smart_lines_adjust" );
	manager.addAction( new SubtitleActionWrapper( smartLinesAdjustAction, SubtitleActionWrapper::HasLine ) );
	smartLinesAdjustAction->setText( i18n( "Smart Lines Adjust..." ) );
	smartLinesAdjustAction->setToolTip( i18n( "Automatically set line breaks" ) );
	connect( smartLinesAdjustAction, SIGNAL( activated() ), this, SLOT( smartLinesAdjust() ) );

	KAction* unbreakLinesAction = new KAction( actionCollection, "unbreak_lines" );
	manager.addAction( new SubtitleActionWrapper( unbreakLinesAction, SubtitleActionWrapper::HasLine ) );
	unbreakLinesAction->setText( i18n( "Unbreak Lines..." ) );
	unbreakLinesAction->setToolTip( i18n( "Remove line breaks from lines" ) );
	connect( unbreakLinesAction, SIGNAL( activated() ), this, SLOT( unbreakLines() ) );

	KAction* simplifyWhiteSpaceAction = new KAction( actionCollection, "simplify_white_space" );
	manager.addAction( new SubtitleActionWrapper( simplifyWhiteSpaceAction, SubtitleActionWrapper::HasLine ) );
	simplifyWhiteSpaceAction->setText( i18n( "Simplify White Space..." ) );
	simplifyWhiteSpaceAction->setToolTip( i18n( "Remove unneeded spaces from lines" ) );
	connect( simplifyWhiteSpaceAction, SIGNAL( activated() ), this, SLOT( simplifySpaces() ) );

	KAction* changeCaseAction = new KAction( actionCollection, "change_case" );
	manager.addAction( new SubtitleActionWrapper( changeCaseAction, SubtitleActionWrapper::HasLine ) );
	changeCaseAction->setText( i18n( "Change Case..." ) );
	changeCaseAction->setToolTip( i18n( "Change lines text to upper, lower, title or sentence case" ) );
	connect( changeCaseAction, SIGNAL( activated() ), this, SLOT( changeCase() ) );

	KAction* fixPunctuationAction = new KAction( actionCollection, "fix_punctuation" );
	manager.addAction( new SubtitleActionWrapper( fixPunctuationAction, SubtitleActionWrapper::HasLine ) );
	fixPunctuationAction->setText( i18n( "Fix Punctuation..." ) );
	fixPunctuationAction->setToolTip( i18n( "Fix punctuation errors in lines" ) );
	connect( fixPunctuationAction, SIGNAL( activated() ), this, SLOT( fixPunctuation() ) );

	updateActionsState();
}

void LinesView::updateActionsState()
{
	SubtitleActionWrappersManager& manager = SubtitleActionWrappersManager::instance();

	manager.setSubtitle( m_subtitle );

	if ( m_subtitle )
		manager.setViewState( childCount(), QListViewItemIterator( this, QListViewItemIterator::Selected ).current() != 0 );
}


void LinesView::loadSubtitle( Subtitle* subtitle )
{
	m_subtitle = subtitle;

	connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( syncInsertedLines(unsigned,unsigned) ) );
	connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( syncRemovedLines(unsigned,unsigned) ) );

	setUpdatesEnabled( false );

	int size = m_subtitle->linesCount();

	LinesViewItem* after = 0;
	for ( int idx = 0; idx < size; ++idx )
		after = createItem( m_subtitle->line( idx ), after, idx + 1 );

	setUpdatesEnabled( true );

	setCurrentItem( firstChild(), true, true );

	updateActionsState();

	if ( m_errorsDialog )
		m_errorsDialog->errorsView()->loadSubtitle( subtitle );
}

void LinesView::closeSubtitle()
{
	m_subtitle->disconnect( this );
	m_subtitle = 0;

	m_playingItem = 0;

	m_find = 0;
	m_findReplaceItem = 0;

	m_spellItem = 0;
	m_spellFirstItem = 0;

	clear();
	updateActionsState();

	if ( m_errorsDialog )
		m_errorsDialog->errorsView()->closeSubtitle();
}

SubtitleLine* LinesView::currentLine()
{
	QListViewItem* currentItem = this->currentItem();
	return currentItem ? (static_cast<LinesViewItem*>( currentItem ))->line() : 0;
}

void LinesView::contentsMouseDoubleClickEvent( QMouseEvent* e )
{
	QListViewItem* item = itemAt( viewport()->mapFromGlobal( e->globalPos() ) );
	if ( item != 0 )
		emit lineDoubleClicked( (static_cast<LinesViewItem*>( item ))->line() );
}

void LinesView::contextMenuEvent( QContextMenuEvent* e )
{
	SubtitleLine* referenceLine = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling() )
		if ( item->isSelected() )
		{
			referenceLine = (static_cast<LinesViewItem*>( item ))->line();
			break;
		}

	if ( ! referenceLine )
		return;

	static KIconLoader iconLoader;


	KPopupMenu textsMenu;
	textsMenu.insertItem( i18n( "Smart Lines Adjust..." ), this, SLOT( smartLinesAdjust() ) );
	textsMenu.insertItem( i18n( "Unbreak Lines" ), this, SLOT( unbreakLines() ) );
	textsMenu.insertItem( i18n( "Simplify White Space" ), this, SLOT( simplifySpaces() ) );
	textsMenu.insertItem( i18n( "Change Case..." ), this, SLOT( changeCase() ) );
	textsMenu.insertItem( i18n( "Fix Punctuation" ), this, SLOT( fixPunctuation() ) );
	textsMenu.insertSeparator();
	textsMenu.insertItem( iconLoader.loadIcon("spellcheck",KIcon::Small), i18n( "Spelling..." ), this, SLOT( checkSpelling() ) );

	KPopupMenu stylesMenu;
	int styleFlags = referenceLine->styleFlags();
	stylesMenu.setCheckable( true );
	stylesMenu.insertItem( iconLoader.loadIcon( "text_bold", KIcon::Small ), i18n( "Bold" ),
							this, SLOT( toggleLinesBold() ), 0, SubtitleLine::Bold );
	stylesMenu.setItemChecked( SubtitleLine::Bold, styleFlags & SubtitleLine::Bold );
	stylesMenu.insertItem( iconLoader.loadIcon( "text_italic", KIcon::Small ), i18n( "Italic" ),
							this, SLOT( toggleLinesItalic() ), 0, SubtitleLine::Italic );
	stylesMenu.setItemChecked( SubtitleLine::Italic, styleFlags & SubtitleLine::Italic );
	stylesMenu.insertItem( iconLoader.loadIcon( "text_under", KIcon::Small ), i18n( "Underline" ),
							this, SLOT( toggleLinesUnderline() ), 0, SubtitleLine::Underline );
	stylesMenu.setItemChecked( SubtitleLine::Underline, styleFlags & SubtitleLine::Underline );

	KPopupMenu timesMenu;
	QString shiftMillis( m_generalConfig.option( OPT_SHIFT_SELECTED_LINES_MSECS ) );
	timesMenu.insertItem( i18n( "Shift +%1 Milliseconds" ).arg( shiftMillis ), this, SLOT( shiftSelectedLinesBackwards() ) );
	timesMenu.insertItem( i18n( "Shift -%1 Milliseconds" ).arg( shiftMillis ), this, SLOT( shiftSelectedLinesForwards() ) );
	timesMenu.insertSeparator();
	timesMenu.insertItem( i18n( "Shift Lines..." ), this, SLOT( shiftLines() ) );
	timesMenu.insertItem( i18n( "Sort Lines" ), this, SLOT( sortLines() ) );
	timesMenu.insertItem( i18n( "Duration Limits..." ), this, SLOT( durationLimits() ) );
	timesMenu.insertItem( i18n( "Automatic Durations..." ), this, SLOT( autoDurations() ) );
	timesMenu.insertItem( i18n( "Maximize Durations" ), this, SLOT( maximizeDurations() ) );

	KPopupMenu errorsMenu;
	errorsMenu.setCheckable( true );
	errorsMenu.insertItem( i18n( "Mark" ), this, SLOT( toggleLinesMark() ), 0, 0 );
	errorsMenu.setItemChecked( 0, referenceLine->marked() );
	errorsMenu.insertSeparator();
	errorsMenu.insertItem( i18n( "Check Errors..." ), this, SLOT( checkErrors() ) );
	errorsMenu.insertItem( i18n( "Clear Errors..." ), this, SLOT( clearErrors() ) );
	errorsMenu.insertSeparator();
	errorsMenu.insertItem( i18n( "Show Errors..." ), this, SLOT( showErrors() ) );

	KPopupMenu menu;
	menu.setCheckable( true );
	menu.insertItem( i18n( "Select All" ), this, SLOT( selectAllLines() ) );
	menu.insertSeparator();
	menu.insertItem( i18n( "Remove" ), this, SLOT( removeLines() ) );
	menu.insertItem( i18n( "Insert Before" ), this, SLOT( insertLineBefore() ) );
	menu.insertItem( i18n( "Insert After" ), this, SLOT( insertLineAfter() ) );
	menu.insertSeparator();
	menu.insertItem( i18n( "Join Lines" ), this, SLOT( joinLines() ) );
	menu.insertItem( i18n( "Split Lines" ), this, SLOT( splitLines() ) );
	menu.insertSeparator();
	menu.insertItem( i18n( "Texts" ), &textsMenu );
// 	menu.insertSeparator();
	menu.insertItem( i18n( "Styles" ), &stylesMenu );
// 	menu.insertSeparator();
	menu.insertItem( i18n( "Times" ), &timesMenu );
// 	menu.insertSeparator();
	menu.insertItem( i18n( "Errors" ), &errorsMenu );


	m_contextMenuMode = true;
	menu.exec( e->globalPos() );
	e->ignore();
	m_contextMenuMode = false;

	KListView::contextMenuEvent( e );
}

void LinesView::editLineTextInline()
{
	currentItem()->setRenameEnabled( Text, true );
	currentItem()->startRename( Text );
}


void LinesView::insertLineBefore()
{
	QListViewItem* currentItem = this->currentItem();
	int index;
	if ( currentItem != 0 )
	{
		index = itemIndex( currentItem );
		currentItem->setSelected( false );
	}
	else
		index = 0;

	SubtitleLine* line = new SubtitleLine();
	if ( index < (signed)m_subtitle->linesCount() )
		line->setShowTime( m_subtitle->line( index )->showTime() - 1100 );
	line->setDurationTime( 1000 );

	m_subtitle->insertLine( line, index );

	updateActionsState();
}

void LinesView::insertLineAfter()
{
	QListViewItem* currentItem = this->currentItem();
	int index;
	if ( currentItem != 0 )
	{
		index = itemIndex( currentItem ) + 1;
		currentItem->setSelected( false );
	}
	else
		index = 0;

	SubtitleLine* line = new SubtitleLine();
	if ( index > 0 )
		line->setShowTime( m_subtitle->line( index-1 )->hideTime() + 100 );
	line->setDurationTime( 1000 );

	m_subtitle->insertLine( line, index );

	updateActionsState();
}

RangeList LinesView::selectionRanges()
{
	RangeList ranges;

	int firstIndex = -1, index = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++index )
	{
		if ( item->isSelected() )
		{
			if ( firstIndex == -1 ) // mark start of selected range
				firstIndex = index;
		}
		else
		{
			if ( firstIndex != -1 )
			{
				ranges << Range( firstIndex, index - 1 );
				firstIndex = -1;
			}
		}
	}

	if ( firstIndex != -1 )
		ranges << Range( firstIndex, index - 1 );

	return ranges;
}

void LinesView::removeLines()
{
	m_subtitle->removeLines( selectionRanges() );

	updateActionsState();
}

void LinesView::joinLines()
{
	m_nextItemCurrentOnDelete = false;

	const RangeList& ranges = selectionRanges();

// 	if ( ranges.count() > 1 && KMessageBox::Continue != KMessageBox::warningContinueCancel(
// 		this,
// 		i18n( "Current selection has multiple ranges.\nContinuing will join them all." ),
// 		i18n( "Join Lines" ) ) )
// 		return;

	m_subtitle->joinLines( ranges );

	updateActionsState();

	m_nextItemCurrentOnDelete = true;
}

void LinesView::splitLines()
{
	m_subtitle->splitLines( selectionRanges() );

	updateActionsState();
}


void LinesView::selectAllLines()
{
	selectAll( true );
}

void LinesView::setCurrentItem( QListViewItem* newCurrentItem, bool clearSelection, bool force )
{
	if ( ! newCurrentItem || /*m_find || m_replace ||*/ m_spell )
		return; // weird behavior when we change the current item and a modal dialog is shown

	if ( clearSelection )
		this->clearSelection();
	newCurrentItem->setSelected( true );

	ensureItemVisible( newCurrentItem );

	if ( force || newCurrentItem != currentItem() )
	{
		KListView::setCurrentItem( newCurrentItem );
		emit currentChanged( newCurrentItem );
	}
}

void LinesView::setCurrentLine( SubtitleLine* line )
{
	if ( line )
		setCurrentItem( item( line->index() ), true );
}


void LinesView::setCurrentLineShowTime( const Time& showTime )
{
	LinesViewItem* currentItem = static_cast<LinesViewItem*>( this->currentItem() );
	if ( currentItem )
		currentItem->line()->setShowTime( showTime - m_generalConfig.optionAsInt( OPT_SET_CURRENT_LINE_TIME_COMP_MSECS ) );
}

void LinesView::setCurrentLineHideTime( const Time& hideTime )
{
	LinesViewItem* currentItem = static_cast<LinesViewItem*>( this->currentItem() );
	if ( currentItem )
	{
		currentItem->line()->setHideTime( hideTime - m_generalConfig.optionAsInt( OPT_SET_CURRENT_LINE_TIME_COMP_MSECS ) );
		QListViewItem* nextItem = currentItem->nextSibling();
		if ( nextItem )
			setCurrentItem( nextItem, true );
	}
}

void LinesView::adjustFromCurrentLine( const Time& currentLineShowTime )
{
	SubtitleLine* currentLine = this->currentLine();
	if ( currentLine )
	{
		m_subtitle->adjustLines(
			Range::higher( currentLine->index() ),
			currentLineShowTime,
			m_subtitle->lastLine()->showTime()
		);
	}
}

void LinesView::adjustUpToCurrentLine( const Time& currentLineShowTime )
{
	SubtitleLine* currentLine = this->currentLine();
	if ( currentLine )
	{
		m_subtitle->adjustLines(
			Range::lower( currentLine->index() ),
			m_subtitle->firstLine()->showTime(),
			currentLineShowTime
		);
	}
}

void LinesView::shiftFromCurrentLine( const Time& currentLineShowTime )
{
	SubtitleLine* currentLine = this->currentLine();
	if ( currentLine )
	{
		m_subtitle->shiftLines(
			Range::higher( currentLine->index() ),
			currentLineShowTime.timeMillis() - currentLine->showTime().timeMillis()
		);
	}
}

void LinesView::shiftUpToCurrentLine( const Time& currentLineShowTime )
{
	SubtitleLine* currentLine = this->currentLine();
	if ( currentLine )
	{
		m_subtitle->shiftLines(
			Range::lower( currentLine->index() ),
			currentLineShowTime.timeMillis() - currentLine->showTime().timeMillis()
		);
	}
}


void LinesView::setPlayingLine( SubtitleLine* line )
{
	if ( m_playingItem && m_playingItem->line() == line )
		return;

	if ( m_playingItem )
		m_playingItem->setPlaying( false );

	m_playingItem = line ? item( line->number() - 1 ) : 0;
	if ( m_playingItem )
		m_playingItem->setPlaying( true );
}

void LinesView::toggleLinesMark()
{
	m_subtitle->toggleMarked( selectionRanges() );
}

void LinesView::toggleLinesBold()
{
	m_subtitle->toggleStyleFlag( selectionRanges(), SubtitleLine::Bold );
}

void LinesView::toggleLinesItalic()
{
	m_subtitle->toggleStyleFlag( selectionRanges(), SubtitleLine::Italic );
}

void LinesView::toggleLinesUnderline()
{
	m_subtitle->toggleStyleFlag( selectionRanges(), SubtitleLine::Underline );
}

LinesViewItem* LinesView::nextItem( LinesViewItem* item, const QString& caption, bool backwards, bool* circled )
{
	if ( backwards )
	{
		if ( (item = static_cast<LinesViewItem*>( item->itemAbove() )) == 0 )
		{
			if ( KMessageBox::warningContinueCancel(
					this,
					i18n( "Beginning of subtitle reached.\nContinue from the end?" ),
					caption
				 ) != KMessageBox::Continue )
				return 0;
			item = static_cast<LinesViewItem*>( lastItem() );
			if ( circled )
				*circled = true;
		}
	}
	else
	{
		if ( (item = item->nextSiblingCasted()) == 0 )
		{
			if ( KMessageBox::warningContinueCancel(
					this,
					i18n( "End of subtitle reached.\nContinue from the beginning?" ),
					caption
				 ) != KMessageBox::Continue )
				return 0;
			item = static_cast<LinesViewItem*>( firstChild() );
			if ( circled )
				*circled = true;
		}
	}

	return item;
}

void LinesView::iterateFind()
{
	LinesViewItem* findStartItem = static_cast<LinesViewItem*>( currentItem() );
	m_findReplaceItem = findStartItem;
	bool circled = false;
	bool backwards = m_find->options() & KFindDialog::FindBackwards;
	KFind::Result res = KFind::NoMatch;

	do
	{
		if ( m_find->needData() )
			m_find->setData( m_findReplaceItem->line()->text() );

		res = m_find->find();

		if ( res == KFind::NoMatch )
		{
			m_findReplaceItem = nextItem( m_findReplaceItem, i18n( "Find" ), backwards, &circled );
			if ( m_findReplaceItem == 0 ) // arrived to end and user requested to stop
				break;
		}
		else
			m_findFound = true;
	}
	while ( res == KFind::NoMatch && (m_findReplaceItem != findStartItem || ! circled) );

	if ( m_findReplaceItem == findStartItem && ! m_findFound && m_findReplaceItem != 0 )
		KMessageBox::sorry( this, i18n( "No instances of '%1' found!" ).arg( m_find->pattern() ), i18n( "Find" ) );
}

void LinesView::find( bool backwards )
{
	if ( ! m_findDialog )
		m_findDialog = new KFindDialog( this, "FindDialog" );

	m_findDialog->setOptions(
		backwards ?
			m_findDialog->options() | KFindDialog::FindBackwards :
			m_findDialog->options() & ~KFindDialog::FindBackwards
	);

	if ( m_findDialog->exec() != QDialog::Accepted )
		return;

	if ( ! m_find )
	{
		m_find = new KFind( m_findDialog->pattern(), m_findDialog->options(), this );
		m_find->closeFindNextDialog();
		connect( m_find, SIGNAL(highlight(const QString&,int,int)), this, SLOT(onHighlightText(const QString&,int,int)) );
	}
	else
	{
		m_find->setPattern( m_findDialog->pattern() );
		m_find->setOptions( m_findDialog->options() );
	}

	m_findFound = false;
	iterateFind();
}

void LinesView::findNext()
{
	if ( ! m_find )
		find( false );
	else
	{
		m_find->setOptions( m_find->options() & ~KFindDialog::FindBackwards );
		iterateFind();
	}
}

void LinesView::findPrevious()
{
	if ( ! m_find )
		find( true );
	else
	{
		m_find->setOptions( m_find->options() | KFindDialog::FindBackwards );
		iterateFind();
	}
}

void LinesView::iterateReplace()
{
	LinesViewItem* replaceStartItem = static_cast<LinesViewItem*>( currentItem() );
	m_findReplaceItem = replaceStartItem;
	bool circled = false;
	KFind::Result res = KFind::NoMatch;

	do
	{
		if ( m_replace->needData() )
			m_replace->setData( m_findReplaceItem->line()->text() );

		res = m_replace->replace();

		if ( res == KFind::NoMatch )
		{
			m_findReplaceItem = nextItem( m_findReplaceItem, i18n( "Replace" ), false, &circled );
			if ( m_findReplaceItem == 0 ) // arrived to end and user requested to stop
				break;
		}
	}
	while ( res == KFind::NoMatch && (m_findReplaceItem != replaceStartItem || ! circled) );
}

void LinesView::onHighlightText( const QString&, int matchingIndex, int matchedLength )
{
	setCurrentItem( m_findReplaceItem, true );

	emit currentLineTextHighlighted( matchingIndex, matchingIndex + matchedLength - 1 );
}

void LinesView::onReplaceText( const QString& text, int, int, int )
{
	m_findReplaceItem->line()->setText( text );
}

void LinesView::replace()
{
	if ( ! m_replaceDialog )
		m_replaceDialog = new KReplaceDialog( this, "ReplaceDialog" );

	if ( m_replaceDialog->exec() != QDialog::Accepted )
		return;

	if ( m_replace )
		delete m_replace;
	m_replace = new KReplace( m_replaceDialog->pattern(), m_replaceDialog->replacement(), m_replaceDialog->options(), 0 );

	// Connect findNext signal - called when pressing the button in the dialog
	connect( m_replace, SIGNAL(findNext()),
			 this, SLOT(iterateReplace()) );

	// Connect signals to code which handles highlighting of found text, and on-the-fly replacement.
	connect( m_replace, SIGNAL(highlight( const QString&, int, int )),
			 this, SLOT(onHighlightText( const QString&, int, int )) );

	// Connect replace signal - called when doing a replacement
	connect( m_replace, SIGNAL(replace(const QString&, int, int, int)),
			 this, SLOT(onReplaceText(const QString&, int, int, int)) );

	iterateReplace();
}

void LinesView::gotoLine()
{
	GotoLineDialog gotoLineDlg( itemIndex( currentItem() ) + 1, childCount() );
	if ( gotoLineDlg.exec() == QDialog::Accepted )
		setCurrentItem( item( gotoLineDlg.lineNumber() - 1 ), true );
}

void LinesView::checkSpelling()
{
	KSpellConfig* config = new KSpellConfig( this, "KSpellConfig" );
// 	config->setDictionary( m_generalConfig.option( OPT_SPELL_CHECK_DICT ) );
	m_spell = new KSpell( this, i18n( "Spell Checking" ), this, SLOT(onSpellCheckReady()), config, true, true );

	connect( m_spell, SIGNAL(done(const QString&)), this, SLOT(onSpellCheckDone(const QString&)) );

	connect( m_spell, SIGNAL(death()), this, SLOT(onSpellCheckCleanup()) );

	connect( m_spell, SIGNAL(misspelling(const QString&, const QStringList&, unsigned int) ),
			 this, SLOT(onWordMisspelled(const QString&, const QStringList&, unsigned int)) );

	connect( m_spell, SIGNAL(corrected(const QString&, const QString&, unsigned int)),
			 this, SLOT(onWordCorrected(const QString&, const QString&, unsigned int)) );
}

void LinesView::onSpellCheckReady()
{
	m_spellFirstItem = static_cast<LinesViewItem*>( currentItem() );
	m_spellItem = m_spellFirstItem;

	m_spell->check( m_spellItem->line()->text() );
}

void LinesView::onWordMisspelled( const QString& text, const QStringList&, unsigned int pos )
{
	setCurrentItem( m_spellItem, true );

	emit currentLineTextHighlighted( pos, pos + text.length() - 1 );
}

void LinesView::onWordCorrected( const QString& oldWord, const QString& newWord, unsigned int pos )
{
	m_spellItem->line()->setText( m_spellItem->line()->text().replace( pos, oldWord.length(), newWord ) );
}

void LinesView::onSpellCheckDone( const QString& )
{
	if ( m_spell->dlgResult() == KS_CANCEL || m_spell->dlgResult() == KS_STOP )
		m_spell->cleanUp();
	else
	{
		if ( (m_spellItem = nextItem( m_spellItem, i18n( "Spelling" ) )) == 0 )
		{
			// arrived to end
			m_spell->cleanUp();
			return;
		}

		if ( m_spellItem == m_spellFirstItem ) // we have already checked all the lines
			m_spell->cleanUp();
		else
			m_spell->check( m_spellItem->line()->text() );
	}
}

void LinesView::onSpellCheckCleanup()
{
	delete m_spell;
	m_spell = 0;
}


void LinesView::checkErrors()
{
	static CheckErrorsDialog* dlg = new CheckErrorsDialog( this );

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Check Lines Errors" ) );

		if ( dlg->clearOtherErrors() )
			m_subtitle->clearErrors(
				dlg->allLines() ?
					Range::full() :
					selectionRanges(),
				SubtitleLine::AllErrors & ~dlg->errorsToClear()
			);

		if ( dlg->clearMarks() )
			m_subtitle->setMarked(
				dlg->allLines() ?
					Range::full() :
					selectionRanges(),
				false
			);

		m_subtitle->checkErrors(
			dlg->allLines() ?
				Range::full() :
				selectionRanges(),
			dlg->errorsToClear(),
			m_errorsConfig.optionAsInt( OPT_MIN_DURATION ),
			m_errorsConfig.optionAsInt( OPT_MAX_DURATION ),
			m_errorsConfig.optionAsInt( OPT_MIN_DURATION_PER_CHARACTER ),
			m_errorsConfig.optionAsInt( OPT_MAX_DURATION_PER_CHARACTER ),
			m_errorsConfig.optionAsInt( OPT_MAX_CHARACTERS ),
			m_errorsConfig.optionAsInt( OPT_MAX_LINES )
		);
	}
}

void LinesView::clearErrors()
{
	static ClearErrorsDialog* dlg = new ClearErrorsDialog( this );

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		SubtitleCompositeActionExecutor executor( *m_subtitle, i18n( "Clear Lines Errors" ) );

		if ( dlg->clearMarks() )
			m_subtitle->setMarked(
				dlg->allLines() ?
					Range::full() :
					selectionRanges(),
				false
			);

		if ( dlg->errorsToClear() )
			m_subtitle->clearErrors(
				dlg->allLines() ?
					Range::full() :
					selectionRanges(),
				dlg->errorsToClear()
			);
	}
}

void LinesView::showErrors()
{
	if ( ! m_errorsDialog->isShown() )
		m_errorsDialog->show();
}

void LinesView::showErrorsConfig()
{
	m_configDialog->show();
	m_configDialog->showPage( ConfigDialog::Errors );
}


void LinesView::iterateFindError( bool backwards )
{
	LinesViewItem* findErrorStartItem = static_cast<LinesViewItem*>( currentItem() );
	LinesViewItem* findErrorItem = findErrorStartItem;

	bool circled = false;
	bool foundError = false;

	do
	{
		findErrorItem = nextItem( findErrorItem, i18n( "Find Error" ), backwards, &circled );
		if ( findErrorItem == 0 ) // arrived to end and user requested to stop
			break;

		if ( foundError = (findErrorItem->line()->errorFlags() || findErrorItem->line()->marked()))
			setCurrentItem( findErrorItem, true );
	}
	while ( (! foundError) && (findErrorItem != findErrorStartItem || ! circled) );

	if ( findErrorItem == findErrorStartItem && ! foundError && findErrorItem != 0 )
		KMessageBox::sorry( this, i18n( "No line with errors found!" ).arg( m_find->pattern() ), i18n( "Find Error" ) );
}

void LinesView::findNextError()
{
	iterateFindError( false );
}

void LinesView::findPreviousError()
{
	iterateFindError( true );
}


void LinesView::shiftLines()
{
	static ShiftLinesDialog* dlg = new ShiftLinesDialog( this );

	dlg->resetShiftTime();
	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->shiftLines(
			dlg->allLines() ?
				Range::full() :
				selectionRanges(),
			dlg->shiftTimeMillis()
		);
	}
}

void LinesView::adjustLines()
{
	static AdjustLinesDialog* dlg = new AdjustLinesDialog( this );

	dlg->setFirstLineTime( m_subtitle->firstLine()->showTime() );
	dlg->setLastLineTime( m_subtitle->lastLine()->showTime() );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->adjustLines(
			Range::full(),
			dlg->firstLineTime(),
			dlg->lastLineTime()
		);
	}
}

void LinesView::sortLines()
{
	static SelectableTargetActionDialog* dlg = new SelectableTargetActionDialog(
		i18n( "Sort Lines" ),
		QString::null,
		this
	);

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->sortLines(
			dlg->allLines() ?
				Range::full() :
				selectionRanges()
		);
	}
}


void LinesView::changeFrameRate()
{
	static ChangeFrameRateDialog* dlg = new ChangeFrameRateDialog( m_subtitle->framesPerSecond(), this );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->changeFramesPerSecond(
			dlg->toFramesPerSecond(),
			dlg->fromFramesPerSecond()
		);
	}
}

void LinesView::durationLimits()
{
	static DurationLimitsDialog* dlg = new DurationLimitsDialog(
		m_errorsConfig.optionAsInt( OPT_MIN_DURATION ),
		m_errorsConfig.optionAsInt( OPT_MAX_DURATION ),
		this
	);

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->applyDurationLimits(
			dlg->allLines() ? Range::full() : selectionRanges(),
			dlg->enforceMinDuration() ? dlg->minDuration() : 0,
			dlg->enforceMaxDuration() ? dlg->maxDuration() : 86399999,
			! dlg->preventOverlap()
		);
	}
}

void LinesView::autoDurations()
{
	static AutoDurationsDialog* dlg = new AutoDurationsDialog( 60, 50, 50, this );

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->setAutoDurations(
			dlg->allLines() ? Range::full() : selectionRanges(),
			dlg->charMillis(),
			dlg->wordMillis(),
			dlg->lineMillis(),
			! dlg->preventOverlap()
		);
	}
}

void LinesView::maximizeDurations()
{
	static SelectableTargetActionDialog* dlg = new SelectableTargetActionDialog(
		i18n( "Maximize Durations" ),
		i18n( "Maximize durations in" ),
		this
	);

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->setMaximunDurations(
			dlg->allLines() ?
				Range::full() :
				selectionRanges()
		);
	}
}


void LinesView::syncWithSubtitle()
{
	static SynchronizeWithSubtitleDialog* dlg = new SynchronizeWithSubtitleDialog(
		m_generalConfig.option( OPT_DEFAULT_SUBS_ENCODING ),
		this
	);

	if ( dlg->exec() == QDialog::Accepted )
	{

		bool codecFound = true;
		QTextCodec* codec = KGlobal::charsets()->codecForName( dlg->subtitleEncoding(), codecFound );
		if ( ! codecFound )
			codec = KGlobal::locale()->codecForEncoding();

		Subtitle referenceSubtitle;
		if ( FormatsManager::readSubtitle( referenceSubtitle, dlg->subtitlePath(), codec ) )
		{
			if ( dlg->adjustToReferenceSubtitle() )
			{
				if ( referenceSubtitle.linesCount() <= 1 )
					KMessageBox::sorry( this, i18n( "The reference subtitle must have more than one line to proceed." ) );
				else
					m_subtitle->adjustLines(
						Range::full(),
						referenceSubtitle.firstLine()->showTime(),
						referenceSubtitle.lastLine()->showTime()
					);
			}
			else // if ( dlg->synchronizeToReferenceTimes() )
				m_subtitle->syncToSubtitle( referenceSubtitle );
		}
		else
			KMessageBox::sorry( this, i18n( "Couldn't parse the reference subtitle file." ) );
	}
}


void LinesView::shiftSelectedLinesForwards()
{
	m_subtitle->shiftLines(
		selectionRanges(),
		m_generalConfig.optionAsInt( OPT_SHIFT_SELECTED_LINES_MSECS )
	);
}

void LinesView::shiftSelectedLinesBackwards()
{
	m_subtitle->shiftLines(
		selectionRanges(),
		- m_generalConfig.optionAsInt( OPT_SHIFT_SELECTED_LINES_MSECS )
	);
}


void LinesView::smartLinesAdjust()
{
	static SmartLinesAdjustDialog* dlg = new SmartLinesAdjustDialog( 30, this );

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->adjustTexts(
			dlg->allLines() ?
				Range::full() :
				selectionRanges(),
			dlg->minLengthForLineBreak()
		);
	}
}

void LinesView::unbreakLines()
{
	static SelectableTargetActionDialog* dlg = new SelectableTargetActionDialog(
		i18n( "Unbreak Lines" ),
		i18n( "Unbreak" ),
		this
	);

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->unbreakTexts(
			dlg->allLines() ?
				Range::full() :
				selectionRanges()
		);
	}
}

void LinesView::simplifySpaces()
{
	static SelectableTargetActionDialog* dlg = new SelectableTargetActionDialog(
		i18n( "Simplify Spaces" ),
		i18n( "Simplify spaces in" ),
		this
	);

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->simplifySpaces(
			dlg->allLines() ?
				Range::full() :
				selectionRanges()
		);
	}
}

void LinesView::changeCase()
{
	static ChangeLinesTextCaseDialog* dlg = new ChangeLinesTextCaseDialog( this );

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->changeCase(
			dlg->allLines() ?
				Range::full() :
				selectionRanges(),
			dlg->caseOperation(),
			dlg->lowerFirst()
		);
	}
}

void LinesView::fixPunctuation()
{
	static FixLinesPunctuationDialog* dlg = new FixLinesPunctuationDialog( this );

	dlg->setSelectedLinesOnlyMode( m_contextMenuMode );

	if ( dlg->exec() == QDialog::Accepted )
	{
		m_subtitle->fixPunctuation(
			dlg->allLines() ?
				Range::full() :
				selectionRanges(),
			dlg->spaces(),
			dlg->quotes(),
			dlg->englishI(),
			dlg->ellipisis()
		);
	}
}

int LinesView::itemIndex( QListViewItem* targetItem )
{
	int itemIndex = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++itemIndex )
	{
		if ( item == targetItem )
			return itemIndex;
	}
	return -1;
}

LinesViewItem* LinesView::item( unsigned targetIndex )
{
	unsigned itemIndex = 0;
	for ( QListViewItem* item = firstChild(); item != 0; item = item->nextSibling(), ++itemIndex )
	{
		if ( itemIndex == targetIndex )
			return static_cast<LinesViewItem*>( item );
	}
	return 0;
}

LinesViewItem* LinesView::createItem( SubtitleLine* line, LinesViewItem* after, unsigned number )
{
	LinesViewItem* item = new LinesViewItem( line, this, after );
	item->updateNumber( number ? number : (after ? (after->line()->number() + 1) : 1) );
	return item;
}

void LinesView::syncInsertedLines( unsigned firstIndex, unsigned lastIndex )
{
	unsigned index = firstIndex;
	LinesViewItem* item = index ? this->item( index - 1 ) : 0;
	for ( ; index <= lastIndex; ++index )
	{
		item = createItem( m_subtitle->line( index ), item, index + 1 );

		setSelected( item, true );
		if ( index == lastIndex )
			setCurrentItem( item, false );
	}

	// update the remaining lines numbers
	for ( item = item->nextSiblingCasted(); item != 0; item = item->nextSiblingCasted() )
		item->updateNumber( ++index );

	updateActionsState();
}

void LinesView::syncRemovedLines( unsigned firstIndex, unsigned lastIndex )
{
	LinesViewItem *item = this->item( firstIndex ), *itemAux;
	unsigned index = firstIndex;

	for ( ; index <= lastIndex; ++index )
	{
		itemAux = item->nextSiblingCasted();
		if ( item == m_playingItem )
			m_playingItem = 0;

		delete item;
		item = itemAux;
	}

	QListViewItem* currentItem( this->currentItem() );
	if ( currentItem )
	{
		if ( m_nextItemCurrentOnDelete )
			setCurrentItem( currentItem, false );
		else if ( ! currentItem->isSelected() )
		{
			QListViewItem* itemAbove = currentItem->itemAbove();
			if ( itemAbove && itemAbove->isSelected() )
				setCurrentItem( itemAbove, false );
			else
			{
				QListViewItem* itemBelow = currentItem->itemBelow();
				if ( itemBelow && itemBelow->isSelected() )
					setCurrentItem( itemBelow, false );
			}
		}
	}

	// update the remaining lines numbers
	for ( ; item != 0; item = item->nextSiblingCasted() )
		item->updateNumber( ++firstIndex );

	updateActionsState();
}

void LinesView::onCurrentChanged( QListViewItem* currentItem )
{
	emit currentLineChanged( currentItem ? (static_cast<LinesViewItem*>( currentItem ))->line() : 0 );
}

void LinesView::loadConfig()
{
	KConfigGroupSaver saver( kapp->config(), "ListView Settings" );

	restoreLayout( kapp->config(), "ListView Settings" );

	for ( int idx = 0; idx < columns(); ++idx )
		if ( columnWidth( idx ) == 0 )
			header()->setResizeEnabled( false, idx );
}

void LinesView::saveConfig()
{
	KConfigGroupSaver saver( kapp->config(), "ListView Settings" );

	saveLayout( kapp->config(), "ListView Settings" );
}

#include "linesview.moc"
