/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "linesview.h"
#include "linesviewitem.h"

#include "subtitle.h"
#include "subtitleline.h"

#include <qpainter.h>
#include <qheader.h>

#include <kdebug.h>
#include <kapplication.h>
#include <kiconloader.h>

using namespace SubtitleComposer;

TimeValidator LinesViewItem::s_timeValidator;

LinesViewItem::LinesViewItem( SubtitleLine* line, LinesView* parent, LinesViewItem* after ):
	QObject( 0 ),
	KListViewItem( parent, after ),
	m_line( line ),
	m_isPlaying( false )
{
	connect( line, SIGNAL( textChanged(const QString&) ), this, SLOT( updateText(const QString&) ) );
	connect( line, SIGNAL( showTimeChanged(const Time&) ), this, SLOT( updateShowTime(const Time&) ) );
	connect( line, SIGNAL( hideTimeChanged(const Time&) ), this, SLOT( updateHideTime(const Time&) ) );
	connect( line, SIGNAL( styleFlagsChanged(int) ), this, SLOT( updateStyles(int) ) );
	connect( line, SIGNAL( errorFlagsChanged(int) ), this, SLOT( updateErrors(int) ) );
	connect( line, SIGNAL( markChanged(bool) ), this, SLOT( updateMark(bool) ) );

	updateText( line->text() );
	updateShowTime( line->showTime() );
	updateHideTime( line->hideTime() );
	updateStyles( line->styleFlags() );
	updateErrors( line->errorFlags() );
	updateMark( line->marked() );
}

LinesViewItem::~LinesViewItem()
{
}

const QPixmap& LinesViewItem::markPixmap()
{
	static QPixmap errorPixmap;

	if ( errorPixmap.isNull() )
		errorPixmap = kapp->iconLoader()->loadIcon( "messagebox_warning", KIcon::Desktop, 13 );

	return errorPixmap;
}

const QPixmap& LinesViewItem::errorPixmap()
{
	static QPixmap errorPixmap;

	if ( errorPixmap.isNull() )
		errorPixmap = kapp->iconLoader()->loadIcon( "messagebox_critical", KIcon::Desktop, 13 );

	return errorPixmap;
}

SubtitleLine* LinesViewItem::line()
{
	return m_line;
}

bool LinesViewItem::isPlaying()
{
	return m_isPlaying;
}

void LinesViewItem::setPlaying( bool playing )
{
	if ( m_isPlaying != playing )
	{
		m_isPlaying = playing;
		repaint();
	}
}

LinesViewItem* LinesViewItem::nextSiblingCasted()
{
	return static_cast<LinesViewItem*>( nextSibling() );
}

int LinesViewItem::width( const QFontMetrics& fm, const QListView* lv, int c ) const
{
	return KListViewItem::width( fm, lv, c ) + 10;
}

void LinesViewItem::updateNumber( unsigned number )
{
	setText( LinesView::Number, QString::number( number ) );
}

void LinesViewItem::updateText( const QString& text )
{
	setText( LinesView::Text, QString( text ).replace( "\n", "|" ) );
}

void LinesViewItem::updateShowTime( const Time& showTime )
{
	setText( LinesView::ShowTime, showTime.toString() );
}

void LinesViewItem::updateHideTime( const Time& hideTime )
{
	setText( LinesView::HideTime, hideTime.toString() );
}

void LinesViewItem::updateStyles( int )
{
	repaint();
}

void LinesViewItem::updateErrors( int )
{
	repaint();
}

void LinesViewItem::updateMark( bool )
{
	repaint();
}

void LinesViewItem::paintCell( QPainter* p, const QColorGroup &cg, int column, int width, int align )
{
	int height = this->height();

	//p->fillRect( 0, 0, width, height, isSelected() ? cg.highlight() : (m_isPlaying ? cg.highlight().light() : cg.base()) );
	p->fillRect( 0, 0, width, height, m_isPlaying ? (isSelected() ? cg.highlight().light( 140 ) : cg.highlight().light( 165 )) : (isSelected() ? cg.highlight() : cg.base()) );

	QPen pen;
	pen.setWidth( 1 );
	pen.setColor( cg.mid().light( 110 ) );
	pen.setStyle( Qt::DotLine );
	p->setPen( pen );

	for ( int x = 0; x < width; x += 2 )
		p->drawPoint( x, height-1 );

	if ( column )
		for ( int y = 1; y < height; y += 2 )
			p->drawPoint( 0, y );

	pen.setStyle( Qt::SolidLine );
	pen.setColor( isSelected() ? cg.highlightedText() : cg.text() );
	p->setPen( pen );

	int xoffset = align & Qt::AlignHCenter ? 2 : 7;

	if ( column == LinesView::Text )
	{
		QFont font( p->font() );
		int styleFlags = m_line->styleFlags();
		font.setBold( styleFlags & SubtitleLine::Bold );
		font.setItalic( styleFlags & SubtitleLine::Italic );
		font.setUnderline( styleFlags & SubtitleLine::Underline );
		p->setFont( font );

		if ( m_line->marked() )
		{
			p->drawPixmap( xoffset, 1, markPixmap() );
			xoffset += 16;
		}
		else if ( m_line->errorFlags() )
		{
			p->drawPixmap( xoffset, 1, errorPixmap() );
			xoffset += 16;
		}
	}

	p->drawText( xoffset, 0 /*yoffset*/, width-2, height, align, text( column ) );

// 	QLinesViewItem::paintCell( p, auxcg, column, width, align );
}

void LinesViewItem::okRename( int column )
{
	int columnWidth = listView()->columnWidth( column );
	KListViewItem::okRename( column );

	QString text( this->text( column ) );

	if ( column == LinesView::Text )
		m_line->setText( text.replace( "|", "\n" ) );
	else
	{
		int timeMillis;
		if ( column == LinesView::ShowTime )
		{
			if ( s_timeValidator.parse( text, timeMillis ) )
				m_line->setShowTime( timeMillis );
			else
			{
				updateShowTime( m_line->showTime() );
				listView()->setColumnWidth( column, columnWidth );
			}
		}
		else if ( column == LinesView::HideTime )
		{
			if ( s_timeValidator.parse( text, timeMillis ) )
				m_line->setHideTime( timeMillis );
			else
			{
				updateHideTime( m_line->hideTime() );
				listView()->setColumnWidth( column, columnWidth );
			}
		}
	}

	// items are only editable when we say so
	setRenameEnabled( LinesView::ShowTime, false );
	setRenameEnabled( LinesView::HideTime, false );
	setRenameEnabled( LinesView::Text, false );
}

void LinesViewItem::cancelRename( int column )
{
	KListViewItem::cancelRename( column );

	// items are only editable when we say so
	setRenameEnabled( LinesView::ShowTime, false );
	setRenameEnabled( LinesView::HideTime, false );
	setRenameEnabled( LinesView::Text, false );
}

#include "linesviewitem.moc"
