/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"
#include "filedialogs.h"
#include "actionsdialogs.h"
#include "actions.h"
#include "linesviewitem.h"
#include "config/generalconfigdefs.h"
#include "config/subtitletextconfigdefs.h"
#include "config/errorsconfigdefs.h"
#include <formatsmanager.h>
#include <playersmanager.h>
#include <commondefs.h>

#include <qprocess.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qtextcodec.h>
#include <qlayout.h>

#include <kapplication.h>
#include <kaboutdata.h>
#include <kcharsets.h>
#include <ksconfig.h>
#include <kmessagebox.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <kconfig.h>
#include <kmenubar.h>
#include <kurldrag.h>

using namespace SubtitleComposer;

MainWindow::MainWindow():
	KMainWindow( 0, "MainWindow" ),
	m_config(),
	m_lastDir( QDir::homeDirPath() ),
	m_subtitle( 0 ),
	m_subtitlePath(),
	m_subtitleEncoding(),
	m_subtitleFormat()
{
	actionCollection()->setHighlightingEnabled( true );
	SubtitleActionWrappersManager::instance().setActionCollection( actionCollection() );

	// The config object has to be filled first with all the groups and default data...
	m_config.setConfigGroup( AppConfigGroup( "General", generalConfigDefaults() ) );
	m_config.setConfigGroup( AppConfigGroup( "Subtitle Text", subTextConfigDefaults() ) );
	m_config.setConfigGroup( AppConfigGroup( "Errors", errorsConfigDefaults() ) );
	QStringList playerNames( PlayersManager::playerNames() );
	for ( QStringList::Iterator it = playerNames.begin(); it != playerNames.end(); ++it )
		m_config.setConfigGroup( PlayersManager::player( *it )->configGroup() );

	// ...only then can be loaded from disk
	m_config.load( ::locate( "appdata", "subtitlecomposerrc" ) );
	for ( QStringList::Iterator it = playerNames.begin(); it != playerNames.end(); ++it )
		PlayersManager::player( *it )->setConfigGroup( *m_config.configGroup( *it ) );

	m_configDialog = new ConfigDialog( m_config, this );

	QSplitter* splitter = new QSplitter( this );
	splitter->setOrientation( Qt::Vertical );
	splitter->setMargin( 5 );

	m_playerWidget = new PlayerWidget( m_config, splitter );
	QWidget* widget = new QWidget( splitter );
	QLayout* layout = new QVBoxLayout( widget );
	layout->setMargin( 0 );
	layout->setSpacing( 5 );
	m_linesView = new LinesView( m_config, m_configDialog, widget );
	m_linesView->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	m_linesView->setAcceptDrops( true );
	m_linesView->viewport()->installEventFilter( this );
	layout->add( m_linesView );

	m_curLineWidget = new CurrentLineWidget( widget );
	m_curLineWidget->setMaximumHeight( m_curLineWidget->minimumSizeHint().height() );
	layout->add( m_curLineWidget );

	splitter->setCollapsible( widget, false );
	QValueList<int> heights;
	heights << 200 << 500;
	splitter->setSizes( heights );

	setCentralWidget( splitter ); // tell the KMainWindow that this is indeed the main widget
	setupActions(); // then, setup our actions
	statusBar()->show(); // a status bar
	toolBar()->show(); // and a tool bar

	setupGUI();

	setAcceptDrops( true );

	connect( actionCollection(), SIGNAL(actionStatusText(const QString&)), statusBar(), SLOT( message(const QString&) ) );
	connect( actionCollection(), SIGNAL(clearStatusText()), statusBar(), SLOT(clear()) );

	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_linesView, SLOT( loadSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleOpened(Subtitle*) ), m_playerWidget, SLOT( loadSubtitle(Subtitle*) ) );
	connect( this, SIGNAL( subtitleClosed() ), m_linesView, SLOT( closeSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_playerWidget, SLOT( closeSubtitle() ) );
	connect( this, SIGNAL( subtitleClosed() ), m_curLineWidget, SLOT( closeSubtitle() ) );

	connect( m_playerWidget, SIGNAL(setPlayingLine(SubtitleLine*)), m_linesView, SLOT(setPlayingLine(SubtitleLine*)) );
	connect( m_playerWidget, SIGNAL(setCurrentLine(SubtitleLine*)), m_linesView, SLOT(setCurrentLine(SubtitleLine*)) );
	connect( m_playerWidget, SIGNAL(setCurrentLineShowTime(const Time&)), m_linesView, SLOT(setCurrentLineShowTime(const Time&)) );
	connect( m_playerWidget, SIGNAL(setCurrentLineHideTime(const Time&)), m_linesView, SLOT(setCurrentLineHideTime(const Time&)) );
	connect( m_playerWidget, SIGNAL(adjustFromCurrentLine(const Time&)),
			 m_linesView, SLOT(adjustFromCurrentLine(const Time&)) );
	connect( m_playerWidget, SIGNAL(adjustUpToCurrentLine(const Time&)),
			 m_linesView, SLOT(adjustUpToCurrentLine(const Time&)) );
	connect( m_playerWidget, SIGNAL(shiftFromCurrentLine(const Time&)),
			 m_linesView, SLOT(shiftFromCurrentLine(const Time&)) );
	connect( m_playerWidget, SIGNAL(shiftUpToCurrentLine(const Time&)),
			 m_linesView, SLOT(shiftUpToCurrentLine(const Time&)) );


	connect( m_linesView, SIGNAL( currentLineChanged(SubtitleLine*) ), m_curLineWidget, SLOT( setCurrentLine(SubtitleLine*)) );
	connect( m_linesView, SIGNAL(currentLineTextHighlighted(int,int)), m_curLineWidget->textEdit(), SLOT(setSelection(int,int)) );
	connect( m_linesView, SIGNAL( lineDoubleClicked(SubtitleLine*) ), this, SLOT( onLineDoubleClicked(SubtitleLine*) ) );

	connect( m_configDialog, SIGNAL( accepted() ), this, SLOT( updateConfigFromDialog() ) );

	connect( &m_actionsManagerProxy, SIGNAL( stateChanged() ), this, SLOT( updateActionsState() ) );
	connect( &m_actionsManagerProxy, SIGNAL( stateChanged() ), this, SLOT( updateTitle() ) );

	loadConfig();

	updateActionsState();
}

MainWindow::~MainWindow()
{
	delete m_subtitle;
}

QMap<QString,QString> MainWindow::generalConfigDefaults()
{
	QMap<QString,QString> keysAndDefaults;

// 	keysAndDefaults[OPT_SPELL_CHECK_DICT] = KSpellConfig().dictionary();
	keysAndDefaults[OPT_DEFAULT_SUBS_ENCODING] = KGlobal::locale()->codecForEncoding()->name();
#ifdef HAVE_GSTREAMER
	keysAndDefaults[OPT_PLAYER_BACKEND] = "GStreamer";
#else
	keysAndDefaults[OPT_PLAYER_BACKEND] = "MPlayer";
#endif
	keysAndDefaults[OPT_JUMP_TO_POSITION] = "1500";
	keysAndDefaults[OPT_AUTO_VIDEO_LOAD] = "true";
	keysAndDefaults[OPT_SEEK_FORWARDS_BACKWARDS_SECS] = "15";
	keysAndDefaults[OPT_SHIFT_SELECTED_LINES_MSECS] = "100";
	keysAndDefaults[OPT_SET_CURRENT_LINE_TIME_COMP_MSECS] = "250";

	return keysAndDefaults;
}

QMap<QString,QString> MainWindow::subTextConfigDefaults()
{
	QMap<QString,QString> keysAndDefaults;

	keysAndDefaults[OPT_FAMILY] = "Sans Serif";
	keysAndDefaults[OPT_SIZE] = "15";
	keysAndDefaults[OPT_COLOR] = Qt::yellow.name();

	return keysAndDefaults;
}

QMap<QString,QString> MainWindow::errorsConfigDefaults()
{
	QMap<QString,QString> keysAndDefaults;

	keysAndDefaults[OPT_MIN_DURATION] = "700";
	keysAndDefaults[OPT_MAX_DURATION] = "5000";
	keysAndDefaults[OPT_MIN_DURATION_PER_CHARACTER] = "30";
	keysAndDefaults[OPT_MAX_DURATION_PER_CHARACTER] = "150";
	keysAndDefaults[OPT_MAX_CHARACTERS] = "80";
	keysAndDefaults[OPT_MAX_LINES] = "2";
	keysAndDefaults[OPT_AUTO_CLEAR_FIXED] = "true";

	return keysAndDefaults;
}


void MainWindow::setupActions()
{
	SubtitleActionWrappersManager& manager = SubtitleActionWrappersManager::instance();
	KActionCollection* actionCollection = manager.actionCollection();

	KAction* quitAction = KStdAction::quit( this, SLOT( showConfirmQuit() ), actionCollection );
	quitAction->setToolTip( i18n( "Exit the application" ) );

	KAction* prefsAction = KStdAction::preferences( this, SLOT( showPreferences() ), actionCollection );
	prefsAction->setToolTip( i18n( "Configure Subtitle Composer" ) );

	m_newSubtitleAction = KStdAction::openNew(
		this,								// Action receiver
		SLOT( newSubtitle() ),				// Action slot
		actionCollection,					// Action collection manager
		"new_subtitle" );					// Action name/identifier
	m_newSubtitleAction->setToolTip( i18n( "Create an empty subtitle" ) );

	m_openSubtitleAction = KStdAction::open(
		this,								// Action receiver
		SLOT( openSubtitle() ),				// Action slot
		actionCollection,					// Action collection manager
		"open_subtitle" );					// Action name/identifier
	m_openSubtitleAction->setToolTip( i18n( "Open subtitle file" ) );

	m_recentSubtitlesAction = new KRecentFilesActionExt( actionCollection, "recent_subtitles" );
	m_recentSubtitlesAction->setIgnoreURLQueryItems( true );
	m_recentSubtitlesAction->setMaxItems( 10 );
	m_recentSubtitlesAction->setText( i18n( "Open &Recent" ) );
	m_recentSubtitlesAction->setIcon( "fileopen" );
	m_recentSubtitlesAction->setToolTip( i18n( "Open subtitle file" ) );
	connect( m_recentSubtitlesAction, SIGNAL(urlSelected(const KURL&)), this, SLOT(openSubtitle(const KURL&)) );

	m_encodingsComboBox = new KComboBox( this );
	QStringList encodings( KGlobal::charsets()->availableEncodingNames() );
	for ( QStringList::Iterator it = encodings.begin(); it != encodings.end(); ++it )
	{
		bool found = false;
		QTextCodec* codec = KGlobal::charsets()->codecForName( *it, found );
		if ( found )
			m_encodingsComboBox->insertItem( QString( codec->name() ).upper() );
	}
	m_encodingsComboBox->setCurrentItem( m_config.configGroup( "General" )->option( OPT_DEFAULT_SUBS_ENCODING ) );
	connect( m_encodingsComboBox, SIGNAL(activated(const QString&)), this, SLOT(openSubtitle(const QString&)) );

	m_changeEncodingAction = new KWidgetAction( m_encodingsComboBox,
		i18n( "Change Encoding" ),			// Action text
		KShortcut(),						// Action accelerator
		this,								// Action receiver
		SLOT( openSubtitle() ),				// Action slot
		actionCollection,					// Action collection manager
		"change_subtitle_encoding" );		// Action name/identifier
	m_changeEncodingAction->plug( toolBar( "mainToolBar" ) );
	m_changeEncodingAction->setToolTip( i18n( "Reload opened file with a different encoding" ) );

	m_saveSubtitleAction = KStdAction::save(
		this,								// Action receiver
		SLOT( saveSubtitle() ),				// Action slot
		actionCollection,					// Action collection manager
		"save_subtitle" );					// Action name/identifier
	m_saveSubtitleAction->setToolTip( i18n( "Save currently opened file" ) );

	m_saveSubtitleAsAction = KStdAction::saveAs(
		this,								// Action receiver
		SLOT( saveSubtitleAs() ),			// Action slot
		actionCollection,					// Action collection manager
		"save_subtitle_as" );				// Action name/identifier
	m_saveSubtitleAsAction->setToolTip( i18n( "Save currently opened file with different settings" ) );

	m_closeSubtitleAction = KStdAction::close(
		this,								// Action receiver
		SLOT( closeSubtitle() ),			// Action slot
		actionCollection,					// Action collection manager
		"close_subtitle" );					// Action name/identifier
	m_closeSubtitleAction->setToolTip( i18n( "Close currently opened file" ) );

	KAction* splitSubtitleAction = new KAction( actionCollection, "split_subtitle" );
	manager.addAction( new SubtitleActionWrapper( splitSubtitleAction, SubtitleActionWrapper::HasMultipleLines ) );
	splitSubtitleAction->setText( i18n( "Split Subtitle..." ) );
	splitSubtitleAction->setToolTip( i18n( "Split the currently opened subtitle in two parts" ) );
	connect( splitSubtitleAction, SIGNAL( activated() ), this, SLOT( splitSubtitle() ) );

	KAction* joinSubtitlesAction = new KAction( actionCollection, "join_subtitles" );
	manager.addAction( new SubtitleActionWrapper( joinSubtitlesAction, SubtitleActionWrapper::Opened ) );
	joinSubtitlesAction->setText( i18n( "Join Subtitles..." ) );
	joinSubtitlesAction->setToolTip( i18n( "Appends to the currently opened subtitle another one" ) );
	connect( joinSubtitlesAction, SIGNAL( activated() ), this, SLOT( joinSubtitles() ) );


	m_undoAction = KStdAction::undo(
		&m_actionsManagerProxy,				// Action receiver
		SLOT( undo() ),						// Action slot
		actionCollection,					// Action collection manager
		"undo" );							// Action name/identifier

	m_redoAction = KStdAction::redo(
		&m_actionsManagerProxy,				// Action receiver
		SLOT( redo() ),						// Action slot
		actionCollection,					// Action collection manager
		"redo" );							// Action name/identifier
}

void MainWindow::updateActionsState()
{
	if ( m_subtitle == 0 )
	{
		m_changeEncodingAction->setEnabled( false );

		m_saveSubtitleAction->setEnabled( false );
		m_saveSubtitleAsAction->setEnabled( false );
		m_closeSubtitleAction->setEnabled( false );

		m_undoAction->setText( i18n( "Undo" ) );
		m_undoAction->setEnabled( false );

		m_redoAction->setText( i18n( "Redo" ) );
		m_redoAction->setEnabled( false );
	}
	else
	{
		m_changeEncodingAction->setEnabled( ! m_subtitlePath.isEmpty() && ! m_subtitle->isDirty() );

		m_saveSubtitleAction->setEnabled( m_subtitle->isDirty() );
		m_saveSubtitleAsAction->setEnabled( true );
		m_closeSubtitleAction->setEnabled( true );

		bool undoEnabled = m_actionsManagerProxy.hasUndo();
		bool redoEnabled = m_actionsManagerProxy.hasRedo();

		m_undoAction->setText( i18n( "Undo" ) + (undoEnabled ? " " + m_actionsManagerProxy.undoDescription() : QString::null) );
		m_undoAction->setEnabled( undoEnabled );

		m_redoAction->setText( i18n( "Redo" ) + (redoEnabled ? " " + m_actionsManagerProxy.redoDescription() : QString::null) );
		m_redoAction->setEnabled( redoEnabled );
	}
}

void MainWindow::updateTitle()
{
	if ( m_subtitle )
		setCaption( m_subtitlePath.isEmpty() ? i18n( "Untitled" ) : m_subtitlePath, m_subtitle->isDirty() );
	else
		setCaption( QString::null );
}

void MainWindow::newSubtitle()
{
	if ( ! closeSubtitle() )
		return;

	m_subtitle = new Subtitle();

	m_actionsManagerProxy.registerActionsManager( m_subtitle->actionsManager() );

	emit subtitleOpened( m_subtitle );

	updateActionsState();
	updateTitle();
}

void MainWindow::openSubtitle()
{
	OpenSubtitleDialog openDlg( m_lastDir, m_config.configGroup( "General" )->option( OPT_DEFAULT_SUBS_ENCODING ) );

	if ( openDlg.exec() == QDialog::Accepted )
	{
		if ( ! closeSubtitle() )
			return;

		m_lastDir = openDlg.selectedFile();

		KURL fileURL;
		fileURL.setProtocol( "file" );
		fileURL.setPath( openDlg.selectedFile() );
		fileURL.addQueryItem( "encoding", openDlg.selectedEncoding() );
		openSubtitle( fileURL );
	}
}

void MainWindow::openSubtitle( const KURL& url )
{
	if ( ! closeSubtitle() )
		return;

	if ( url.protocol() != "file" )
		return;

	QString path = url.path();

	bool codecFound = true;
	QString encoding = url.queryItem( "encoding" );
	if ( encoding.isEmpty() )
		encoding = m_config.configGroup( "General" )->option( OPT_DEFAULT_SUBS_ENCODING );
	QTextCodec* codec = KGlobal::charsets()->codecForName( encoding, codecFound );
	if ( ! codecFound )
		codec = KGlobal::locale()->codecForEncoding();

	m_subtitle = new Subtitle();

	if ( FormatsManager::readSubtitle( *m_subtitle, path, codec, &m_subtitleFormat ) )
	{
		m_actionsManagerProxy.registerActionsManager( m_subtitle->actionsManager() );

		emit subtitleOpened( m_subtitle );

		if ( m_config.configGroup( "General" )->optionAsBool( OPT_AUTO_VIDEO_LOAD ) )
			m_playerWidget->openVideoForSubtitle( path );

		addToRecentFiles( path, codec->name() );

		m_subtitlePath = path;
		m_subtitleEncoding = codec->name();

		m_encodingsComboBox->setCurrentItem( m_subtitleEncoding.upper() );

		updateActionsState();
		updateTitle();
	}
	else
	{
		delete m_subtitle;
		m_subtitle = 0;

		KMessageBox::sorry( this, i18n( "Couldn't parse the subtitle file." ) );
	}
}

void MainWindow::openSubtitle( const QString& encoding )
{
	if ( encoding != m_subtitleEncoding )
	{
		KURL fileURL;
		fileURL.setProtocol( "file" );
		fileURL.setPath( m_subtitlePath );
		fileURL.addQueryItem( "encoding", encoding );
		openSubtitle( fileURL );
	}
}


bool MainWindow::saveSubtitle()
{
	if ( m_subtitlePath.isEmpty() || m_subtitleEncoding.isEmpty() || ! FormatsManager::hasOutputFormat( m_subtitleFormat ) )
		return saveSubtitleAs();

	bool codecFound = true;
	QTextCodec* codec = KGlobal::charsets()->codecForName( m_subtitleEncoding, codecFound );
	if ( ! codecFound )
		codec = KGlobal::locale()->codecForEncoding();

	if ( FormatsManager::writeSubtitle( *m_subtitle, m_subtitlePath, codec, m_subtitleFormat, true ) )
	{
		addToRecentFiles( m_subtitlePath, codec->name() );

		m_saveSubtitleAction->setEnabled( false );
		updateTitle();
		return true;
	}
	else
	{
		KMessageBox::sorry( this, i18n( "There was an error saving the subtitle.") );
		return false;
	}
}

bool MainWindow::saveSubtitleAs()
{
	SaveSubtitleDialog saveDlg( m_subtitlePath, m_subtitleEncoding.isEmpty() ? m_config.configGroup( "General" )->option( OPT_DEFAULT_SUBS_ENCODING ) : m_subtitleEncoding, m_subtitleFormat );

	if ( saveDlg.exec() == QDialog::Accepted )
	{
		if ( QFile::exists( saveDlg.selectedFile() ) )
		{
			if ( KMessageBox::warningContinueCancel( this, i18n( "A file named \"%1\" already exists. Are you sure you want to overwrite it?" ).arg( QFileInfo( saveDlg.selectedFile() ).fileName() ), i18n( "Overwrite File?" ), i18n( "Overwrite" ) ) != KMessageBox::Continue )
				return false;
		}

		m_subtitlePath = saveDlg.selectedFile();
		m_subtitleEncoding = saveDlg.selectedEncoding();
		m_subtitleFormat = saveDlg.selectedFormat();

		return saveSubtitle();
	}

	return false;
}

bool MainWindow::closeSubtitle()
{
	if ( m_subtitle )
	{
		if ( m_subtitle->isDirty() )
		{
			int result = KMessageBox::warningYesNoCancel(
				0,
				i18n( "Currently opened subtitle has unsaved changes.\nDo you want to save them?" ),
				i18n( "Close File" ) + " - SubtitleComposer" );
			if ( result == KMessageBox::Cancel )
				return false;
			else if ( result == KMessageBox::Yes )
				if ( ! saveSubtitle() )
					return false;
		}

		emit subtitleClosed();

		m_actionsManagerProxy.unregisterActionsManager( m_subtitle->actionsManager() );

		delete m_subtitle;
		m_subtitle = 0;

		m_subtitlePath = QString::null;
		m_subtitleEncoding = QString::null;
		m_subtitleFormat = QString::null;

		updateActionsState();
		updateTitle();
	}

	return true;
}


void MainWindow::joinSubtitles()
{
	static QString defaultEncoding( m_config.configGroup( "General" )->option( OPT_DEFAULT_SUBS_ENCODING ) );
	static JoinSubtitlesDialog* dlg = new JoinSubtitlesDialog( defaultEncoding, this );

	if ( dlg->exec() == QDialog::Accepted )
	{
		bool codecFound;
		QTextCodec* codec = KGlobal::charsets()->codecForName( dlg->subtitleEncoding(), codecFound );
		if ( ! codecFound )
			codec = KGlobal::locale()->codecForEncoding();

		Subtitle secondSubtitle;
		if ( FormatsManager::readSubtitle( secondSubtitle, dlg->subtitlePath(), codec ) )
			m_subtitle->joinSubtitles( secondSubtitle, dlg->shiftTime().timeMillis() );
		else
			KMessageBox::sorry( this, i18n( "Couldn't read the subtitle file to append." ) );
	}
}


void MainWindow::splitSubtitle()
{
	static SplitSubtitleDialog* dlg = new SplitSubtitleDialog( this );

	if ( dlg->exec() == QDialog::Accepted )
	{
		Subtitle newSubtitle;
		m_subtitle->splitSubtitle( newSubtitle, dlg->splitTime().timeMillis(), dlg->shiftNewSubtitle() );
		if ( newSubtitle.linesCount() )
		{
			QString encoding = m_subtitleEncoding.isEmpty() ? "UTF-8" : m_subtitleEncoding;
			QTextCodec* codec = QTextCodec::codecForName( encoding );
			QString format = m_subtitleFormat.isEmpty() ? "SubRip" : m_subtitleFormat;
			QFileInfo fileInfo(
				m_subtitlePath.isEmpty() ?
					QFileInfo( QDir( System::tempDir() ), "untitled.srt" ) :
					m_subtitlePath
			);

			QString newFilePath( System::newFile(
				fileInfo.dirPath(),
				fileInfo.baseName( true ) + " - splitted",
				fileInfo.extension( false )
			) );

			if ( FormatsManager::writeSubtitle( newSubtitle, newFilePath, codec, format, true ) )
			{
				KURL newFileURL( newFilePath );
				newFileURL.addQueryItem( "encoding", encoding );

				QProcess newProcess( QString( kapp->aboutData()->appName() ) );
				newProcess.addArgument( newFileURL.prettyURL() );
				if ( ! newProcess.start() )
					KMessageBox::sorry( this, i18n( "Couldn't open a new Subtitle Composer window.\nThe new subtitle part was saved as %1." ).arg( newFilePath ) );
			}
			else
			{
				m_subtitle->actionsManager().undo();
				KMessageBox::sorry( this, i18n( "Couldn't write the splitted subtitle file." ) );
			}
		}
		else
			KMessageBox::information( this, i18n( "The specified time does not split the subtitles." ) );
	}
}


void MainWindow::addToRecentFiles( KURL url, const QString& encoding )
{
	url.addQueryItem( "encoding", encoding );
	m_recentSubtitlesAction->add( url );
}

void MainWindow::addToRecentFiles( const QString& path, const QString encoding )
{
	KURL url;
	url.setProtocol( "file" );
	url.setPath( path );

	addToRecentFiles( url, encoding );
}

void MainWindow::onLineDoubleClicked( SubtitleLine* line )
{
	int mseconds = line->showTime().timeMillis() - m_config.configGroup( "General" )->optionAsInt( OPT_JUMP_TO_POSITION );
	PlayersManager::activePlayer()->seek( mseconds > 0 ? mseconds / 1000.0 : 0.0 );
}

void MainWindow::showPreferences()
{
	m_configDialog->show();
}

void MainWindow::showConfirmQuit()
{
	if ( ! closeSubtitle() )
		return;

	saveConfig();
	kapp->quit();
}

void MainWindow::updateConfigFromDialog()
{
	m_config = m_configDialog->getConfig();

	QStringList playerNames( PlayersManager::playerNames() );
	for ( QStringList::Iterator it = playerNames.begin(); it != playerNames.end(); ++it )
		PlayersManager::player( *it )->setConfigGroup( *m_config.configGroup( *it ) );

	m_config.save( ::locateLocal( "appdata", "subtitlecomposerrc" ) );
}

bool MainWindow::queryClose()
{
	if ( ! closeSubtitle() )
		return false;

	saveConfig();
	return true;
}

bool MainWindow::eventFilter( QObject* object, QEvent* event )
{
	if ( object != m_linesView->viewport() )
		return false;

	if ( event->type() == QEvent::DragEnter )
	{
		QDragEnterEvent* dragEnterEvent = static_cast<QDragEnterEvent*>( event );
		dragEnterEvent->accept( KURLDrag::canDecode( dragEnterEvent ) );
		return true;
	}
	else if ( event->type() == QEvent::DragMove )
	{
		return true;
	}
	else if ( event->type() == QEvent::Drop )
	{
		QDropEvent* dropEvent = static_cast<QDropEvent*>( event );

		KURL::List urls;
		if ( KURLDrag::decode( dropEvent, urls ) && ! urls.isEmpty() )
		{
			for ( KURL::List::Iterator it = urls.begin(); it != urls.end(); ++it )
			{
				KURL& url = *it;

				if ( url.protocol() != "file" )
					continue;

				openSubtitle( url );
				break;
			}
		}

		return true; // eat event
	}
	else
		return false; // standard event processing
}

void MainWindow::loadConfig()
{
	KConfigGroupSaver saver( kapp->config(), "MainWindow Settings" );

	m_lastDir = kapp->config()->readPathEntry( "LastDir", QDir::homeDirPath() );
	m_recentSubtitlesAction->loadEntries( kapp->config(), "Recent Subtitles" );

	m_playerWidget->loadConfig();
	m_linesView->loadConfig();
}

void MainWindow::saveConfig()
{
	KConfigGroupSaver saver( kapp->config(), "MainWindow Settings" );

	kapp->config()->writePathEntry( "LastDir", m_lastDir );
	m_recentSubtitlesAction->saveEntries( kapp->config(), "Recent Subtitles" );

	m_playerWidget->saveConfig();
	m_linesView->saveConfig();
}

#include "mainwindow.moc"
