/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "playerwidget.h"
#include "actions.h"
#include "layeredwidget.h"
#include "config/generalconfigdefs.h"
#include "config/subtitletextconfigdefs.h"
#include <playersmanager.h>
#include <commondefs.h>

#include <qlayout.h>
#include <qpoint.h>

#include <klocale.h>
#include <kapplication.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <kmessagebox.h>
#include <kurldrag.h>

using namespace SubtitleComposer;

// FIXME wtf is this ??
#define MAGIC_NUMBER -1

PlayerWidget::PlayerWidget( const AppConfig& config, QWidget* parent ):
	QWidget( parent, "PlayerWidget" ),
	m_generalConfig( *config.configGroup( "General" ) ),
	m_fontConfigGroup( *config.configGroup( "Subtitle Text" ) ),
	m_subtitle( 0 ),
	m_overlayLine( 0 ),
	m_playingLine( 0 ),
	m_lastSearchedLineToShowTime( 23, 59, 59, 999 ),
	m_activePlayer( 0 ),
	m_subtitlePath(),
	m_updatePositionControls( 1 ),
	m_updateVideoPosition( false ),
	m_updatePositionLabel( true ),
	m_showPositionEdit( false )
{
	m_layeredWidget = new LayeredWidget( this );
	m_layeredWidget->setAcceptDrops( true );
	m_layeredWidget->installEventFilter( this );
	m_layeredWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	PlayersManager::init( m_layeredWidget, m_generalConfig.option( OPT_PLAYER_BACKEND ) );
	m_activePlayer = PlayersManager::activePlayer();
	m_textOverlay = new TextOverlayWidget( m_layeredWidget );

	KIconLoader iconLoader;

	m_stopButton = new QToolButton( this, "PlayerStopButton" );
	m_playPauseButton = new QToolButton( this, "PlayerPlayPauseButton" );
	m_playIconSet = iconLoader.loadIconSet( "player_play", KIcon::Small );
	m_pauseIconSet = iconLoader.loadIconSet( "player_pause", KIcon::Small );

	m_seekBackwardsButton = new QToolButton( this, "BackwardsButton" );
	m_seekForwardsButton = new QToolButton( this, "ForwardsButton" );

	m_previousLineButton = new QToolButton( this, "PreviousLineButton" );
	m_nextLineButton = new QToolButton( this, "NextLineButton" );

	m_setLineShowTimeButton = new QToolButton( this, "SetLineShowTimeButton" );
	m_setLineHideTimeButton = new QToolButton( this, "SetLineHideTimeButton" );

	m_linkCurrentLineToVideoButton = new QToolButton( this, "LinkCurrentLineToVideoButton" );

	m_toggleMuteButton = new QToolButton( this, "PlayerMuteButton" );

	m_seekSlider = new QuietSlider( 0, 1000, 10, 0, Qt::Horizontal, this, "PlayerSeekSlider" );
	m_seekSlider->setTickmarks( QSlider::NoMarks );

	m_infoControlsGroupBox = new QGroupBox( this, "InfoGroupBox" );
	m_infoControlsGroupBox->setColumnLayout( 0, Qt::Vertical );
	m_infoControlsGroupBox->layout()->setSpacing( 5 );
	m_infoControlsGroupBox->layout()->setMargin( 6 );

	m_positionTagLabel = new QLabel( m_infoControlsGroupBox );
	m_positionTagLabel->setText( i18n( "<b>Position</b>" ) );
	m_positionLabel = new QLabel( m_infoControlsGroupBox, "PositionLabel" );
	m_positionEdit = new TimeEdit( m_infoControlsGroupBox, "PositionEdit" );
	m_lengthTagLabel = new QLabel( m_infoControlsGroupBox );
	m_lengthTagLabel->setText( i18n( "<b>Length</b>") );
	m_lengthLabel = new QLabel( m_infoControlsGroupBox, "LengthLabel" );
	m_fpsTagLabel = new QLabel( m_infoControlsGroupBox );
	m_fpsTagLabel->setText( i18n( "<b>FPS</b>") );
	m_fpsLabel = new QLabel( m_infoControlsGroupBox, "FramesPerSecondLabel" );

	m_lengthLabel->setMinimumWidth( m_positionEdit->sizeHint().width() );

	m_volumeSlider = new PointingSlider( 0, 100, 5, 0, Qt::Vertical, this, "PlayerVolumeSlider" );
	m_volumeSlider->setTickmarks( QSlider::NoMarks );
	m_volumeSlider->setFocusPolicy( QWidget::NoFocus );

	QGridLayout* videoControlsLayout = new QGridLayout();
	videoControlsLayout->setSpacing( 2 );
	videoControlsLayout->addWidget( m_playPauseButton, 0, 0 );
	videoControlsLayout->addWidget( m_stopButton, 0, 1 );
	videoControlsLayout->addWidget( m_seekBackwardsButton, 0, 2 );
	videoControlsLayout->addWidget( m_seekForwardsButton, 0, 3 );
	videoControlsLayout->addItem( new QSpacerItem( 2, 2 ), 0, 4 );
	videoControlsLayout->addWidget( m_previousLineButton, 0, 5 );
	videoControlsLayout->addWidget( m_nextLineButton, 0, 6 );
	videoControlsLayout->addItem( new QSpacerItem( 2, 2 ), 0, 7 );
	videoControlsLayout->addWidget( m_setLineShowTimeButton, 0, 8 );
	videoControlsLayout->addWidget( m_setLineHideTimeButton, 0, 9 );
	videoControlsLayout->addItem( new QSpacerItem( 2, 2 ), 0, 10 );
	videoControlsLayout->addWidget( m_linkCurrentLineToVideoButton, 0, 11 );
	videoControlsLayout->addWidget( m_seekSlider, 0, 12 );

	QGridLayout* audioControlsLayout = new QGridLayout();
	audioControlsLayout->addWidget( m_toggleMuteButton, 0, 0 );
	audioControlsLayout->addWidget( m_volumeSlider, 1, 0 );

	QGridLayout* infoControlsLayout = new QGridLayout( m_infoControlsGroupBox->layout() );
	infoControlsLayout->addWidget( m_fpsTagLabel, 0, 0 );
	infoControlsLayout->addWidget( m_fpsLabel, 1, 0 );
	infoControlsLayout->addItem( new QSpacerItem( 1, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding ), 2, 0 );
	infoControlsLayout->addWidget( m_lengthTagLabel, 3, 0 );
	infoControlsLayout->addWidget( m_lengthLabel, 4, 0 );
	infoControlsLayout->addItem( new QSpacerItem( 1, 1, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding ), 5, 0 );
	infoControlsLayout->addWidget( m_positionTagLabel, 6, 0 );
	infoControlsLayout->addWidget( m_positionLabel, 7, 0 );
	infoControlsLayout->addWidget( m_positionEdit, 8, 0 );

	QGridLayout* mainLayout = new QGridLayout( this );
	mainLayout->setSpacing( 3 );
	mainLayout->addMultiCellWidget( m_infoControlsGroupBox, 0, 1, 0, 0 );
	mainLayout->addWidget( m_layeredWidget, 0, 1 );
	mainLayout->addLayout( audioControlsLayout, 0, 2 );
	mainLayout->addLayout( videoControlsLayout, 1, 1 );

	connect( m_seekSlider, SIGNAL( valueChanged(int) ), this, SLOT(onSeekSliderValueChanged(int)) );
	connect( m_seekSlider, SIGNAL( sliderMoved(int) ), this, SLOT(onSeekSliderDragged(int)) );
	connect( m_seekSlider, SIGNAL( sliderPressed() ), this, SLOT(onSeekSliderPressed()) );
	connect( m_seekSlider, SIGNAL( sliderReleased() ), this, SLOT(onSeekSliderReleased()) );

	connect( m_positionEdit, SIGNAL( valueChanged(int) ), this, SLOT(onPositionEditValueChanged(int)) );
	connect( m_positionEdit, SIGNAL( valueEntered(int) ), this, SLOT(onPositionEditValueChanged(int)) );

	connect( m_volumeSlider, SIGNAL( valueChanged(int) ), this, SLOT(onVolumeSliderValueChanged(int)) );

	connect( &m_generalConfig, SIGNAL(optionChanged(const QString&,const QString&)),
			 this, SLOT(onGeneralOptionChanged(const QString&,const QString&)) );
	connect( &m_fontConfigGroup, SIGNAL(optionChanged(const QString&,const QString&)),
			 this, SLOT(onSubsFontOptionChanged()) );

	setupActions();

	connectActivePlayer();

	connect( m_activePlayer, SIGNAL(finalized(Player*)), this, SLOT(disconnectActivePlayer()) );
	connect( m_activePlayer, SIGNAL(initialized(Player*)), this, SLOT(connectActivePlayer()) );

	setControlsStateClosed();
	onSubsFontOptionChanged();
}

PlayerWidget::~PlayerWidget()
{
	PlayersManager::deinit();
}

void PlayerWidget::setupActions()
{
	SubtitleActionWrappersManager& manager = SubtitleActionWrappersManager::instance();
	KActionCollection* actionCollection = manager.actionCollection();


	KAction* openVideoAction = new KAction( actionCollection, "open_video" );
	openVideoAction->setIcon( "fileopen" );
	openVideoAction->setText( i18n( "Open Video..." ) );
	openVideoAction->setToolTip( i18n( "Open video file" ) );
	connect( openVideoAction, SIGNAL( activated() ), this, SLOT( openVideo() ) );


	m_recentVideosAction = new KRecentFilesActionExt( actionCollection, "recent_videos" );
	m_recentVideosAction->setMaxItems( 5 );
	m_recentVideosAction->setText( i18n( "Open &Recent" ) );
	m_recentVideosAction->setIcon( "fileopen" );
	m_recentVideosAction->setToolTip( i18n( "Open video file" ) );
	connect( m_recentVideosAction, SIGNAL(urlSelected(const KURL&)), this, SLOT(openVideo(const KURL&)) );


	m_closeVideoAction = new KAction( actionCollection, "close_video" );
	m_closeVideoAction->setIcon( "fileclose" );
	m_closeVideoAction->setText( i18n( "Close Video" ) );
	m_closeVideoAction->setToolTip( i18n( "Close video file" ) );
	m_closeVideoAction->setEnabled( false );
	connect( m_closeVideoAction, SIGNAL( activated() ), this, SLOT( closeVideo() ) );


	m_stopAction = new KActionExt( actionCollection, "stop_video" );
	m_stopAction->setIcon( "player_stop" );
	m_stopAction->setText( i18n( "Stop" ) );
	m_stopAction->setToolTip( i18n( "Stop video playback" ) );
	m_stopAction->plug( m_stopButton );
	m_stopAction->setEnabled( false );


	m_playPauseAction = new KActionExt( actionCollection, "play_pause_video" );
	m_playPauseAction->setIcon( "player_play" );
	m_playPauseAction->setText( i18n( "Play" ) );
	m_playPauseAction->setToolTip( i18n( "Toggle video playing/paused" ) );
	m_playPauseAction->plug( m_playPauseButton );
	m_playPauseAction->setEnabled( false );


	m_seekBackwardsAction = new KActionExt( actionCollection, "seek_backwards_video" );
	m_seekBackwardsAction->setIcon( "player_rew" );
	m_seekBackwardsAction->setText( i18n( "Seek Backwards" ) );
	m_seekBackwardsAction->setToolTip( i18n(
		"Seek backwards 1 second",
		"Seek backwards %n seconds",
		m_generalConfig.optionAsInt( OPT_SEEK_FORWARDS_BACKWARDS_SECS )
	) );
	m_seekBackwardsAction->plug( m_seekBackwardsButton );
	m_seekBackwardsAction->setEnabled( false );
	connect( m_seekBackwardsAction, SIGNAL( activated() ), this, SLOT( seekBackwards() ) );


	m_seekForwardsAction = new KActionExt( actionCollection, "seek_forwards_video" );
	m_seekForwardsAction->setIcon( "player_fwd" );
	m_seekForwardsAction->setText( i18n( "Seek Forwards" ) );
	m_seekForwardsAction->setToolTip( i18n(
		"Seek forwards 1 second",
		"Seek forwards %n seconds",
		m_generalConfig.optionAsInt( OPT_SEEK_FORWARDS_BACKWARDS_SECS )
	) );
	m_seekForwardsAction->plug( m_seekForwardsButton );
	m_seekForwardsAction->setEnabled( false );
	connect( m_seekForwardsAction, SIGNAL( activated() ), this, SLOT( seekForwards() ) );


	m_linkCurrentLineToVideoAction = new KToggleActionExt( actionCollection, "link_current_line_to_video" );
	m_linkCurrentLineToVideoAction->setIconSet( GUI::customIconSet( "linkcurrenttovideo", KIcon::Toolbar, 16 ) );
	m_linkCurrentLineToVideoAction->setText( i18n( "Link Current Line to Video Position" ) );
	m_linkCurrentLineToVideoAction->setToolTip( i18n( "Make current line follow the playing video position" ) );
	m_linkCurrentLineToVideoAction->plug( m_linkCurrentLineToVideoButton );
	connect( m_linkCurrentLineToVideoAction, SIGNAL( toggled(bool) ), this, SLOT( onLinkCurrentToPositionToggled(bool) ) );


	m_toggleMuteAction = new KToggleActionExt( actionCollection, "toggle_mute" );
	m_toggleMuteAction->setIcon( "player_mute" );
	m_toggleMuteAction->setText( i18n( "Mute" ), i18n( "Unmute" ) );
	m_toggleMuteAction->setToolTip( i18n( "Enable/disable playback sound" ) );
	m_toggleMuteAction->plug( m_toggleMuteButton );
	connect( m_toggleMuteAction, SIGNAL( toggled(bool) ), this, SLOT( onMuteButtonToggled(bool) ) );


	m_selectAudioTrackAction = new KListAction( actionCollection, "set_audio_track" );
	m_selectAudioTrackAction->setIcon( "kcmsound" );
	m_selectAudioTrackAction->setText( i18n( "Audio Tracks" ) );
	m_selectAudioTrackAction->setToolTip( i18n( "Select active audio track" ) );
	m_selectAudioTrackAction->setEnabled( false );


	KActionExt* previousLineAction = new KActionExt( actionCollection, "jump_to_previous_line" );
	previousLineAction->setIcon( "back" );
	previousLineAction->setText( i18n( "Jump to Previous Line" ) );
	previousLineAction->setToolTip( i18n( "Seek to previous subtitle line show time" ) );
	previousLineAction->plug( m_previousLineButton );
	previousLineAction->setEnabled( false );
	connect( previousLineAction, SIGNAL( activated() ), this, SLOT( jumpToPreviousLine() ) );
	m_previousLineAction = new SubtitleActionWrapper( previousLineAction, SubtitleActionWrapper::HasLine );
	manager.addAction( m_previousLineAction );


	KActionExt* nextLineAction = new KActionExt( actionCollection, "jump_to_next_line" );
	nextLineAction->setIcon( "forward" );
	nextLineAction->setText( i18n( "Jump to Next Line" ) );
	nextLineAction->setToolTip( i18n( "Seek to next subtitle line show time" ) );
	nextLineAction->plug( m_nextLineButton );
	nextLineAction->setEnabled( false );
	connect( nextLineAction, SIGNAL( activated() ), this, SLOT( jumpToNextLine() ) );
	m_nextLineAction = new SubtitleActionWrapper( nextLineAction, SubtitleActionWrapper::HasLine );
	manager.addAction( m_nextLineAction );


	KActionExt* setLineShowTimeAction = new KActionExt( actionCollection, "set_current_line_show_time" );
	setLineShowTimeAction->setIconSet( GUI::customIconSet( "setshowtime", KIcon::Toolbar, 16 ) );
	setLineShowTimeAction->setText( i18n( "Set Current Line Show Time" ) );
	setLineShowTimeAction->setToolTip( i18n( "Set current line show time to video position" ) );
	setLineShowTimeAction->setShortcut( KShortcut( "Shift+A" ) );
	setLineShowTimeAction->plug( m_setLineShowTimeButton );
	setLineShowTimeAction->setEnabled( false );
	connect( setLineShowTimeAction, SIGNAL( activated() ), this, SLOT( setCurrentLineShowTime() ) );
	m_setLineShowTimeAction = new SubtitleActionWrapper( setLineShowTimeAction, SubtitleActionWrapper::HasSelectedLine );
	manager.addAction( m_setLineShowTimeAction );


	KActionExt* setLineHideTimeAction = new KActionExt( actionCollection, "set_current_line_hide_time" );
	setLineHideTimeAction->setIconSet( GUI::customIconSet( "sethidetime", KIcon::Toolbar, 16 ) );
	setLineHideTimeAction->setText( i18n( "Set Current Line Hide Time" ) );
	setLineHideTimeAction->setToolTip( i18n( "Set current line hide time to video position" ) );
	setLineHideTimeAction->setShortcut( KShortcut( "Shift+S" ) );
	setLineHideTimeAction->plug( m_setLineHideTimeButton );
	setLineHideTimeAction->setEnabled( false );
	connect( setLineHideTimeAction, SIGNAL( activated() ), this, SLOT( setCurrentLineHideTime() ) );
	m_setLineHideTimeAction = new SubtitleActionWrapper( setLineHideTimeAction, SubtitleActionWrapper::HasSelectedLine );
	manager.addAction( m_setLineHideTimeAction );


	KActionExt* adjustFromCurrentLineAction = new KActionExt( actionCollection, "adjust_from_current_line" );
	adjustFromCurrentLineAction->setText( i18n( "Adjust From Current Line" ) );
	adjustFromCurrentLineAction->setToolTip( i18n( "Set current line show time to video position and linearly adjust the following lines" ) );
	adjustFromCurrentLineAction->setShortcut( KShortcut( "Alt+Z" ) );
	adjustFromCurrentLineAction->setEnabled( false );
	connect( adjustFromCurrentLineAction, SIGNAL( activated() ), this, SLOT( adjustFromCurrentLine() ) );
	m_adjustFromCurrentLineAction = new SubtitleActionWrapper(adjustFromCurrentLineAction, SubtitleActionWrapper::HasSelectedLine);
	manager.addAction( m_adjustFromCurrentLineAction );


	KActionExt* adjustUpToCurrentLineAction = new KActionExt( actionCollection, "adjust_up_to_current_line" );
	adjustUpToCurrentLineAction->setText( i18n( "Adjust Up To Current Line" ) );
	adjustUpToCurrentLineAction->setToolTip( i18n( "Set current line show time to video position and linearly adjust the preceding lines" ) );
	adjustUpToCurrentLineAction->setShortcut( KShortcut( "Alt+X" ) );
	adjustUpToCurrentLineAction->setEnabled( false );
	connect( adjustUpToCurrentLineAction, SIGNAL( activated() ), this, SLOT( adjustUpToCurrentLine() ) );
	m_adjustUpToCurrentLineAction = new SubtitleActionWrapper(adjustUpToCurrentLineAction, SubtitleActionWrapper::HasSelectedLine);
	manager.addAction( m_adjustUpToCurrentLineAction );


	KActionExt* shiftFromCurrentLineAction = new KActionExt( actionCollection, "shift_from_current_line" );
	shiftFromCurrentLineAction->setText( i18n( "Shift From Current Line" ) );
	shiftFromCurrentLineAction->setToolTip( i18n( "Shift current line show time to video position and equally shift the following lines" ) );
	shiftFromCurrentLineAction->setShortcut( KShortcut( "Shift+Z" ) );
	shiftFromCurrentLineAction->setEnabled( false );
	connect( shiftFromCurrentLineAction, SIGNAL( activated() ), this, SLOT( shiftFromCurrentLine() ) );
	m_shiftFromCurrentLineAction = new SubtitleActionWrapper( shiftFromCurrentLineAction, SubtitleActionWrapper::HasSelectedLine );
	manager.addAction( m_shiftFromCurrentLineAction );


	KActionExt* shiftUpToCurrentLineAction = new KActionExt( actionCollection, "shift_up_to_current_line" );
	shiftUpToCurrentLineAction->setText( i18n( "Shift Up To Current Line" ) );
	shiftUpToCurrentLineAction->setToolTip( i18n( "Shift current line show time to video position and equally shift the preceding lines" ) );
	shiftUpToCurrentLineAction->setShortcut( KShortcut( "Shift+X" ) );
	shiftUpToCurrentLineAction->setEnabled( false );
	connect( shiftUpToCurrentLineAction, SIGNAL( activated() ), this, SLOT( shiftUpToCurrentLine() ) );
	m_shiftUpToCurrentLineAction = new SubtitleActionWrapper( shiftUpToCurrentLineAction, SubtitleActionWrapper::HasSelectedLine );
	manager.addAction( m_shiftUpToCurrentLineAction );
}

void PlayerWidget::connectActivePlayer()
{
	connect( m_activePlayer, SIGNAL(opened(const QString&)), this, SLOT(setControlsStateOpened()) );
	connect( m_activePlayer, SIGNAL(closed()), this, SLOT(setControlsStateClosed()) );
	connect( m_activePlayer, SIGNAL(playing()), this, SLOT(setControlsStatePlaying()) );
	connect( m_activePlayer, SIGNAL(positionChanged(double)), this, SLOT(updatePositionControls(double)) );
	connect( m_activePlayer, SIGNAL(lengthChanged(double)), this, SLOT(updateLengthControls(double)) );
	connect( m_activePlayer, SIGNAL(framesPerSecondChanged(double)), this, SLOT(updateFramesPerSecondControls(double)) );
	connect( m_activePlayer, SIGNAL(paused()), this, SLOT(setControlsStatePaused()) );
	connect( m_activePlayer, SIGNAL(stopped()), this, SLOT(setControlsStateStopped()) );

	connect( m_stopAction, SIGNAL(activated()), m_activePlayer, SLOT(stop()) );
	connect( m_playPauseAction, SIGNAL(activated()), m_activePlayer, SLOT(togglePlayPaused()) );

	connect( m_selectAudioTrackAction, SIGNAL(activated(int)), m_activePlayer, SLOT(setAudioStream(int)) );

	connect( m_activePlayer->videoWidget(), SIGNAL(leftClicked(const QPoint&)), m_activePlayer, SLOT(togglePlayPaused()) );
	connect( m_activePlayer->videoWidget(), SIGNAL(rightClicked(const QPoint&)), this, SLOT(showAudioTracksMenu(const QPoint&)) );
}

void PlayerWidget::disconnectActivePlayer()
{
	disconnect( m_activePlayer, SIGNAL(opened(const QString&)), this, SLOT(setControlsStateOpened()) );
	disconnect( m_activePlayer, SIGNAL(closed()), this, SLOT(setControlsStateClosed()) );
	disconnect( m_activePlayer, SIGNAL(playing()), this, SLOT(setControlsStatePlaying()) );
	disconnect( m_activePlayer, SIGNAL(positionChanged(double)), this, SLOT(updatePositionControls(double)) );
	disconnect( m_activePlayer, SIGNAL(lengthChanged(double)), this, SLOT(updateLengthControls(double)) );
	disconnect( m_activePlayer, SIGNAL(framesPerSecondChanged(double)), this, SLOT(updateFramesPerSecondControls(double)) );
	disconnect( m_activePlayer, SIGNAL(paused()), this, SLOT(setControlsStatePaused()) );
	disconnect( m_activePlayer, SIGNAL(stopped()), this, SLOT(setControlsStateStopped()) );

	disconnect( m_stopAction, SIGNAL(activated()), m_activePlayer, SLOT(stop()) );
	disconnect( m_playPauseAction, SIGNAL(activated()), m_activePlayer, SLOT(togglePlayPaused()) );

	disconnect( m_selectAudioTrackAction, SIGNAL(activated(int)), m_activePlayer, SLOT(setAudioStream(int)) );
}

void PlayerWidget::loadSubtitle( Subtitle* subtitle )
{
	m_subtitle = subtitle;

	connect( m_subtitle, SIGNAL(linesInserted(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );
	connect( m_subtitle, SIGNAL(linesRemoved(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );

	updateControlsState();
}

void PlayerWidget::closeSubtitle()
{
	disconnect( m_subtitle, SIGNAL(linesInserted(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );
	disconnect( m_subtitle, SIGNAL(linesRemoved(unsigned,unsigned)), this, SLOT(invalidateOverlayLine()) );

	m_subtitle = 0;

	invalidateOverlayLine();

	updatePlayingLine( 0 );

	updateControlsState();
}

bool PlayerWidget::openVideo( const KURL& url )
{
	if ( url.protocol() != "file" )
		return false;

	closeVideo();

	if ( m_activePlayer->openFile( url.path() ) )
	{
		m_recentVideosAction->add( url );
		onVolumeSliderValueChanged( m_volumeSlider->value() );
		return true;
	}
	else
	{
		KMessageBox::sorry( this, i18n( "Couldn't play the selected file." ) );
		return false;
	}
}

void PlayerWidget::openVideoForSubtitle( const QString& subtitlePath )
{
	static QStringList extensions( QStringList::split( ":", "avi:ogm:mkv:mpeg:mpg:mp4:rv" ) );

	if ( m_subtitlePath != subtitlePath )
	{
		QFileInfo subFileInfo( subtitlePath );
		QString fileName = subFileInfo.baseName( true ).lower();

		QStringList dirFiles = subFileInfo.dir().entryList( QDir::Files|QDir::Readable );
		for ( QStringList::Iterator it = dirFiles.begin(); it != dirFiles.end(); ++it )
		{
			QFileInfo fileInfo( *it );
			if ( extensions.contains( fileInfo.extension( false ).lower() ) )
			{
				if ( fileName.find( fileInfo.baseName( true ).lower() ) == 0 )
				{
					KURL auxURL;
					auxURL.setProtocol( "file" );
					auxURL.setPath( subFileInfo.dir().filePath( *it ) );
					if ( openVideo( auxURL ) )
						m_subtitlePath = subtitlePath;
					return;
				}
			}
		}
	}
}

void PlayerWidget::openVideo()
{
	KFileDialog openDlg( m_lastDir, QString::null, (QWidget*)0, 0, true );
	openDlg.setCaption( i18n( "Open Video" ) );

	if ( openDlg.exec() == QDialog::Accepted )
	{
		m_lastDir = openDlg.selectedFile();
		openVideo( openDlg.selectedURL() );
	}
}

void PlayerWidget::closeVideo()
{
	m_activePlayer->closeFile();
	m_subtitlePath = QString::null;

	m_lastTime = 0;
}

void PlayerWidget::updateControlsState()
{
	if ( m_activePlayer->filePath().isEmpty() )
		setControlsStateClosed();
	else
	{
		setControlsStateOpened();

		int state = m_activePlayer->state();
		if ( state == Player::Playing )
			setControlsStatePlaying();
		else if ( state == Player::Paused )
			setControlsStatePaused();
		else if ( state == Player::Ready )
			setControlsStateStopped();
	}
}

void PlayerWidget::setControlsStateOpened()
{
	m_closeVideoAction->setEnabled( true );
	m_playPauseAction->setEnabled( true );

	const QStringList& audioStreams = m_activePlayer->audioStreams();
	if ( audioStreams.count() )
	{
		m_selectAudioTrackAction->setItems( audioStreams );
		m_selectAudioTrackAction->setEnabled( true );
		m_selectAudioTrackAction->setCurrentItem( m_activePlayer->audioStream() );
	}

	m_positionTagLabel->setEnabled( true );
	m_lengthTagLabel->setEnabled( true );
	m_fpsTagLabel->setEnabled( true );

	if ( m_showPositionEdit )
		m_positionEdit->show();

	updatePositionControls( m_activePlayer->position() );
	updateLengthControls( m_activePlayer->length() );
	updateFramesPerSecondControls( m_activePlayer->framesPerSecond() );
}

void PlayerWidget::setControlsStateClosed()
{
	setControlsStateStopped();

	m_closeVideoAction->setEnabled( false );
	m_playPauseAction->setEnabled( false );

	m_selectAudioTrackAction->setEnabled( false );

	m_positionTagLabel->setEnabled( false );
 	m_positionLabel->setText( QString::null );
	m_positionEdit->hide();
	m_positionEdit->setValue( 0 );

	m_lengthTagLabel->setEnabled( false );
	m_lengthLabel->setText( QString::null );

	m_fpsTagLabel->setEnabled( false );
	m_fpsLabel->setText( QString::null );
}

void PlayerWidget::setControlsStatePlaying()
{
	m_stopAction->setEnabled( true );
	m_playPauseAction->setIcon( "player_pause" );
	m_playPauseAction->setText( i18n( "Pause" ) );
	m_playPauseButton->setIconSet( m_pauseIconSet );
	m_playPauseButton->setTextLabel( i18n( "Pause" ) );
	m_seekBackwardsAction->setEnabled( true );
	m_seekForwardsAction->setEnabled( true );

	m_previousLineAction->setEnabled( true );
	m_nextLineAction->setEnabled( true );

	m_setLineShowTimeAction->setEnabled( true );
	m_setLineHideTimeAction->setEnabled( true );
	m_adjustFromCurrentLineAction->setEnabled( true );
	m_adjustUpToCurrentLineAction->setEnabled( true );
	m_shiftFromCurrentLineAction->setEnabled( true );
	m_shiftUpToCurrentLineAction->setEnabled( true );

	m_seekSlider->setEnabled( true );
}

void PlayerWidget::setControlsStatePaused()
{
	m_playPauseAction->setIcon( "player_play" );
	m_playPauseAction->setText( i18n( "Play" ) );
	m_playPauseButton->setIconSet( m_playIconSet );
	m_playPauseButton->setTextLabel( i18n( "Play" ) );
}

void PlayerWidget::setControlsStateStopped()
{
	m_stopAction->setEnabled( false );

	m_playPauseAction->setIcon( "player_play" );
	m_playPauseAction->setText( i18n( "Play" ) );
	m_playPauseButton->setIconSet( m_playIconSet );
	m_playPauseButton->setTextLabel( i18n( "Play" ) );

	m_seekBackwardsAction->setEnabled( false );
	m_seekForwardsAction->setEnabled( false );

	m_previousLineAction->setEnabled( false );
	m_nextLineAction->setEnabled( false );

	m_setLineShowTimeAction->setEnabled( false );
	m_setLineHideTimeAction->setEnabled( false );
	m_adjustFromCurrentLineAction->setEnabled( false );
	m_adjustUpToCurrentLineAction->setEnabled( false );
	m_shiftFromCurrentLineAction->setEnabled( false );
	m_shiftUpToCurrentLineAction->setEnabled( false );

	updatePositionControls( 0 );
	m_seekSlider->setEnabled( false );

	updatePlayingLine( 0 );
}

// sets the slider position from the video player reported value
void PlayerWidget::updatePositionControls( double seconds )
{
	if ( m_updatePositionControls > 0 )
	{
		Time time( (int)(seconds*1000) );

		updateOverlayLine( time );
		m_positionLabel->setText( time.toString() );
		if ( m_showPositionEdit && ! m_positionEdit->hasFocus() )
			m_positionEdit->setValue( time.timeMillis() );
		m_updateVideoPosition = false;
		m_seekSlider->setValue( (int)((seconds/m_activePlayer->length())*1000) );
		m_updateVideoPosition = true;
	}
 	else if ( m_updatePositionControls < 0 )
		m_updatePositionControls += 2;
}

void PlayerWidget::connectOverlayLine()
{
	if ( ! m_overlayLine )
		return;

	connect( m_overlayLine, SIGNAL(showTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
	connect( m_overlayLine, SIGNAL(hideTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
}

void PlayerWidget::disconnectOverlayLine()
{
	if ( ! m_overlayLine )
		return;

	disconnect( m_overlayLine, SIGNAL(showTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
	disconnect( m_overlayLine, SIGNAL(hideTimeChanged(const Time&)), this, SLOT(invalidateOverlayLine()) );
}

void PlayerWidget::updateOverlayLine( const Time& time )
{
	if ( ! m_subtitle )
		return;

	bool seekedBackwards = m_lastTime > time;
	m_lastTime = time;

	if ( m_overlayLine )
	{
		if ( ! seekedBackwards && time <= m_overlayLine->hideTime() ) // m_overlayLine is the line to show or the next line to show
		{
			if ( time >= m_overlayLine->showTime() ) // m_overlayLine is the line to show
			{
				m_textOverlay->setText( m_overlayLine->text() );
				int styleFlags = m_overlayLine->styleFlags();
				m_textOverlay->setStyle(
					styleFlags & SubtitleLine::Bold,
					styleFlags & SubtitleLine::Italic,
					styleFlags & SubtitleLine::Underline
				);
				updatePlayingLine( m_overlayLine );
			}
			return;
		}
		else // m_overlayLine is no longer the line to show nor the next line to show
		{
			m_textOverlay->setText( "" );
			disconnectOverlayLine();
			m_overlayLine = 0;
			m_lastSearchedLineToShowTime = Time( 23, 59, 59, 999 );
		}
	}

	if ( seekedBackwards || m_lastSearchedLineToShowTime > time )
	{
		// search the next line to show:
		unsigned linesCount = m_subtitle->linesCount();
		for ( unsigned idx = 0; idx < linesCount; ++idx )
		{
			if ( time <= m_subtitle->line( idx )->hideTime() )
			{
				m_lastSearchedLineToShowTime = time;
				m_overlayLine = m_subtitle->line( idx );
				connectOverlayLine();
				if ( m_overlayLine->showTime() <= time && time <= m_overlayLine->hideTime() )
				{
					m_textOverlay->setText( m_overlayLine->text() );
					int styleFlags = m_overlayLine->styleFlags();
					m_textOverlay->setStyle(
						styleFlags & SubtitleLine::Bold,
						styleFlags & SubtitleLine::Italic,
						styleFlags & SubtitleLine::Underline
					);
					updatePlayingLine( m_overlayLine );
				}
				return;
			}
		}
		updatePlayingLine( 0 );
	}

}

void PlayerWidget::invalidateOverlayLine()
{
	m_textOverlay->setText( "" );

	disconnectOverlayLine();
	m_overlayLine = 0;

	m_lastSearchedLineToShowTime = Time( 23, 59, 59, 999 );

	updateOverlayLine( (int)(m_activePlayer->position()*1000) );
}

void PlayerWidget::updatePlayingLine( SubtitleLine* playingLine )
{
	if ( ! playingLine || m_playingLine != playingLine )
	{
		m_playingLine = playingLine;
		emit setPlayingLine( m_playingLine );
	}
}

void PlayerWidget::updateLengthControls( double seconds )
{
	m_lengthLabel->setText( Time( (int)(seconds*1000) ).toString() );
}

void PlayerWidget::updateFramesPerSecondControls( double framesPerSecond )
{
	m_fpsLabel->setText( QString::number( framesPerSecond, 'f', 3 ) );
}

void PlayerWidget::seekBackwards()
{
	double position = m_activePlayer->position() - m_generalConfig.optionAsInt( OPT_SEEK_FORWARDS_BACKWARDS_SECS );
	m_activePlayer->seek( position > 0.0 ? position : 0.0 );
}

void PlayerWidget::seekForwards()
{
	double position = m_activePlayer->position() + m_generalConfig.optionAsInt( OPT_SEEK_FORWARDS_BACKWARDS_SECS );
	m_activePlayer->seek( position <= m_activePlayer->length() ? position : m_activePlayer->length() );
}

void PlayerWidget::jumpToPreviousLine()
{
	if ( m_playingLine )
	{
		SubtitleLine* prevLine = m_overlayLine->prevLine();
		if ( prevLine )
		{
			m_activePlayer->seek( (prevLine->showTime().timeMillis()) / 1000.0 );
			emit setCurrentLine( prevLine );
		}
	}
}

void PlayerWidget::jumpToNextLine()
{
	if ( m_playingLine )
	{
		SubtitleLine* nextLine = m_overlayLine->nextLine();
		if ( nextLine )
		{
			m_activePlayer->seek( (nextLine->showTime().timeMillis()) / 1000.0 );
			emit setCurrentLine( nextLine );
		}
	}
}

void PlayerWidget::setCurrentLineShowTime()
{
	emit setCurrentLineShowTime( Time( (int)(m_activePlayer->position()*1000 )) );
}

void PlayerWidget::setCurrentLineHideTime()
{
	emit setCurrentLineHideTime( Time( (int)(m_activePlayer->position()*1000 )) );
}

void PlayerWidget::adjustFromCurrentLine()
{
	emit adjustFromCurrentLine( Time( (int)(m_activePlayer->position()*1000 )) );
}

void PlayerWidget::adjustUpToCurrentLine()
{
	emit adjustUpToCurrentLine( Time( (int)(m_activePlayer->position()*1000 )) );
}

void PlayerWidget::shiftFromCurrentLine()
{
	emit shiftFromCurrentLine( Time( (int)(m_activePlayer->position()*1000 )) );
}

void PlayerWidget::shiftUpToCurrentLine()
{
	emit shiftUpToCurrentLine( Time( (int)(m_activePlayer->position()*1000 )) );
}


void PlayerWidget::onSeekSliderPressed()
{
	m_updatePositionControls = 0;
}

void PlayerWidget::onSeekSliderReleased()
{
	m_updatePositionControls = MAGIC_NUMBER;
}

void PlayerWidget::onSeekSliderValueChanged( int position )
{
	if ( m_updateVideoPosition )
	{
		m_updatePositionControls = MAGIC_NUMBER;
		m_activePlayer->seek( m_activePlayer->length()*position/1000.0 );
	}
}

void PlayerWidget::onSeekSliderDragged( int position )
{
	m_positionLabel->setText( Time( (int)(m_activePlayer->length()*position) ).toString() );
	if ( m_showPositionEdit )
		m_positionEdit->setValue( (int)(m_activePlayer->length()*position) );
}

void PlayerWidget::onPositionEditValueChanged( int position )
{
	if ( m_positionEdit->hasFocus() )
	{
		m_updatePositionControls = MAGIC_NUMBER;
		m_activePlayer->seek( position/1000.0 );
	}
}

void PlayerWidget::onVolumeSliderValueChanged( int position )
{
	m_activePlayer->setVolume( 100 - position );
}

void PlayerWidget::onMuteButtonToggled( bool value )
{
	if ( m_toggleMuteAction->isChecked() != value )
		m_toggleMuteAction->setChecked( value );

	m_activePlayer->setMuted( value );
}

void PlayerWidget::onLinkCurrentToPositionToggled( bool value )
{
	if ( m_linkCurrentLineToVideoAction->isChecked() != value )
		m_linkCurrentLineToVideoAction->setChecked( value );

	if ( value )
		connect( this, SIGNAL( setPlayingLine(SubtitleLine*) ), this, SIGNAL( setCurrentLine(SubtitleLine*) ) );
	else
		disconnect( this, SIGNAL( setPlayingLine(SubtitleLine*) ), this, SIGNAL( setCurrentLine(SubtitleLine*) ) );
}

void PlayerWidget::showAudioTracksMenu( const QPoint& point )
{
	const QStringList& audioStreams = m_activePlayer->audioStreams();
	if ( audioStreams.count() )
	{
		KIconLoader iconLoader;
		KPopupMenu popup;
		popup.insertTitle( iconLoader.loadIcon( "kcmsound", KIcon::Small ), i18n( "Audio Tracks" ), -1 );
		int streamIndex = 0;
		for ( QStringList::ConstIterator it = audioStreams.begin(); it != audioStreams.end(); ++it, ++streamIndex )
			popup.insertItem( *it, streamIndex );
		popup.setItemChecked( m_activePlayer->audioStream(), true );
		streamIndex = popup.exec( point );
		if ( streamIndex != -1 )
			m_activePlayer->setAudioStream( streamIndex );
	}
}

void PlayerWidget::mouseReleaseEvent( QMouseEvent* e )
{
	if ( e->button() == Qt::RightButton && m_infoControlsGroupBox->geometry().contains( m_infoControlsGroupBox->mapFromGlobal( e->globalPos() ) ) )
	{
		e->accept();

		KPopupMenu popup;
// 		popup.insertTitle( iconLoader.loadIcon( "kcmsound", KIcon::Small ), i18n( "Audio Tracks" ), -1 );
		popup.insertItem( i18n( "Show editable position control" ), 0 );
		popup.setItemChecked( 0, m_showPositionEdit );
		int index = popup.exec( e->globalPos() );
		if ( index == 0 )
		{
			m_showPositionEdit = ! m_showPositionEdit;
			if ( m_showPositionEdit && ! m_activePlayer->filePath().isEmpty() )
				m_positionEdit->show();
			else
				m_positionEdit->hide();
		}
	}
	else
		QWidget::mouseReleaseEvent( e );
}

bool PlayerWidget::eventFilter( QObject* object, QEvent* event )
{
	if ( object != m_layeredWidget )
		return false;

	if ( event->type() == QEvent::DragEnter )
	{
		QDragEnterEvent* dragEnterEvent = static_cast<QDragEnterEvent*>( event );
		dragEnterEvent->accept( KURLDrag::canDecode( dragEnterEvent ) );
		return true;
	}
	else if ( event->type() == QEvent::DragMove )
	{
		return true;
	}
	else if ( event->type() == QEvent::Drop )
	{
		QDropEvent* dropEvent = static_cast<QDropEvent*>( event );

		KURL::List urls;
		if ( KURLDrag::decode( dropEvent, urls ) && ! urls.isEmpty() )
		{
			for ( KURL::List::Iterator it = urls.begin(); it != urls.end(); ++it )
			{
				KURL& url = *it;

				if ( url.protocol() != "file" )
					continue;

				openVideo( url );
				break;
			}
		}

		return true; // eat event
	}
	else
		return false; // standard event processing
}

void PlayerWidget::onGeneralOptionChanged( const QString& option, const QString& value )
{
	if ( option == OPT_PLAYER_BACKEND )
	{
		QString videoPath = m_activePlayer->filePath();
		if ( ! videoPath.isEmpty() )
			m_activePlayer->closeFile();

		disconnect( m_activePlayer, SIGNAL(finalized(Player*)), this, SLOT(disconnectActivePlayer()) );
		disconnect( m_activePlayer, SIGNAL(initialized(Player*)), this, SLOT(connectActivePlayer()) );
		disconnectActivePlayer();

		m_activePlayer = PlayersManager::setActivePlayer( value, m_layeredWidget );

		connect( m_activePlayer, SIGNAL(finalized(Player*)), this, SLOT(disconnectActivePlayer()) );
		connect( m_activePlayer, SIGNAL(initialized(Player*)), this, SLOT(connectActivePlayer()) );
		connectActivePlayer();

		onVolumeSliderValueChanged( m_volumeSlider->value() );
		onMuteButtonToggled( m_toggleMuteAction->isChecked() );

		if ( ! videoPath.isEmpty() )
			m_activePlayer->openFile( videoPath );
	}
}

void PlayerWidget::onSubsFontOptionChanged()
{
	QFont font( m_textOverlay->font() );
	font.setFamily( m_fontConfigGroup.option( OPT_FAMILY ) );
	font.setPointSize( m_fontConfigGroup.optionAsInt( OPT_SIZE ) );
	m_textOverlay->setFont( font );
	m_textOverlay->setColor( m_fontConfigGroup.optionAsColor( OPT_COLOR ) );
}

void PlayerWidget::loadConfig()
{
	KConfigGroupSaver saver( kapp->config(), "Player Settings" );

	m_lastDir = kapp->config()->readPathEntry( "LastDir", QDir::homeDirPath() );
	m_recentVideosAction->loadEntries( kapp->config(), "Recent Videos" );

	m_volumeSlider->setValue( kapp->config()->readNumEntry( "Volume", 0 ) );
	onVolumeSliderValueChanged( m_volumeSlider->value() );

	m_toggleMuteAction->setChecked( kapp->config()->readBoolEntry( "Muted", false ) );
	onMuteButtonToggled( m_toggleMuteAction->isChecked() );

	//m_linkCurrentLineToVideoAction->setChecked( kapp->config()->readBoolEntry( "LinkCurrentLineToVideo", false ) );
	//onLinkCurrentToPositionToggled( m_linkCurrentLineToVideoAction->isChecked() );

	m_showPositionEdit = kapp->config()->readBoolEntry( "EditablePositionControl", false );
	if ( m_showPositionEdit && ! m_activePlayer->filePath().isEmpty() )
		m_positionEdit->show();
	else
		m_positionEdit->hide();
}

void PlayerWidget::saveConfig()
{
	KConfigGroupSaver saver( kapp->config(), "Player Settings" );

	kapp->config()->writePathEntry( "LastDir", m_lastDir );
	m_recentVideosAction->saveEntries( kapp->config(), "Recent Videos" );

	kapp->config()->writeEntry( "Volume", m_volumeSlider->value() );
	kapp->config()->writeEntry( "Muted", m_toggleMuteAction->isChecked() );

	//kapp->config()->writeEntry( "LinkCurrentLineToVideo", m_linkCurrentLineToVideoAction->isChecked() );

	kapp->config()->writeEntry( "EditablePositionControl", m_showPositionEdit );
}

#include "playerwidget.moc"
