/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *   based on Kaffeine by Jürgen Kofler                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "gstplayer.h"
#include "gstplayerconfigdefs.h"
#include "gstplayerconfigwidget.h"

#include <gst/interfaces/xoverlay.h>

#include <kdebug.h>
#include <klocale.h>
#include <kurl.h>

#include <qtimer.h>

using namespace SubtitleComposer;

GstPlayer::GstPlayer():
	Player( defaultConfigGroup(), "GstPlayer" ),
	m_play( NULL ),
	m_videoSink( NULL ),
	m_audioSink( NULL ),
	m_bus( NULL ),
	m_busTimer( new QTimer( this ) ),
	m_updatePosition( false ),
	m_length( GST_CLOCK_TIME_NONE )
{
	connect( m_busTimer, SIGNAL(timeout()), this, SLOT(slotReadBus()) );
}

GstPlayer::~GstPlayer()
{
	__finalize();
}

PlayerVideoWidget* GstPlayer::_initialize( QWidget* videoWidgetParent )
{
	if ( ! initGStreamer() )
	{
		kdError() << "GStreamer initialization failed!" << endl;
		return 0;
	}

	return new PlayerVideoWidget( videoWidgetParent, "MPlayerVideoWidget", Qt::WNoAutoErase );
}

bool GstPlayer::_finalize()
{
	return __finalize();
}

bool GstPlayer::__finalize()
{
	if ( isInitialized() )
	{
		if ( GST_IS_X_OVERLAY( m_videoSink ) )
		{
			gst_x_overlay_set_xwindow_id( GST_X_OVERLAY( m_videoSink ), 0 );
// 			gst_x_overlay_expose( GST_X_OVERLAY( m_videoSink ) );
		}

// 		if ( ! videoWidgetDestroyed )
// 		{
// 			m_videoWidget->disconnect();
// 			m_videoWidget->hide();
// 			delete m_videoWidget;
// 		}
//
// 		m_videoWidget = 0;

		deletePlaybin();

		return true;
	}
	else
		return false;
}

AppConfigGroup GstPlayer::defaultConfigGroup()
{
	QMap<QString,QString> keysAndDefaults;

	keysAndDefaults[OPT_AUDIOSINK] = QString::null;
	keysAndDefaults[OPT_VIDEOSINK] = QString::null;

	return AppConfigGroup( "GStreamer", keysAndDefaults );
}

SubtitleComposer::AppConfigGroupWidget* GstPlayer::newAppConfigGroupWidget( QWidget* parent )
{
	return new GstPlayerConfigWidget( m_configGroup, parent );
}

bool GstPlayer::_openFile( const QString& filePath )
{
	return __openFile( filePath, true );
}

bool GstPlayer::__openFile( const QString& filePath, bool play )
{
	if ( ! m_play && ! createPlaybin() )
		return false;

	changeState( GST_STATE_READY, 1000 );

	KURL url;
	url.setProtocol( "file" );
	url.setPath( filePath );

	gchar *uri = g_strdup( url.url().local8Bit() );
	g_object_set( G_OBJECT( m_play ), "uri", uri, NULL );
	g_free( uri );

	g_object_set( G_OBJECT(m_play), "suburi", NULL, NULL );
	//g_object_set( G_OBJECT(m_play), "current-subpicture", -2, NULL ); // how to disable this?
	//g_object_set( G_OBJECT(m_play), "current-text", -2, NULL ); // how to disable this?
	g_object_set( G_OBJECT(m_play), "subtitle-font-desc", "Sans 0", NULL ); // make the font as small as possible...

	m_length = GST_CLOCK_TIME_NONE;

	if ( play ) // needed when first opening file to get stream data
	{
		if ( ! changeState( GST_STATE_PLAYING, 3000 ) )
			return false;

		updateState( Playing );
		updateAudioData();
		updateVideoData();
	}

	return true;
}

bool GstPlayer::_closeFile()
{
	if ( ! m_play )
		return false;

	if ( _stop() )
		return true;

	return false;
}

bool GstPlayer::_play()
{
	if ( ! m_play )
		return false;

	return changeState( GST_STATE_PLAYING, 1000 );
}

bool GstPlayer::_pause()
{
	if ( ! m_play )
		return false;

	return changeState( GST_STATE_PAUSED, 1000 );
}

bool GstPlayer::_seek( double seconds )
{
	if ( ! m_play )
		return false;

	gst_element_seek_simple(
		m_play,
		GST_FORMAT_TIME, // time in nanoseconds
		(GstSeekFlags)(GST_SEEK_FLAG_FLUSH|GST_SEEK_FLAG_ACCURATE), // accurate seek requested, flush currently playing
		(gint64)(seconds*GST_SECOND)
	);

	return true;
}

bool GstPlayer::_stop()
{
	if ( ! m_play )
		return false;

	bool stopped = changeState( GST_STATE_READY, 1000 );
	if ( stopped )
		g_object_set( G_OBJECT( m_play ), "current-audio", audioStream(), NULL );
	return true;
}

bool GstPlayer::_setAudioStream( int /*audioStream*/ )
{
	if ( ! m_play )
		return false;

	return false;
	// g_object_set( G_OBJECT( m_play ), "current-audio", audioStream, NULL );
}

bool GstPlayer::_setVolume( double volume )
{
	if ( ! m_play )
		return false;

	g_object_set( G_OBJECT( m_play ), "volume", volume * 0.01, NULL );

	return true;
}

bool GstPlayer::initGStreamer()
{
	if ( ! gst_init_check( NULL, NULL, NULL ) )
		return false;

	// Init audio driver

	QStringList audioSinkNames = QStringList::split( " ", "alsasink osssink gconfaudiosink artsdsink autoaudiosink" );
	audioSinkNames.prepend( m_configGroup.option( OPT_AUDIOSINK ) );
	for ( QStringList::Iterator it = audioSinkNames.begin(); it != audioSinkNames.end(); ++it )
		if ( ! (*it).isEmpty() && (m_audioSink = gst_element_factory_make( (*it).ascii(), "audiosink" )) != NULL )
			break;

	if ( m_audioSink == NULL )
		return false;
	else
		gst_element_set_state( m_audioSink, GST_STATE_READY );


	// Init video driver

	QStringList videoSinkNames = QStringList::split( " ", "xvimagesink ximagesink gconfvideosink autovideosink" );
	videoSinkNames.prepend( m_configGroup.option( OPT_VIDEOSINK ) );
	for ( QStringList::Iterator it = videoSinkNames.begin(); it != videoSinkNames.end(); ++it )
		if ( ! (*it).isEmpty() && (m_videoSink = gst_element_factory_make( (*it).ascii(), "videosink" )) != NULL )
			break;

	if ( m_videoSink == NULL )
		return false;
	else
		gst_element_set_state( m_videoSink, GST_STATE_READY );

	return true;
}

bool GstPlayer::createPlaybin()
{
	m_play = gst_element_factory_make( "playbin", "play" );
	if ( ! m_play )
		return false;

	if ( ! m_videoSink || ! m_audioSink )
	{
		gst_object_unref( GST_OBJECT(m_play) );
		return false;
	}

	g_object_set( G_OBJECT( m_play ), "video-sink", m_videoSink, NULL );
	g_object_set( G_OBJECT( m_play ), "audio-sink", m_audioSink, NULL );
	gst_element_set_state( m_play, GST_STATE_READY );

	m_bus = gst_pipeline_get_bus( GST_PIPELINE(m_play) );

	// FIXME this could replace timers, but doesn't work without a glib event loop running... maybe for Qt4?
	// 	gst_bus_add_watch( m_bus, busWatchCallBack, (gpointer)this );

	m_busTimer->start( 20 );

	return true;
}

void GstPlayer::deletePlaybin()
{
	if ( m_bus )
	{
		m_busTimer->stop();
		gst_object_unref( GST_OBJECT( m_bus ) );
		m_bus = NULL;
	}

	if ( m_play )
	{
		gst_element_set_state( m_play, GST_STATE_NULL );
		gst_object_unref( GST_OBJECT( m_play ) );
		m_play = NULL;
	}
}

bool GstPlayer::changeState( GstState state, unsigned timeout )
{
	if ( GST_STATE( m_play ) == state )
		return true;

	if ( m_videoSink && GST_IS_X_OVERLAY( m_videoSink ) )
	{
		gst_x_overlay_set_xwindow_id( GST_X_OVERLAY( m_videoSink ), videoWidget()->videoLayer()->winId() );
		gst_x_overlay_expose( GST_X_OVERLAY( m_videoSink ) );
	}

	GstStateChangeReturn ret = gst_element_set_state( m_play, state );
	if ( ret == GST_STATE_CHANGE_SUCCESS )
		return true;

	if ( ret != GST_STATE_CHANGE_ASYNC )
		return false;

	if ( ! timeout )
		return true;

 	// wait for state change or timeout
	return gst_element_get_state( m_play, NULL, NULL, timeout*GST_MSECOND ) == GST_STATE_CHANGE_SUCCESS;
}

void GstPlayer::updatePosition()
{
	if ( ! m_play )
		return;

	gint64 time;
	GstFormat fmt = GST_FORMAT_TIME;

	if ( ! GST_CLOCK_TIME_IS_VALID ( m_length ) )
	{
		if ( gst_element_query_duration( m_play, &fmt, &time ) )
		{
			m_length = time;
			updateLength( (double)time / GST_SECOND );
		}
	}

	if ( gst_element_query_position( m_play, &fmt, &time ) )
		Player::updatePosition( ((double)time / GST_SECOND) );
}

void GstPlayer::slotReadBus()
{
	if ( ! isInitialized() )
		return;

	if ( m_updatePosition )
		updatePosition();

	if ( ! m_bus )
		return;

	GstMessage* msg;
	while ( (msg = gst_bus_pop( m_bus )) != 0 )
	{
		GstElement* src = (GstElement*)GST_MESSAGE_SRC( msg );

		switch( GST_MESSAGE_TYPE( msg ) )
		{
			case GST_MESSAGE_STATE_CHANGED:
			{
				if ( src != m_play )
					break;

				GstState old, cur, pending;
				gst_message_parse_state_changed( msg, &old, &cur, &pending );
				if ( cur == old )
					break;

				if ( cur == GST_STATE_PAUSED )
				{
					m_updatePosition = false;
					updateState( Paused );
				}
				else if ( cur == GST_STATE_PLAYING )
				{
					m_updatePosition = true;
					updateState( Playing );
				}
				else if ( cur == GST_STATE_READY )
				{
					m_updatePosition = false;
					updateState( Ready );
				}

				if ( old == GST_STATE_READY )
					updateVideoData();

				break;
			}
			case GST_MESSAGE_SEGMENT_DONE:
			{
				GstFormat format = GST_FORMAT_TIME;
				gint64 position;
				gst_message_parse_segment_done( msg, &format, &position );
				Player::updatePosition( (double)position / GST_SECOND );
			}
			case GST_MESSAGE_DURATION:
			{
				GstFormat format = GST_FORMAT_TIME;
				gint64 duration;
				gst_message_parse_duration( msg, &format, &duration );
				m_length = duration;
				updateLength( (double)duration / GST_SECOND );
			}
			case GST_MESSAGE_EOS:
			{
				m_updatePosition = false;
				updateState( Ready );
				__openFile( filePath(), false );
				return;
			}
			case GST_MESSAGE_ERROR:
			{
				gchar* debug = NULL;
				GError* error = NULL;

				gst_message_parse_error( msg, &error, &debug );
				g_error_free( error );
				g_free( debug );
				gst_element_set_state( m_play, GST_STATE_NULL );
				kdDebug() << "GST_MESSAGE_ERROR:" << QString( error->message ) << endl << QString( debug ) << endl;
				break;
			}

			default:
				break;
		}

		gst_message_unref( msg );
	}
}


GList* GstPlayer::streamInfoForType( const gchar* type /*"AUDIO", "SUBTITLE" or "VIDEO"*/ )
{
	if ( m_play == NULL )
		return NULL;

	GValueArray* infoArray = NULL;
	g_object_get( m_play, "stream-info-value-array", &infoArray, NULL );
	if ( infoArray == NULL )
		return NULL;

	GList* ret = NULL;
	for ( guint idx = 0; idx < infoArray->n_values; ++idx )
	{
		GObject* info = (GObject*)g_value_get_object( g_value_array_get_nth( infoArray, idx ) );
		if ( info )
		{
			gint typeID = -1;
			g_object_get( info, "type", &typeID, NULL );
			GParamSpec* pspec = g_object_class_find_property( G_OBJECT_GET_CLASS( info ), "type" );
			GEnumValue* val = g_enum_get_value( G_PARAM_SPEC_ENUM( pspec )->enum_class, typeID );
			if ( val )
			{
				if( g_ascii_strcasecmp( val->value_nick, type ) == 0 || g_ascii_strcasecmp( val->value_name, type ) == 0 )
					ret = g_list_prepend( ret, g_object_ref( info ) );
			}
		}
	}
	g_value_array_free( infoArray );

	return g_list_reverse( ret );
}

void GstPlayer::updateVideoData()
{
	if ( ! m_play )
		return;

	GList* list = streamInfoForType( "VIDEO" );
	if ( list == NULL )
		return;

	for ( GList* l = list; l != NULL; l = l->next )
	{
		GstPad* pad = NULL;
		g_object_get( l->data, "object", &pad, NULL );

		GstCaps* caps = gst_pad_get_negotiated_caps( pad );
		if ( ! caps )
			continue;

		const GstStructure* capsStruct = gst_caps_get_structure( caps, 0 );
		if ( ! capsStruct )
			continue;

		int width = 0, height = 0;
		gst_structure_get_int( capsStruct, "width", &width );
		gst_structure_get_int( capsStruct, "height", &height );
		const GValue* par;
		if ( (par = gst_structure_get_value( capsStruct, "pixel-aspect-ratio" )) )
		{
			int num = gst_value_get_fraction_numerator( par );
			int den = gst_value_get_fraction_denominator( par );
			if ( num > den )
				width = (int)((float)num * width / den);
			else
				height = (int)((float)den * height / num);
		}
		videoWidget()->setVideoResolution( width, height );

		const GValue* fps;
		if ( (fps = gst_structure_get_value( capsStruct, "framerate" )) )
		{
			int num = gst_value_get_fraction_numerator( fps );
			int den = gst_value_get_fraction_denominator( fps );
			updateFramesPerSecond( (double)num/den );
		}

		gst_caps_unref( caps );
		return;
	}
}


void GstPlayer::updateAudioData()
{
	GList* list = streamInfoForType( "AUDIO" );
	if ( list == NULL )
		return;

	QStringList audioStreams;

	gint trackNumber = 1;
	for ( GList* l = list; l != NULL; l = l->next, ++trackNumber )
	{
		gchar* languageCode = NULL;
		gchar* codec = NULL;
		g_object_get( l->data, "codec", &codec, "language-code", &languageCode, NULL );

		QString audioStreamName;
		if ( languageCode )
		{
			audioStreamName = QString( languageCode );
			g_free( languageCode );
		}
		if ( codec )
		{
			audioStreamName += (audioStreamName.isEmpty() ? "" : " / ") + QString( codec );
			g_free( codec );
		}
		if ( audioStreamName.isEmpty() )
			audioStreamName = i18n( "Audio Track #%1" ).arg( trackNumber );
		audioStreams << audioStreamName;
	}

	g_list_foreach( list, (GFunc)g_object_unref, NULL );
	g_list_free( list );

	updateAudioStreams( audioStreams );
}

/*gboolean GstPlayer::busWatchCallBack( GstBus* bus, GstMessage* msg, gpointer data )
{
	qDebug( "processing message" );

	GstPlayer* player = static_cast<GstPlayer*>( data );

	switch( GST_MESSAGE_TYPE( msg ) )
	{
		// same as slotReadBus
	}

	return true;
}*/
