/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *   based on smplayer by Ricardo Villalba                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mplayer.h"
#include "mplayerconfigdefs.h"
#include "mplayerconfigwidget.h"
#include "mplayerprocess.h"
#include <qxtsignalwaiter.h>

#include <qdir.h>
#include <qfileinfo.h>

#include <klocale.h>
#include <kstandarddirs.h>

using namespace SubtitleComposer;

MPlayer::MPlayer():
	Player( defaultsConfigGroup(), "MPlayer" ),
	m_process( new MPlayerProcess( m_configGroup, this ) ),
	m_position( 0.0 ),
	m_reportUpdates( true )
{
	connect( m_process, SIGNAL(receivedCurrentPosition(double)),
			 this, SLOT(onCurrentPositionReceived(double)) );

	connect( m_process, SIGNAL(receivedPauseToggled()),
			 this, SLOT(onPauseToggledReceived()) );

	connect( m_process, SIGNAL(processExited()),
			 this, SLOT(onProcessExited()) );

	connect( m_process, SIGNAL(receivedWindowResolution(int,int)),
			 this, SLOT(onWindowResolutionReceived(int,int)) );

	connect( m_process, SIGNAL(receivedEndOfFile()),
			 this, SLOT(onEndOfFileReceived()) );
}

MPlayer::~MPlayer()
{
	__finalize();
}

PlayerVideoWidget* MPlayer::_initialize( QWidget* videoWidgetParent )
{
	return new PlayerVideoWidget( videoWidgetParent, "GstPlayerVideoWidget", Qt::WNoAutoErase );
}

bool MPlayer::_finalize()
{
	return __finalize();
}

bool MPlayer::__finalize()
{
	if ( isInitialized() )
	{
// 		if ( ! videoWidgetDestroyed )
// 		{
// 			m_videoWidget->disconnect();
// 			m_videoWidget->hide();
// 			delete m_videoWidget;
// 		}
//
// 		m_videoWidget = 0;

		return true;
	}
	else
		return false;
}

AppConfigGroup MPlayer::defaultsConfigGroup()
{
	QMap<QString,QString> keysAndDefaults;

	keysAndDefaults[OPT_EXEC_PATH] = "mplayer";
	keysAndDefaults[OPT_VO] = QString::null;
	keysAndDefaults[OPT_AO] = QString::null;

	keysAndDefaults[OPT_CACHE] = "5120";

	keysAndDefaults[OPT_VOLUME_NORM] = "false";

	keysAndDefaults[OPT_INPUT_CONF_PATH] = ::locate( "appdata", "input.conf" );

	keysAndDefaults[OPT_FRAME_DROP] = "false";
	keysAndDefaults[OPT_HARD_FRAME_DROP] = "false";
	keysAndDefaults[OPT_AUTOSYNC_FACTOR] = QString::null;

	keysAndDefaults[OPT_REGEX_ENDOFFILE] = "Exiting... \\(End of file\\)";
	keysAndDefaults[OPT_REGEX_NOVIDEO] = "Video: no video";

	return AppConfigGroup( "MPlayer", keysAndDefaults );
}

SubtitleComposer::AppConfigGroupWidget* MPlayer::newAppConfigGroupWidget( QWidget* parent )
{
	return new MPlayerConfigWidget( m_configGroup, parent );
}


bool MPlayer::_openFile( const QString& filePath )
{
	m_position = 0.0;

	QxtSignalWaiter waiter( m_process, SIGNAL( mediaDataLoaded() ) );
	setupProcessArgs( filePath );
	if ( ! m_process->start()  )
		return false;

	if ( ! waiter.wait( 2000 ) ) // timed out
	{
		_stop();
		return false;
	}

	const MediaData& mediaData = m_process->mediaData();

	QStringList audioStreams;
	for ( int trackNumber = 1; trackNumber <= mediaData.audioTracks.tracksCount(); ++trackNumber )
	{
		const TrackData& trackData = mediaData.audioTracks.itemAt( trackNumber-1 );
		QString audioStreamName;
		if ( ! trackData.lang().isEmpty() )
			audioStreamName = trackData.lang();
		if ( ! trackData.name().isEmpty() )
			audioStreamName += (audioStreamName.isEmpty() ? "" : " / ") + trackData.name();
		if ( audioStreamName.isEmpty() )
			audioStreamName = i18n( "Audio Track #%1" ).arg( trackNumber );

		audioStreams << audioStreamName;
	}

	updateAudioStreams( audioStreams );
	updateLength( mediaData.duration );
	if ( mediaData.videoFPS != 0.0 )
		updateFramesPerSecond( mediaData.videoFPS );

	return true;
}

bool MPlayer::_closeFile()
{
	return _stop();
}

bool MPlayer::_stop()
{
	if ( ! m_process->isRunning() )
		return true;

	m_process->sendQuit();
	m_process->waitForProcess();

	return m_process->isRunning();
}

bool MPlayer::_play()
{
	if ( ! m_process->isRunning() ) // state was Ready
	{
		m_position = 0.0;
		setupProcessArgs( filePath() );
		return m_process->start();
	}
	else  // state was Paused
		m_process->sendPause();

	return true;
}

bool MPlayer::_pause()
{
	if ( ! m_process->isRunning() )
	{
		m_position = 0.0;
		setupProcessArgs( filePath() );
		if ( ! m_process->start() )
			return false;
		m_process->writeToStdin( "pause\n" );
		return true;
	}

	m_process->sendPause();
	return true;
}


bool MPlayer::_seek( double seconds )
{
	bool wasPaused = isPaused();

	m_reportUpdates = false;

	m_process->sendMute( true );
	double seekPosition = seconds;
/*	qDebug( "orig pos %f", m_position );
	qDebug( "seek pos %f", seconds );*/
	do
	{
		m_process->sendSeek( seekPosition, true );
		seekPosition -= 1.0;
// 		qDebug( "current pos %f", m_position );
	}
	while ( m_position > seconds );

	m_process->sendMute( true );

	if ( ! wasPaused )
		m_process->sendPause();

	m_reportUpdates = true;

	updatePosition( m_position );

	return true;
}

bool MPlayer::_setAudioStream( int audioStream )
{
	m_process->sendAudioStream( audioStream );
	return true;
}

bool MPlayer::_setVolume( double volume )
{
	m_process->sendVolume( volume );
	return true;
}


void MPlayer::onProcessExited()
{
	updateState( Ready );
}

void MPlayer::onEndOfFileReceived()
{
	updateState( Ready );
}

void MPlayer::onCurrentPositionReceived( double seconds )
{
	m_position = seconds;

	if ( ! m_reportUpdates )
		return;

	if ( ! isPlaying() )
		updateState( Playing );

	updatePosition( seconds );
}

void MPlayer::onPauseToggledReceived()
{
	if ( ! m_reportUpdates )
		return;

	updateState( isPlaying() ? Paused : Playing );
}

void MPlayer::onWindowResolutionReceived( int width, int height )
{
	videoWidget()->setVideoResolution( width, height );
}

void MPlayer::setupProcessArgs( const QString& filePath )
{
	m_process->clearArguments();

	m_process->addArgument( m_configGroup.option( OPT_EXEC_PATH ) );

	if ( audioStream() >= 0 && audioStreams().count() > 1 )
	{
		m_process->addArgument( "-aid" );
		m_process->addArgument( QString::number( audioStream() ) );
	}

	m_process->addArgument( "-noquiet" );


	// No mplayer fullscreen mode
	m_process->addArgument( "-nofs" );

	// makes mplayer emit all kinds of additional information
	m_process->addArgument( "-identify" );

	m_process->addArgument( "-slave" );

	if ( ! m_configGroup.option( OPT_VO ).isEmpty() )
	{
		m_process->addArgument( "-vo" );
		m_process->addArgument( m_configGroup.option( OPT_VO ) );
	}

	if ( ! m_configGroup.option( OPT_AO ).isEmpty() )
	{
		m_process->addArgument( "-ao");
		m_process->addArgument( m_configGroup.option( OPT_AO ) );
	}

	m_process->addArgument( "-zoom" );
	m_process->addArgument( "-nokeepaspect" );

	if ( m_configGroup.optionAsBool( OPT_FRAME_DROP ) )
		m_process->addArgument( "-framedrop" );

	if ( m_configGroup.optionAsBool( OPT_HARD_FRAME_DROP ) )
		m_process->addArgument( "-hardframedrop" );

	if ( ! m_configGroup.option( OPT_AUTOSYNC_FACTOR ).isEmpty() )
	{
		m_process->addArgument( "-autosync" );
		m_process->addArgument( m_configGroup.option( OPT_AUTOSYNC_FACTOR ) );
	}

	if ( ! m_configGroup.option( OPT_INPUT_CONF_PATH ).isEmpty() )
	{
		m_process->addArgument( "-input" );
		m_process->addArgument( "conf=" + m_configGroup.option( OPT_INPUT_CONF_PATH ) );
	}

	m_process->addArgument( "-wid" );
	m_process->addArgument( QString::number( (int)videoWidget()->videoLayer()->winId() ) );

	// Set monitoraspect to desktop aspect
	m_process->addArgument( "-monitoraspect" );
	m_process->addArgument( QString::number( videoWidget()->desktopAspectRatio() ) );

	// Turn off automatic subtitle file loading
	m_process->addArgument( "-noautosub" );

	if ( ! m_configGroup.option( OPT_CACHE ).isEmpty() )
	{
		m_process->addArgument( "-cache" );
		m_process->addArgument( m_configGroup.option( OPT_CACHE ) );

		m_process->addArgument( "-cache-min" );
		m_process->addArgument( QString::number( 99 ) );

		m_process->addArgument( "-cache-seek-min" );
		m_process->addArgument( QString::number( 99 ) );
	}

	m_process->addArgument( "-osdlevel" );
	m_process->addArgument( QString::number( 0 ) ); // no OSD

	if ( m_configGroup.optionAsBool( OPT_VOLUME_NORM ) )
	{
		m_process->addArgument( "-af" );
		m_process->addArgument( "volnorm=2" );
	}

	m_process->addArgument( "-softvol" );
	//m_process->addArgument( "-softvol-max" );
	//m_process->addArgument( QString::number( 110 ) );

	m_process->addArgument( filePath );
}

