/***************************************************************************
 *   smplayer, GUI front-end for mplayer.                                  *
 *   Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>           *
 *                                                                         *
 *   modified for inclusion in Subtitle Composer                           *
 *   Copyright (C) 2007 by Sergio Pistone <sergio_pistone@yahoo.com.ar>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mplayerprocess.h"
#include "mplayerconfigdefs.h"

#include <qxtsignalwaiter.h>

#include <qapplication.h>
#include <qthread.h>
#include <qregexp.h>
#include <qstringlist.h>

class Sleeper : public QThread
{
	public:
		static void sleep( unsigned long secs )		{ QThread::sleep( secs ); }
		static void msleep( unsigned long msecs )	{ QThread::msleep( msecs ); }
		static void usleep( unsigned long usecs )	{ QThread::usleep(usecs); }
};

using namespace SubtitleComposer;

MPlayerProcess::MPlayerProcess( const AppConfigGroup& configGroup, QObject* parent, const char* name ):
	QProcess( parent, name ),
	m_configGroup( configGroup ),
	m_mediaData(),
	m_incompleteLine(),
	m_hasNotifiedRunning( false ),
	m_isPaused( false )
{
	setCommunication( QProcess::Stdin | QProcess::Stdout | QProcess::Stderr );

	connect( this, SIGNAL(readyReadStdout()), this, SLOT(read()) );
	connect( this, SIGNAL(readyReadStderr()), this, SLOT(readFromStderr()) );

	initRegularExpressions();
}

MPlayerProcess::~MPlayerProcess()
{
}

const MediaData& MPlayerProcess::mediaData()
{
	return m_mediaData;
}

bool MPlayerProcess::start( QStringList* env )
{
	initRegularExpressions(); // Update configurable regular expressions

	m_mediaData.reset();
	m_hasNotifiedRunning = false;
	m_isPaused = false;
	m_incompleteLine = QString::null;

	return QProcess::start( env );
}

void MPlayerProcess::waitForProcess()
{
	static const unsigned long sleepInterval = 100;

	int n = 0;
	while ( isRunning() )
	{
		qApp->processEvents();
		n++;
		Sleeper::msleep( sleepInterval );
		if ( n > 50 )
			break;
	}

	if ( isRunning() )
	{
		tryTerminate();
		n = 0;
		while ( isRunning() )
		{
			n++;
			qApp->processEvents();
			Sleeper::msleep( sleepInterval );
			if ( n > 10 )
				break;
		}
	}

	if ( isRunning() )
	{
		kill();
		n = 0;
		while ( isRunning() )
		{
			n++;
			qApp->processEvents();
			Sleeper::msleep( sleepInterval );
			if ( n > 10 )
				break;
		}
	}
}


void MPlayerProcess::readFromStderr()
{
	QString line;
	while (canReadLineStderr()) {
		line = "stderr: " + readLineStderr();
		parseLine( line );
	}
}

void MPlayerProcess::read()
{
	read( readStdout() );
}

void MPlayerProcess::read( const QByteArray& text )
{
	QString line;
	QString l = m_incompleteLine;
	l += text;
	l = l.replace( 0x0D, '\n' );

	for ( int pos = l.find( '\n' ); pos > -1; pos = l.find( '\n' ) )
	{
		line = l.left( pos );
		parseLine( QString::fromLocal8Bit( line ) );
		l = l.mid( pos + 1 );
	}

	m_incompleteLine = l;
}

static QRegExp rx_av( "^[AV]: *([0-9,:.-]+)" );
static QRegExp rx_frame( "^[AV]:.* (\\d+)\\/.\\d+" );// [0-9,.]+");
static QRegExp rx( "^(.*)=(.*)" );
static QRegExp rx_audio_mat( "^ID_AID_(\\d+)_(LANG|NAME)=(.*)" );
static QRegExp rx_title( "^ID_DVD_TITLE_(\\d+)_(LENGTH|CHAPTERS|ANGLES)=(.*)" );
static QRegExp rx_winresolution( "^VO: \\[(.*)\\] (\\d+)x(\\d+) => (\\d+)x(\\d+)" );
static QRegExp rx_paused( "^  =====  PAUSE  =====" );
static QRegExp rx_paused2( "^ID_PAUSED" );
static QRegExp rx_novideo( "^Video: no video" );
static QRegExp rx_play( "^Starting playback..." );
static QRegExp rx_endoffile( "^Exiting... \\(End of file\\)" );
static QRegExp rx_aspect2( "^Movie-Aspect is ([0-9,.]+):1" );
static QRegExp rx_subs("^ID_SID_(\\d+)_(LANG|NAME)=(.*)"); // Subtitles


void MPlayerProcess::initRegularExpressions()
{
	if ( ! m_configGroup.option( OPT_REGEX_ENDOFFILE ).isEmpty() )
		rx_endoffile.setPattern( m_configGroup.option( OPT_REGEX_ENDOFFILE ) );

	if ( ! m_configGroup.option( OPT_REGEX_NOVIDEO ).isEmpty() )
		rx_novideo.setPattern( m_configGroup.option( OPT_REGEX_NOVIDEO ) );
}

void MPlayerProcess::sendMute( bool pause )
{
	if ( ! isRunning() )
		return;

	bool wasPlaying = ! m_isPaused;

	QxtSignalWaiter wroteToStdinWaiter( this, SIGNAL(wroteToStdin()) );
	QxtSignalWaiter pauseWaiter( this, SIGNAL(receivedPauseToggled()) );

	writeToStdin( "pausing mute\n" );

	wroteToStdinWaiter.wait( 5000 ); // wait till the data is actually written to stdin
	pauseWaiter.wait( 5000 ); // wait till the data is read and processed by mplayer and our parsing signals are sent

	if ( wasPlaying && ! pause )
		sendPause(); // restore the previous (playing) state
}

void MPlayerProcess::sendPause()
{
	if ( ! isRunning() )
		return;

	QxtSignalWaiter wroteToStdinWaiter( this, SIGNAL( wroteToStdin() ) );
	QxtSignalWaiter pauseWaiter( this, SIGNAL( receivedPauseToggled() ) );

	writeToStdin( "pause\n" );

	wroteToStdinWaiter.wait( 5000 ); // wait till the data is actually written to stdin
	pauseWaiter.wait( 5000 ); // wait till the data is read and processed by mplayer and our parsing signals are sent
}

void MPlayerProcess::sendFrameStep()
{
	if ( ! isRunning() )
		return;

	QxtSignalWaiter wroteToStdinWaiter( this, SIGNAL(wroteToStdin()) );
	QxtSignalWaiter pauseWaiter( this, SIGNAL(receivedPauseToggled()) );

	writeToStdin( "pausing frame_step\n" );

	wroteToStdinWaiter.wait( 5000 ); // wait till the data is actually written to stdin
	pauseWaiter.wait( 5000 ); // wait till the data is read and processed by mplayer and our parsing signals are sent
}

void MPlayerProcess::sendSeek( double seconds, bool pause )
{
	if ( ! isRunning() )
		return;

	bool wasPlaying = ! m_isPaused;

	QxtSignalWaiter wroteToStdinWaiter( this, SIGNAL(wroteToStdin()) );
	QxtSignalWaiter pauseWaiter( this, SIGNAL(receivedPauseToggled()) );

	writeToStdin( "pausing seek " + QString::number( seconds ) + " 2\n" );

	wroteToStdinWaiter.wait( 5000 ); // wait till the data is actually written to stdin
	pauseWaiter.wait( 5000 ); // wait till the data is read and processed by mplayer and our parsing signals are sent

	if ( wasPlaying && ! pause ) // restore the previous state
		sendPause();
}

void MPlayerProcess::sendVolume( double volume )
{
	if ( ! isRunning() )
		return;

	QxtSignalWaiter wroteToStdinWaiter( this, SIGNAL(wroteToStdin()) );
	writeToStdin( "volume " + QString::number( volume ) + " 1\n" );
	wroteToStdinWaiter.wait( 5000 ); // wait till the data is actually written to stdin
}

void MPlayerProcess::sendAudioStream( int audioStream )
{
	if ( ! isRunning() )
		return;

	QxtSignalWaiter wroteToStdinWaiter( this, SIGNAL(wroteToStdin()) );
	writeToStdin( "switch_audio " + QString::number( audioStream ) + "\n" );
	wroteToStdinWaiter.wait( 5000 ); // wait till the data is actually written to stdin
}

void MPlayerProcess::sendQuit()
{
	if ( ! isRunning() )
		return;

	QxtSignalWaiter wroteToStdinWaiter( this, SIGNAL(wroteToStdin()) );
	writeToStdin( "quit\n" );
	wroteToStdinWaiter.wait( 5000 ); // wait till the data is actually written to stdin
}


void MPlayerProcess::parseLine( const QString& line )
{
	if ( m_mediaData.videoFPS != 0.0 && rx_frame.search( line ) > -1 )
	{
		// try to parse the position from the reported frame number

		if ( ! m_hasNotifiedRunning )
		{
			emit mediaDataLoaded();
			m_hasNotifiedRunning = true;
		}

		if ( m_isPaused )
		{
			m_isPaused = false;
			emit receivedPauseToggled();
		}

		double position = rx_frame.cap( 1 ).toInt() / m_mediaData.videoFPS;
		if ( rx_av.search( line ) > -1 )
		{
			double aux = rx_av.cap( 1 ).toDouble();
			if ( position - aux > 0.5 || position - aux < -0.5 )
				 // mplayer is reporting badly, use innacurate (but safe) value instead
				position = aux;
		}

		emit receivedCurrentPosition( position );
	}
	else

	if ( rx_av.search( line ) > -1 )
	{
		// parse the reported position in seconds

		if ( ! m_hasNotifiedRunning )
		{
			emit mediaDataLoaded();
			m_hasNotifiedRunning = true;
		}

		if ( m_isPaused )
		{
			m_isPaused = false;
			emit receivedPauseToggled();
		}

		emit receivedCurrentPosition( rx_av.cap( 1 ).toDouble() );
	}

	else
	{
		// Parse other things

		// End of file
		if ( rx_endoffile.search( line ) > -1 )
		{
			emit receivedEndOfFile();
		}
		else

		// Window resolution
		if ( rx_winresolution.search( line ) > -1 )
		{
			int w = rx_winresolution.cap( 4 ).toInt();
			int h = rx_winresolution.cap( 5 ).toInt();
			emit receivedWindowResolution( w, h );
		}
		else

		// No video
		if ( rx_novideo.search( line ) > -1 )
		{
			m_mediaData.hasVideo = false;
		}
		else

		// Pause
		if ( m_mediaData.hasVideo && rx_paused.search( line ) > -1 )
		{
			emit receivedPauseToggled();
			m_isPaused = true;
		}
		else if ( ! m_mediaData.hasVideo && rx_paused2.search( line ) > -1 )
		{
			emit receivedPauseToggled();
			m_isPaused = true;
		}

		// The following things are not sent when the file has started to play
		// (or if sent, smplayer will ignore anyway...)
		// So not process anymore, if video is playing to save some time
		if ( m_hasNotifiedRunning )
			return;

		// Matroska subtitles
		if ( rx_subs.search( line ) > -1 )
		{
			int ID = rx_subs.cap( 1 ).toInt();
			QString lang = rx_subs.cap( 3 );
			QString t = rx_subs.cap( 2 );

			if ( t == "NAME" )
				m_mediaData.subtitleTracks.addName( ID, lang );
			else
				m_mediaData.subtitleTracks.addLang( ID, lang );
		}
		else

		// Matroska audio
		if ( rx_audio_mat.search( line ) > -1 )
		{
			int ID = rx_audio_mat.cap( 1 ).toInt();
			QString lang = rx_audio_mat.cap( 3 );
			QString t = rx_audio_mat.cap( 2 );

			if ( t == "NAME" )
				m_mediaData.audioTracks.addName( ID, lang );
			else
				m_mediaData.audioTracks.addLang( ID, lang );
		}
		else

		// Aspect ratio for old versions of mplayer
		if ( rx_aspect2.search( line ) > -1 )
			m_mediaData.videoAspect = rx_aspect2.cap( 1 ).toDouble();
		else

		//Generic things
		if ( rx.search( line ) > -1 )
		{
			QString tag = rx.cap( 1 );
			QString value = rx.cap( 2 );

			if ( tag == "ID_AUDIO_ID" ) // Generic audio
				m_mediaData.audioTracks.addID( value.toInt() );
			else if ( tag == "ID_SUBTITLE_ID" ) // Generic subtitle
				m_mediaData.subtitleTracks.addID( value.toInt() );
			else if ( tag == "ID_LENGTH" )
				m_mediaData.duration = value.toDouble();
			else if ( tag == "ID_VIDEO_WIDTH" )
				m_mediaData.videoWidth = value.toInt();
			else if ( tag == "ID_VIDEO_HEIGHT" )
				m_mediaData.videoHeight = value.toInt();
			else if ( tag == "ID_VIDEO_ASPECT" )
			{
				m_mediaData.videoAspect = value.toDouble();
				if ( m_mediaData.videoAspect == 0.0 ) // I hope width & height are already set.
					m_mediaData.videoAspect = (double) m_mediaData.videoWidth / m_mediaData.videoHeight;
			}
			else if ( tag == "ID_VIDEO_FPS" )
			{
				bool ok;
				m_mediaData.videoFPS = value.toDouble( &ok );
				if ( ! ok )
					m_mediaData.videoFPS = 0.0;
			}
		}
	}

}
