/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "player.h"
#include "videowidget.h"

#include <qfileinfo.h>
#include <math.h>

using namespace SubtitleComposer;

#define DEFAULT_MIN_POSITION_DELTA 0.02

Player::Player( const AppConfigGroup& configGroup, const char* name ):
	QObject( 0, name ),
	m_initialized( false ),
	m_filePath(),
	m_state( Ready ),
	m_position( -1.0 ),
	m_length( -1.0 ),
	m_framesPerSecond( -1.0 ),
	m_minPositionDelta( DEFAULT_MIN_POSITION_DELTA ),
	m_audioStream( -1 ),
	m_audioStreams(),
	m_muted( false ),
	m_logarithmicVolume( true ),
	m_userVolume( 100.0 ),
	m_volume( 100.0 ),
	m_videoWidget( 0 ),
	m_configGroup( configGroup )
{
}

Player::~Player()
{
}

SubtitleComposer::PlayerVideoWidget* Player::videoWidget()
{
	return m_videoWidget;
}

bool Player::isInitialized()
{
	return m_initialized;
}

bool Player::initialize( QWidget* videoWidgetParent )
{
	if ( m_initialized )
		return true;

	m_videoWidget = _initialize( videoWidgetParent );
	m_initialized = m_videoWidget != 0;

	if ( m_initialized )
	{
		connect( m_videoWidget, SIGNAL(destroyed()), this, SLOT(onVideoWidgetDestroyed()) );

		m_videoWidget->show();
		m_videoWidget->lower(); // send to back so that the subtitles are drawn overlayed on the video

		// FIXME there's got to be a better way to make videoWidgetParent update it's geometry than this...
		QRect geometry = videoWidgetParent->geometry();
		geometry.setHeight( geometry.height() + 1 );
		videoWidgetParent->setGeometry( geometry );

		emit initialized( this );
	}

	return m_initialized;
}

bool Player::finalize()
{
	if ( ! m_initialized )
		return true;

	closeFile();

	m_initialized = ! _finalize();

	m_videoWidget->disconnect();
	m_videoWidget->hide();
	delete m_videoWidget;

	m_videoWidget = 0;

	if ( ! m_initialized )
		emit finalized( this );

	return ! m_initialized;
}

bool Player::reinitialize()
{
	if ( ! m_initialized )
		return false;

	QWidget* videoWidgetParent = videoWidget()->parentWidget();

	QString oldFilePath = m_filePath;
	int oldState = m_state;
	double oldPosition = position();

	if ( ! finalize() )
		return false;

	if ( ! initialize( videoWidgetParent ) )
		return false;

	if ( oldFilePath.isEmpty() )
		return true;

	if ( ! openFile( oldFilePath ) )
		return true;

	if ( oldState == Playing )
	{
		_play();
		_seek( oldPosition );
	}

	return true;
}

void Player::onVideoWidgetDestroyed()
{
	if ( ! m_initialized )
		return;

	closeFile();

	m_initialized = ! _finalize();

	m_videoWidget = 0;

	if ( ! m_initialized )
		emit finalized( this );
}

const AppConfigGroup& Player::configGroup()
{
	return m_configGroup;
}

void Player::setConfigGroup( const AppConfigGroup& newConfigGroup )
{
	if ( m_configGroup.isCompatibleWith( newConfigGroup ) )
	{
		if ( m_configGroup != newConfigGroup )
		{
			m_configGroup = newConfigGroup;
			reinitialize();
		}
	}
}

const QString& Player::filePath()
{
	return m_filePath;
}

bool Player::openFile( const QString& filePath )
{
	if ( ! m_initialized )
		return false;

	if ( ! m_filePath.isEmpty() ) // client should close the file first
		return false;

	QFileInfo fileInfo( filePath );
	if ( ! fileInfo.exists() || ! fileInfo.isFile() || ! fileInfo.isReadable() )
		return false;

	if ( m_filePath == fileInfo.absFilePath() )
		return true;

	if ( _openFile( fileInfo.absFilePath() ) )
	{
		m_filePath = fileInfo.absFilePath();

		m_audioStream = m_audioStreams.count() ? 0 : -1;

		emit opened( m_filePath );
		return true;
	}
	else
		return false;
}

bool Player::closeFile()
{
	if ( ! m_initialized )
		return false;

	if ( ! m_filePath.isEmpty() )
	{
		if ( m_state != Ready )
			_stop();

		if ( ! _closeFile() )
			return false;

		m_filePath = QString::null;

		m_position = -1.0;
		m_length = -1.0;
		m_framesPerSecond = -1.0;
		m_minPositionDelta = DEFAULT_MIN_POSITION_DELTA;

		m_audioStream = -1;
		m_audioStreams.clear();

		emit closed();
	}

	return true;
}

int Player::state()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return -1;

	return m_state;
}

bool Player::isPlaying()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	return m_state == Playing;
}

bool Player::play()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	return m_state == Playing ? true : _play();
}

bool Player::replay()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	int oldState = m_state;
	double oldPosition = position();

	if ( ! _stop() )
		return false;

	if ( oldState == Playing )
	{
		if ( ! _play() )
			return false;
	}

	_seek( oldPosition );

	return true;
}

bool Player::isPaused()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	return m_state == Paused;
}

bool Player::pause()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	return m_state == Paused ? true : _pause();
}

bool Player::togglePlayPaused()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	return m_state == Playing ? _pause() : _play();
}

double Player::position()
{
	return (! m_initialized || m_filePath.isEmpty()) ? -1.0 : m_position;
}

double Player::length()
{
	return (! m_initialized || m_filePath.isEmpty()) ? -1.0 : m_length;
}

double Player::framesPerSecond()
{
	return (! m_initialized || m_filePath.isEmpty()) ? -1.0 : m_framesPerSecond;
}

bool Player::seek( double seconds )
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	if ( seconds < 0 || seconds > m_length )
		return false;

	if ( seconds == m_position )
		return true;

	if ( m_state == Playing || m_state == Paused )
		return _seek( seconds );
	else
		return false;
}

bool Player::isStopped()
{
	return (! m_initialized || m_filePath.isEmpty()) ? false : m_state == Ready;
}

bool Player::stop()
{
	if ( ! m_initialized || m_filePath.isEmpty() )
		return false;

	return m_state == Ready ? true : _stop();
}

double Player::logarithmicVolume( double volume )
{
	static const double base = 4.0;
	static const double power = 1.0;
	static const double divisor = pow( base, power );

	double scaledVol = volume*power*power/100.0;
	double factor = pow( base, scaledVol/power );

	return (scaledVol*factor/divisor)*(100.0/(power*power));
}

bool Player::logarithmicVolume()
{
	return m_logarithmicVolume;
}

void Player::setLogarithmicVolume( bool logarithmic )
{
	if ( m_logarithmicVolume != logarithmic )
	{
		m_logarithmicVolume = logarithmic;

		m_volume = m_muted ? 0 : (m_logarithmicVolume ? logarithmicVolume( m_userVolume ) : m_userVolume);

		if ( m_initialized && ! m_muted && m_state == Playing )
			_setVolume( m_volume );
	}
}

double Player::volume()
{
	return m_userVolume;
}

void Player::setVolume( double volume )
{
	if ( volume < 0.0 )
		volume = 0.0;
	else if ( volume > 100.0 )
		volume = 100.0;

	if ( m_userVolume != volume )
	{
		m_userVolume = volume;

		m_volume = m_muted ? 0 : (m_logarithmicVolume ? logarithmicVolume( m_userVolume ) : m_userVolume);

		if ( m_initialized && ! m_muted && m_state == Playing )
			_setVolume( m_volume );

		emit volumeChanged( m_userVolume );
	}
}

bool Player::isMuted()
{
	return m_muted;
}

void Player::toggleMuted()
{
	setMuted( ! m_muted );
}

void Player::setMuted( bool muted )
{
	if ( m_muted != muted )
	{
		m_muted = muted;

		m_volume = m_muted ? 0 : (m_logarithmicVolume ? logarithmicVolume( m_userVolume ) : m_userVolume);

		if ( m_initialized && m_state == Playing )
			_setVolume( m_volume );

		emit mutedChanged( m_muted );
	}
}

int Player::audioStream()
{
	return (! m_initialized || m_filePath.isEmpty()) ? -1 : m_audioStream;
}

const QStringList& Player::audioStreams()
{
	static QStringList empty;

	return (! m_initialized || m_filePath.isEmpty()) ? empty : m_audioStreams;
}

void Player::setAudioStream( int streamIndex )
{
	if ( ! m_initialized || m_filePath.isEmpty() || m_audioStreams.size() <= 1 )
		return;

	if ( m_audioStream != streamIndex && streamIndex >= 0 && streamIndex < (signed)m_audioStreams.size() )
	{
		m_audioStream = streamIndex;

		if ( m_state != Ready )
		{
			if ( ! _setAudioStream( streamIndex ) )
				replay();
		}

		if ( m_state == Playing )
			_setVolume( m_volume );

		emit audioStreamChanged( streamIndex );
	}
}

void Player::updatePosition( double position )
{
	if ( m_position != position )
	{
		if ( m_position <= 0 || m_minPositionDelta <= 0.0 )
		{
			m_position = position;
			emit positionChanged( position );
		}
		else
		{
			double positionDelta = m_position - position;
			if ( positionDelta >= m_minPositionDelta || -positionDelta >= m_minPositionDelta )
			{
				m_position = position;
				emit positionChanged( position );
			}
		}
	}
}

void Player::updateLength( double length )
{
	if ( m_length != length )
	{
		m_length = length;
		emit lengthChanged( length );
	}
}

void Player::updateFramesPerSecond( double framesPerSecond )
{
	if ( framesPerSecond > 0 && m_framesPerSecond != framesPerSecond )
	{
		m_framesPerSecond = framesPerSecond;
		m_minPositionDelta = 1.0/framesPerSecond;
		emit framesPerSecondChanged( framesPerSecond );
	}
}

void Player::updateAudioStreams( const QStringList& audioStreams )
{
	m_audioStreams = audioStreams;
}

void Player::updateState( int state )
{
	if ( m_state != state )
	{
		m_state = state;

		if ( state == Playing )
		{
			_setVolume( m_volume );
			emit playing();
		}
		else if ( state == Paused )
			emit paused();
		else //if ( state == Ready )
		{
			if ( ! m_filePath.isEmpty() )
				emit stopped();
		}
	}
}
