#ifndef PLAYER_H
#define PLAYER_H

/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
	#include <config.h>
#endif

#include <appconfiggroup.h>
#include <appconfiggroupwidget.h>
#include <videowidget.h>

#include <qobject.h>
#include <qwidget.h>

namespace SubtitleComposer
{
	class Player : public QObject
	{
		Q_OBJECT

		friend class PlayersManager;

		public:

			typedef enum { Playing=0, Paused, Ready /*Stopped/Finished*/ };

			Player( const AppConfigGroup& configGroup, const char* name=0 );
			virtual ~Player();

			PlayerVideoWidget* videoWidget();

			const AppConfigGroup& configGroup();
			void setConfigGroup( const AppConfigGroup& configGroup );

			virtual AppConfigGroupWidget* newAppConfigGroupWidget( QWidget* parent ) = 0;

			const QString& filePath();

			int state();
			bool isPlaying();
			bool isPaused();
			double position();
			double length();
			double framesPerSecond();
			bool isStopped();

			double volume();
			bool logarithmicVolume();
			bool isMuted();
			int audioStream();
			const QStringList& audioStreams();

		public slots:

			bool openFile( const QString& filePath );
			bool closeFile();

			bool play();
			bool pause();
			bool togglePlayPaused();
			bool seek( double seconds );
			bool stop();

			void setVolume( double volume ); /* volume is a percentage from 0.0 to 100.0 (inclusive) */
			void setLogarithmicVolume( bool logarithmic );
			void toggleMuted();
			void setMuted( bool mute );
			void setAudioStream( int streamIndex );

		signals:

			void initialized( Player* player );
			void finalized( Player* player );

			void opened( const QString& filePath );
			void closed();

			void playing();
			void positionChanged( double seconds );
			void lengthChanged( double seconds );
			void framesPerSecondChanged( double fps );
			void paused();
			void stopped();

			void volumeChanged( double volume );
			void mutedChanged( bool muted );
			void audioStreamChanged( int audioStream );

		protected:

			bool isInitialized();
			// creates the videoWidget and performs required initialization
			virtual PlayerVideoWidget* _initialize( QWidget* videoWidgetParent ) = 0;
			// destroy the videoWidget and performs required deinitialization
			// videoWidgetDestroyed is true when the videoWidget has been destroyed prior to
			// the player finalization (and care must be taken in order not to delete it twice)
			virtual bool _finalize() = 0;

			virtual bool _openFile( const QString& filePath ) = 0; // must set the state!
			virtual bool _closeFile() = 0;

			virtual bool _play() = 0;
			virtual bool _pause() = 0;
			virtual bool _seek( double seconds ) = 0;
			virtual bool _stop() = 0;

			virtual bool _setAudioStream( int audioStream ) = 0;
			virtual bool _setVolume( double volume ) = 0;

			double logarithmicVolume( double percentage );

			void updatePosition( double position );
			void updateLength( double length );
			void updateFramesPerSecond( double framesPerSecond );
			void updateAudioStreams( const QStringList& audioStreams );
			void updateState( int state );

		private:

			bool initialize( QWidget* videoWidgetParent );
			bool finalize();
			bool reinitialize();

			bool replay();

		private slots:

			void onVideoWidgetDestroyed();

		private:

			bool m_initialized;

			QString m_filePath;

			int m_state;
			double m_position;
			double m_length;
			double m_framesPerSecond;
			double m_minPositionDelta;
			int m_audioStream;
			QStringList m_audioStreams;

			bool m_muted;
			bool m_logarithmicVolume;
			double m_userVolume;
			double m_volume;

			PlayerVideoWidget* m_videoWidget;

		protected:

			AppConfigGroup m_configGroup;
	};
}

#endif
