/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "playersmanager.h"

#include "mplayer/mplayer.h"
#ifdef HAVE_GSTREAMER
#include "gstreamer/gstplayer.h"
#endif

#include <kdebug.h>

using namespace SubtitleComposer;

bool PlayersManager::s_loaded( false );
QMap<QString,Player*> PlayersManager::s_playersMap;

bool PlayersManager::s_initialized( false );
QString PlayersManager::s_activePlayerName;

void PlayersManager::checkLoaded()
{
	if ( s_loaded )
		return;

	s_loaded = true;

	static QPtrList<Player> playersList;
	playersList.setAutoDelete( true ); // players will be deleted on program exit

#ifdef HAVE_GSTREAMER
	s_playersMap["GStreamer"] = new GstPlayer();
	playersList.append( s_playersMap["GStreamer"] );
#endif
	s_playersMap["MPlayer"] = new MPlayer();
	playersList.append( s_playersMap["MPlayer"] );
}

void PlayersManager::init( QWidget* videoWidgetParent, const QString& activePlayerName )
{
	checkLoaded();

	if ( s_initialized )
	{
		kdError() << "Players have already been initialized" << endl;
		return;
	}

	s_initialized = true;

	Player* activePlayer = setActivePlayer( activePlayerName, videoWidgetParent ); // try to set requested player as active
	if ( ! activePlayer ) // set the first available player as active
	{
		for ( QMapConstIterator<QString,Player*> it = s_playersMap.begin(); it != s_playersMap.end(); ++it )
			if ( (activePlayer = setActivePlayer( it.key(), videoWidgetParent )) != 0 )
				break;
	}

	if ( ! activePlayer )
		kdError() << "Couldn't initialize any player" << endl;
}

void PlayersManager::deinit()
{
	checkLoaded();

	if ( ! s_initialized )
		return;

	for ( QMapConstIterator<QString,Player*> it = s_playersMap.begin(); it != s_playersMap.end(); ++it )
		it.data()->finalize();

	s_initialized = false;
}

QStringList PlayersManager::playerNames()
{
	checkLoaded();

	return s_playersMap.keys();
}

bool PlayersManager::hasPlayer( const QString& playerName )
{
	checkLoaded();

	return s_playersMap.contains( playerName );
}

Player* PlayersManager::player( const QString& playerName )
{
	checkLoaded();

	return s_playersMap.contains( playerName ) ? s_playersMap[playerName] : 0;
}

Player* PlayersManager::activePlayer()
{
	if ( ! s_initialized )
		return 0;

	return s_playersMap.contains( s_activePlayerName ) ? s_playersMap[s_activePlayerName] : 0;
}

Player* PlayersManager::setActivePlayer( const QString& playerName, QWidget* videoWidgetParent )
{
	if ( ! s_initialized )
		return 0;

	Player* activePlayer = PlayersManager::activePlayer();

	if ( ! s_playersMap.contains( playerName ) )
	{
		kdDebug() << "Attempted to set unknown player as active player: " << playerName << endl;
		return activePlayer;
	}

	if ( activePlayer )
	{
		if ( ! activePlayer->finalize() )
		{
			kdDebug() << "Error finalizing " << s_activePlayerName << " player" << endl;
			return activePlayer;
		}
	}

	if ( ! s_playersMap[playerName]->initialize( videoWidgetParent ) )
	{
		kdDebug() << "Error initializing " << playerName << " player" << endl;
		return activePlayer;
	}

	s_activePlayerName = playerName;

	kdDebug() << "Setting " << playerName << " as active player" << endl;

	return s_playersMap[playerName];
}
